
C-----------------------------------------------------------------------------C
C									      C
C		SUBROUTINE	read_atoms_es				      C
C									      C
C-----------------------------------------------------------------------------C

	SUBROUTINE	read_atoms_es
C	**********	*************

C Read in VdW radii, polar H-atom generation information, and charge sets,
C from the "atoms_es.dat" file.

C at_es_head	-	"atom_es.dat" header record

C Arrays storing the n_at_types basic atom types.

C at_info	-	information string for atom type
C at_VdW	-	up to 10 different VdW radii sets for the atom types
C at_ionise	-	whether atomic charge is part if ionisable group
C at_pK		-	pK of ionisable (adjustable) charge
C at_anyq	-	whether atom carries charge in any charge set
C at_prtner	-	A4 atyp of a partner atom in an ionisable group

C n_res_types	- total number of residue types to read in
C n_chrg	- number of charge sets to read in (columns for each atom)
C n_all_at	- total number of atom/residue combos that we read in

C Arrays storing residue information.

C res_type (ir)		- A4 residue identifier
C n_res_ats (ir)	- number of atoms in this residue
C nat_pter (ir,n1/n2)	- pointers to limits in at arrays for residue ir

C Arrays storing the atom information.

C at_type (iat)		- A4 atom type identifier
C at_class (iat)	- integer atom type
C at_chrg (iat,n)	- the atom charge sets (1 to n)
C at_hgen (ia)		- flag for hydrogen atom generation from this atom
C at_hgen_pter (ia)	- slot in the hgen arrays for hgen information
C at_hgen_root (ia)	- atom type of root atom for a hydrogen

C Arrays storing hydrogen atom generation information (up to 300 slots).
C These slots are accessed for the heavy atom which bonds the hydrogens if
C the at_hgen (i) flag is set, with at_hgen_pter (i) giving the slot, x.

C hgen_int  (x,1)	- code for hydrogen generation
C	    (x,2-4)	- relative residue nos for atoms i,j,k
C	    (x,5)	- number of hydrogen atoms to be generated
C hgen_char (x,1-3)	- A4 atyps for atoms i,j,k used in generation
C hgen_real (x,1/2)	- bond distance, bond angle

	INTEGER		hgen_code
	CHARACTER*9	section_head
	CHARACTER*80	spacer, at_rec
	LOGICAL*1	hgen_curr, last_set

	CHARACTER*4	res_type (200), at_type (2000),
     1			hgen_char (300,3), at_hgen_root (2000),
     2			common_atyp (100), at_prtner (2000),
     3			atyp, atyp_root
	REAL		at_VdW (100,10), at_chrg (2000,4),
     1			hgen_real (300,2), at_pK (2000)
	INTEGER		n_res_ats (200), nat_pter (200,2),
     1			at_class (2000), at_hgen_pter (2000),
     2			hgen_int (300,5), common_pter (100)
	CHARACTER*34	at_info (100)
	CHARACTER*80	at_es_head
	CHARACTER*60	data_path
	LOGICAL*1	at_hgen (2000), at_anyq (2000),
     1			at_ionise (2000)

	COMMON	/atoms_es_info/ at_es_head
	COMMON	/atoms_es_type/ at_info, n_at_types, at_VdW, n_VdW
	COMMON	/atoms_es_resi/	n_res_types, n_chrg, n_all_at,
     1				res_type, n_res_ats, nat_pter,
     2				at_type, at_class, at_chrg,
     3				at_anyq, at_ionise, at_pK,
     4				at_prtner
	COMMON	/atoms_es_hgen/ at_hgen, at_hgen_pter, at_hgen_root,
     1				hgen_int, hgen_real, hgen_char
	COMMON	/common_atoms/	common_atyp, common_pter, n_common

	CALL	getenv ('DATA', data_path)
	ipd		= 1
	DO WHILE (data_path (ipd:ipd) .NE. ' ')
	  ipd		= ipd + 1
	END DO
Clnx	OPEN (UNIT=3,NAME=data_path(1:ipd-1)//'atoms_es.dat',
Clnx     1	      TYPE='OLD', READONLY)
	OPEN (UNIT=3,NAME=data_path(1:ipd-1)//'atoms_es.dat',
     1	      TYPE='OLD')

1020	FORMAT (A80)
1030	FORMAT (A9)
1040	FORMAT (1X,I2,A34,3(2X,F4.2))
1050	FORMAT (A4,1X,I2,1X,I2,1X,I2)
1060	FORMAT (A4)
1070	FORMAT (4X,I1,3(1X,A4),1X,3I2,1X,F5.3,1X,F5.1)
1080	FORMAT (A4,1X,I2,2X,L1)
C ?? 1085	FORMAT (L1,1X,F5.2,1X,A4)
1085	FORMAT (L1,1X,F5.2,1X,A4,4(2X,F6.3))

9000	FORMAT ('0','*** ERROR *** atoms_es * read error')
9100	FORMAT ('0','*** ERROR *** atoms_es * record not in sequence')
9200	FORMAT ('0','*** ERROR *** atoms_es * i_type / i mis-match')
9300	FORMAT ('0','*** ERROR *** atoms_es * too many types or VdW')
9400	FORMAT ('0','*** ERROR *** atoms_es * n_VdW must equal format
     1 specifier 1040')
9500	FORMAT ('0','*** ERROR *** atoms_es * n_chrg must equal format
     1 specifier 1080')
9600	FORMAT ('0','*** ERROR *** atoms_es * too many atoms in file')
9700	FORMAT ('0','*** ERROR *** atoms_es * problem w/ hgen record
     1 or H record')
9800	FORMAT ('0','*** ERROR *** atoms_es * ir/i_this_res mismatch')
9850	FORMAT ('0','*** ERROR *** atoms_es * n_common .gt. 100')

C Read the header record, read past spacers.

	READ (3,1020,ERR=900) spacer
	READ (3,1020,ERR=900) at_es_head
	READ (3,1020,ERR=900) spacer

C Read in section 1 data - this is the atom types with various VdW radii sets.

	READ (3,1030,ERR=900) section_head
	IF (section_head .NE. 'SECTION 1') GOTO 910
	READ (3,1020,ERR=900) spacer
	READ (3,*,ERR=900) n_at_types, n_VdW
	READ (3,1020,ERR=900) spacer
	IF (n_at_types .GT. 100 .OR. n_VdW .GT. 10) GOTO 930
	IF (n_VdW .NE. 3) GOTO 940
	READ (3,1020,ERR=900) spacer
	DO 200 i	= 1, n_at_types
	  READ (3,1040,ERR=900) i_type, at_info (i),
     1				(at_VdW (i,m), m = 1, n_VdW)
	  IF (i_type .NE. i) GOTO 920
200	CONTINUE						! End i

C Read in section 2 data - residue by residue listing of heavy atoms plus
C polar hydrogens.

	READ (3,1020,ERR=900) spacer
	READ (3,1030,ERR=900) section_head
	IF (section_head .NE. 'SECTION 2') GOTO 910
	READ (3,1020,ERR=900) spacer
	READ (3,*,ERR=900) n_res_types, n_chrg
	IF (n_res_types .GT. 200) GOTO 930
C ??	IF (n_chrg .NE. 0) GOTO 950
	IF (n_chrg .NE. 4) GOTO 950
	n_all_at		= 0
	n_all_hgen		= 0
	n_common		= 0
	DO 600 ir		= 1, n_res_types
	  READ (3,1020,ERR=900) spacer
	  READ (3,1050,ERR=900) res_type (ir), i_this_res,
     1				n_res_ats (ir), n_hgen
	  IF (i_this_res .NE. ir) GOTO 980
	  IF (n_res_ats (ir) .NE. 0) THEN
	    nat_pter (ir,1)	= n_all_at + 1
	    nat_pter (ir,2)	= n_all_at + n_res_ats (ir)
	    n_all_at		= nat_pter (ir,2)
	    IF (n_all_at .GT. 2000) GOTO 960
	    iat_read		= 0
	    last_set		= .FALSE.
	    i_h_next		= 0
	    hgen_curr		= .FALSE.
	    DO 500 ia		= 1, (n_res_ats (ir) + n_hgen)
	      READ (3,1020,ERR=900) at_rec
	      READ (at_rec (1:4),1060,ERR=900) atyp
	      IF (atyp .EQ. 'hgen') THEN
		IF (last_set) THEN
		  at_hgen (last_pter)	= .TRUE.
		  last_set		= .FALSE.
		ELSE
		  GOTO 970
		END IF
		n_all_hgen	= n_all_hgen + 1
		IF (n_all_hgen .GT. 300) GOTO 970
		at_hgen_pter (last_pter) = n_all_hgen
		l		= n_all_hgen
		READ (at_rec (5:43),1070,ERR=900) hgen_int (l,1),
     1					(hgen_char (l,m),m=1,3),
     2					(hgen_int  (l,m),m=2,4),
     3					(hgen_real (l,m),m=1,2)
		hgen_curr	= .TRUE.
		hgen_code	= hgen_int (l,1)
		atyp_root	= at_type (last_pter)
		IF	(hgen_code .EQ. 1) THEN
		  n_h_next	= 1
		ELSE IF (hgen_code .EQ. 2) THEN
		  n_h_next	= 1
		ELSE IF (hgen_code .EQ. 3) THEN
		  n_h_next	= 2
		ELSE IF (hgen_code .EQ. 4) THEN
		  n_h_next	= 3
		ELSE IF (hgen_code .EQ. 5) THEN
		  n_h_next	= 2
		ELSE
		  GOTO 970
		END IF
		hgen_int (l,5)	= n_h_next
	      ELSE
		iat_read	= iat_read + 1
		ipter		= nat_pter (ir,1) + iat_read - 1
		at_hgen (ipter)	= .FALSE.
		READ (at_rec (1:10),1080,ERR=900) at_type (ipter),
     1		     at_class (ipter), at_anyq (ipter)
		IF (at_anyq (ipter)) THEN
		  READ (at_rec (13:56),1085,ERR=900)
     1				at_ionise (ipter),
     2				at_pK (ipter),
     3				at_prtner (ipter),
     4				(at_chrg (ipter,m),m=1,n_chrg)
		END IF
C ?? herejim - we could mark the prtner out in iat now, if we want
		last_pter	= ipter
		last_set	= .TRUE.
		IF (res_type (ir) .EQ. 'ANY ') THEN
		  n_common	= n_common + 1
		  IF (n_common .GT. 100) GOTO 985
		  common_atyp (n_common)	= at_type (ipter)
		  common_pter (n_common)	= ipter
		END IF
		IF (hgen_curr) THEN
		  i_h_next	= i_h_next + 1
		  IF (i_h_next .EQ. n_h_next) THEN
		    hgen_curr	= .FALSE.
		    i_h_next	= 0
		  END IF
		  IF (at_rec (1:1) .NE. 'H') GOTO 970
		  at_hgen_root (ipter)	= atyp_root
		END IF
	      END IF
500	    CONTINUE						! End ia
	  END IF
600	CONTINUE						! End ir
	READ (3,1020,ERR=900) spacer
	CLOSE (UNIT=3)

	RETURN

900	WRITE (6,9000)
	CALL	EXIT

910	WRITE (6,9100)
	CALL	EXIT

920	WRITE (6,9200)
	CALL	EXIT

930	WRITE (6,9300)
	CALL	EXIT

940	WRITE (6,9400)
	CALL	EXIT

950	WRITE (6,9500)
	CALL	EXIT

960	WRITE (6,9600)
	CALL	EXIT

970	WRITE (6,9700)
	CALL	EXIT

980	WRITE (6,9800)
	CALL	EXIT

985	WRITE (6,9850)
	CALL	EXIT
	END


C-----------------------------------------------------------------------------C
C									      C
C		SUBROUTINE	read_residue				      C
C									      C
C-----------------------------------------------------------------------------C

	SUBROUTINE	read_residue (end_of_file)
C	**********	************

C Read a residue into the residue buffer arrays.

	LOGICAL		end_of_file
	
	CHARACTER*4	buf_atyp (100,2), buf_res (100,2),
     1			buf_atom (100,2), res_lastrec, res
	REAL		buf_xyz (100,3,2)
	INTEGER		buf_nres (100,2), n_buf (2),
     1			buf_model (100,2)
	CHARACTER*1	buf_chain (100,2), chain_lastrec,
     1			chain
	CHARACTER*60	keep_model
	CHARACTER*80	atom_record
	LOGICAL		first_res, first_atom, keep_mdl

	COMMON	/coord_buf/	buf_atyp, buf_res, buf_xyz,
     1				buf_atom, buf_nres, first_res,
     2				atom_record, n_buf, first_atom,
     3				buf_chain
	COMMON	/mdl_numbers/	buf_model, keep_mdl

	DATA	 first_res, first_atom	/ .TRUE., .TRUE. /

1040	FORMAT (A80)
1060	FORMAT (22X,I4)
1065	FORMAT (21X,A1)
1070	FORMAT (17X,A4)

9000	FORMAT ('0','*** ERROR * read_residue * non-atom/heta rec')
9100	FORMAT ('0','*** ERROR * read_residue * nothing read in')
9200	FORMAT ('0','*** ERROR * read_residue * res mismtch ',A1,I4,A4)

	CALL	GETENV ('KEEP_MODEL', keep_model)
	CALL	leadin (keep_model, ikm)
	IF	(keep_model (1:1) .EQ. 'T') THEN
	  keep_mdl	= .TRUE.
	ELSE
	  keep_mdl	= .FALSE.
	END IF

	IF (first_res) THEN
	  is			= 0
	ELSE
	  DO 200 i		= 1, n_buf (2)
	    buf_atyp (i,1)	= buf_atyp (i,2)
	    IF (keep_mdl) buf_model (i,1) = buf_model (i,2)
	    buf_res (i,1)	= buf_res (i,2)
	    buf_atom (i,1)	= buf_atom (i,2)
	    buf_nres (i,1)	= buf_nres (i,2)
	    buf_chain (i,1)	= buf_chain (i,2)
	    DO 160 n		= 1, 3
	      buf_xyz (i,n,1)	= buf_xyz (i,n,2)
160	    CONTINUE						! End n
200	  CONTINUE						! End i
	  n_buf (1)		= n_buf (2)
	  CALL		read_atom (1, 2)
	  nres_lastrec		= buf_nres (1,2)
	  chain_lastrec		= buf_chain (1,2)
	  res_lastrec		= buf_res (1,2)
	  is			= 1
	END IF

300	READ (1,1040,END=380) atom_record
	IF ((atom_record (1:4) .NE. 'ATOM') .AND.
     1	    (atom_record (1:4) .NE. 'HETA')) GOTO 900
	READ (atom_record,1060) nres
	READ (atom_record,1065) chain
	READ (atom_record,1070) res
	IF (.NOT.first_atom) THEN
C Nov 02	  IF (nres .NE. nres_lastrec) THEN
	  IF ((nres .NE. nres_lastrec) .OR.
     1        (chain.NE.chain_lastrec)) THEN
	    GOTO 400
	  ELSE
	    IF (res .NE. res_lastrec) GOTO 920
	  END IF
	END IF
	first_atom	= .FALSE.
	nres_lastrec	= nres
	chain_lastrec	=chain
	res_lastrec	= res
	is		= is + 1
	CALL	read_atom (is, 2)
	GOTO 300

380	end_of_file	= .TRUE.
400	n_buf (2)	= is
	IF (n_buf (2) .EQ. 0) GOTO 910
	RETURN

900	WRITE (6,9000)
	CALL	EXIT

910	WRITE (6,9100)
	CALL	EXIT

920	WRITE (6,9200) chain, nres, res
	CALL	EXIT
	END


C-----------------------------------------------------------------------------C
C									      C
C		SUBROUTINE	read_atom				      C
C									      C
C-----------------------------------------------------------------------------C

	SUBROUTINE	read_atom (is, ib)
C	**********	*********

C Read atom_record data into residue buffer ib, slot is.

	CHARACTER*4	buf_atyp (100,2), buf_res (100,2),
     1			buf_atom (100,2)
	REAL		buf_xyz (100,3,2)
	INTEGER		buf_nres (100,2), n_buf (2),
     1			buf_model (100,2)
	CHARACTER*1	buf_chain (100,2)
	CHARACTER*80	atom_record
	LOGICAL		first_res, first_atom, keep_mdl

	COMMON	/coord_buf/	buf_atyp, buf_res, buf_xyz,
     1				buf_atom, buf_nres, first_res,
     2				atom_record, n_buf, first_atom,
     3				buf_chain
	COMMON	/mdl_numbers/	buf_model, keep_mdl

1020	FORMAT (A4, 9X,         A4, A4, A1, I4, 4X, 3F8.3)
1040	FORMAT (A4, 2X, I5, 2X, A4, A4, A1, I4, 4X, 3F8.3)

	IF (keep_mdl) THEN
	 READ (atom_record,1040) buf_atom (is,ib),
     1				 buf_model (is,ib),
     2						   buf_atyp (is,ib),
     3	       buf_res (is,ib), buf_chain (is,ib), buf_nres (is,ib),
     4				(buf_xyz (is,n,ib), n=1,3)
	ELSE
	 READ (atom_record,1020) buf_atom (is,ib), buf_atyp (is,ib),
     1	       buf_res (is,ib), buf_chain (is,ib), buf_nres (is,ib),
     2				(buf_xyz (is,n,ib), n=1,3)
	END IF
	RETURN

	END


C-----------------------------------------------------------------------------C
C									      C
C		SUBROUTINE	res_id					      C
C									      C
C-----------------------------------------------------------------------------C

	SUBROUTINE	res_id (res, irt, ia_1, ia_2, na)
C	**********	******

C Return the index (irt) of residue (A4 type res), and the atom index limits.

	CHARACTER*4	res_type (200), at_type (2000),
     1			at_prtner (2000), res
	REAL		at_chrg(2000,4), at_pK (2000)
	INTEGER		n_res_ats (200), nat_pter (200,2),
     1			at_class (2000)
	LOGICAL*1	at_anyq (2000), at_ionise (2000)

	COMMON	/atoms_es_resi/	n_res_types, n_chrg, n_all_at,
     1				res_type, n_res_ats, nat_pter,
     2				at_type, at_class, at_chrg,
     3				at_anyq, at_ionise, at_pK,
     4				at_prtner

9000	FORMAT ('0','*** ERROR * res_id * residue type ',A4,' not
     1 found')

	DO 300 i	= 1, n_res_types
	  IF (res .EQ. res_type (i)) THEN
	    irt		= i
	    ia_1	= nat_pter (i,1)
	    ia_2	= nat_pter (i,2)
	    na		= n_res_ats (i)
	    RETURN
	  END IF
300	CONTINUE						! End i
	WRITE (6,9000) res
	CALL	EXIT
	END


C-----------------------------------------------------------------------------C
C									      C
C		SUBROUTINE	atom_id					      C
C									      C
C-----------------------------------------------------------------------------C

	SUBROUTINE	atom_id (atyp, iat, ia_1,ia_2)
C	**********	*******

C Return the atom index iat for this atom (atyp) between ia_1, ia_2.

	CHARACTER*4	res_type (200), at_type (2000),
     1			at_prtner (2000), atyp
	REAL		at_VdW (100,10), at_chrg (2000,4),
     1			at_pK (2000)
	INTEGER		n_res_ats (200), nat_pter (200,2),
     1			at_class (2000)
	CHARACTER*34	at_info (100)
	LOGICAL*1	at_anyq (2000), at_ionise (2000)
	CHARACTER*60	unfound_bypass
	LOGICAL		bypass

	COMMON	/atoms_es_type/ at_info, n_at_types, at_VdW, n_VdW
	COMMON	/atoms_es_resi/	n_res_types, n_chrg, n_all_at,
     1				res_type, n_res_ats, nat_pter,
     2				at_type, at_class, at_chrg,
     3				at_anyq, at_ionise, at_pK,
     4				at_prtner

9000	FORMAT ('0','*** ERROR *** atom_id * atom type ',A4,' not
     1 found')

	CALL	GETENV ('UNFOUND_BYPASS', unfound_bypass)
	CALL	leadin (unfound_bypass, iub)
	IF	((unfound_bypass(1:1) .EQ. 'T').OR.
     1		 (unfound_bypass(1:3) .EQ. 'yes')) THEN
	  bypass	= .TRUE.
	ELSE
	  bypass	= .FALSE.
	END IF

	DO 300 i	= ia_1, ia_2
	  IF (atyp .EQ. at_type (i)) THEN
	    iat		= i
	    RETURN
	  END IF
300	CONTINUE						! End i

	IF (bypass) THEN
	  IF (atyp (1:1) .EQ. 'C') THEN
	    iat		= 15
	    RETURN
	  END IF
	  IF (atyp (1:1) .EQ. 'O') THEN
	    iat		= 2
	    RETURN
	  END IF
	  IF (atyp (1:1) .EQ. 'N') THEN
	    iat		= 5
	    RETURN
	  END IF
	  IF (atyp (1:1) .EQ. 'S') THEN
	    iat		= 22
	    RETURN
	  END IF
	  IF (atyp (1:1) .EQ. 'P') THEN
	    iat		= 27
	    RETURN
	  END IF
	END IF

	WRITE (6,9000) atyp
	CALL	EXIT
	END


	SUBROUTINE	leadin (char_array, itop)
C	**********	******

	CHARACTER*60	char_array

	itop	= 1
	DO WHILE (char_array (itop:itop) .NE. ' ')
	  itop	= itop + 1
	END DO
	itop	= itop - 1

	RETURN
	END

