
C	Program		P O L H _ M O V E

C Move polar hydrogens according to simple QQ estimate:

C	Currently only for OHs of S,T,Y and uses MF estimate to generate
C	an average torsions angle, for interactions with a given charge set.

C Output is over-writing of the polay hyd coord file only - a new charge
C set will need to be generated from an additional 'esH' escalc run.

C CURRENTLY we just take the most favourable (O)-H torsions relative to
C 	    the set of bg charges - but we have the facility for iterating
C	    with cyles of (O)H placement and QQ calculation - we already
C           have the other charge ints for the parts of OH gps.
C WHEN/IF   we implement the iteration, we anticipate a degree of
C	    oscillation to overcome.

C Jim Warwicker, Manchester, Feb 2003
C	April 2003 - add a facility to demand that any new H position does
C	not get closer than a certain amount to a charged atom (eg 2.0 A).

	CHARACTER*60	molecule_coords, ph, hbond_lim, ionise_all,
     1			oh_remove
	LOGICAL		all_ionise, remove_oh

	COMMON	/env/		molecule_coords, imc, ph, iph,
     1				ph_here, hbond_lim, ihb, hbond,
     2				all_ionise, remove_oh

	CALL	GETENV ('MOLECULE_COORDS', molecule_coords)
	CALL	leader (molecule_coords, imc)

	CALL	GETENV ('PH', ph)
	CALL	leader (ph, iph)
	READ (ph(1:iph),*) ph_here

	CALL	GETENV ('IONISE_ALL', ionise_all)
	CALL	leader (ionise_all, iia)
	IF ((ionise_all (1:iia).EQ.'yes') .OR.
     1	    (ionise_all (1:iia).EQ.'T')) THEN
	  all_ionise	= .TRUE.
	ELSE
	  all_ionise	= .FALSE.
	END IF

	CALL	GETENV ('OH_REMOVE', oh_remove)
	CALL	leader (oh_remove, ior)
	IF ((oh_remove (1:ior).EQ.'yes') .OR.
     1	    (oh_remove (1:ior).EQ.'T')) THEN
	  remove_oh	= .TRUE.
	ELSE
	  remove_oh	= .FALSE.
	END IF

	CALL	GETENV ('HBOND_LIM', hbond_lim)
	CALL	leader (hbond_lim, ihb)
	IF (ihb .NE. 0) THEN
	  READ (hbond_lim(1:ihb),*) hbond
	ELSE
	  hbond		= 3.0
	END IF

C Read and store the es calcn file that contains polar hyds - flag
C those records that contain torsionable polar hyds.

	CALL	read_polh_coords

C Next we generate all polar hyd torsions - so that we have a complete
C set of xyz at which to calculate Q interactions before we get the Qs.

	CALL	hyds_torsion

C Read the charges, and for each of the possible polar hyd xyz, we:

C A.	Calculate Q intn summed from bg Qs (excl other movers+their roots).
C B.	Calculate Q intn summed over the roots of other overs.
C C.	Leave a slot for Q intn from other movers.

C i.e. At no point do we add in Q intns for a mover with its own root.
C i.e. we will start with A to calc most probable torsions, and then have
C      option of iterating with A+B+C calc.

	CALL	read_polh_charges

C Get the best (mean-field) OH-hyd positions.

	CALL	calc_best_hyds

C Re-write the 'esH' coordinates with the new polar hyd (O)H locations.

	CALL	write_best

	CALL	EXIT
	END


	SUBROUTINE	read_polh_coords
C	**********	****************

C Get the es PDB coords - assuming a pdb_es processed 'esH' file with a
C single header record: flag torsionable polar hydrogens.

	CHARACTER*80	line, pdb_store (100000)
	CHARACTER*60	molecule_coords, ph, hbond_lim
	CHARACTER*4	atom, res, atyp, atyph, atomj,
     1			res_root, atyp1_root, atyp2_root, atyp3_root
	CHARACTER*1	chain, chain_root
	REAL		xyz(3), xyzh(3), xyz1(3), xyz2(3), xyz3(3)
	LOGICAL		move_here, root1_found, root2_found,
     1			root3_found, all_ionise, remove_oh
	REAL		move_pdb_xyz (20000,5,3),
     1			move_pdb_rot (20000,30,6), move_pdb_Q (20000)
	INTEGER		move_pdb_nres (20000), move_pdb_ind (20000)
	CHARACTER*4	move_pdb_res (20000), move_pdb_atyp (20000,4)
	CHARACTER*1	move_pdb_chain (20000)
	LOGICAL		move_clash (20000,30)

	COMMON	/env/		molecule_coords, imc, ph, iph,
     1				ph_here, hbond_lim, ihb, hbond,
     2				all_ionise, remove_oh
	COMMON	/pdb/		pdb_store, n_pdb_store
	COMMON	/move_pdb/	n_move_pdb, move_pdb_xyz,
     1				move_pdb_res, move_pdb_nres,
     2				move_pdb_chain, move_pdb_ind,
     3				move_pdb_atyp, move_pdb_rot,
     4				move_pdb_Q, move_clash

1100	FORMAT (A80)
1150	FORMAT (A4)
1200	FORMAT (13X, A4, A4, A1, I4)
1300	FORMAT (13X, A4, A4, A1, I4, 4X, 3F8.3)
1500	FORMAT (' Root atoms for ',A1,1X,I4,1X,A4,' UNfound: SKIPPED')
1600	FORMAT (/,'Closest h-bond applied from i/p at ',F6.2,/)
1640	FORMAT (/,'Closest h-bond set at default of   ',F6.2,/)
1700	FORMAT (/,'*** All ionisable gps at -1/+1 for Hmove calc ***')
1740	FORMAT (/,'*** Ionisable gps at pH',F5.1,' in Hmove calc ***')

C Use an information file for reporting.

	OPEN (UNIT=4, NAME='polh_move_'//molecule_coords(1:imc)//
     1'.inf', TYPE='NEW')
	WRITE (4,*) ' ----- polh_move information file -----'
	IF (ihb .NE. 0) THEN
	  WRITE (4,1600) hbond
	ELSE
	  WRITE (4,1640) hbond
	END IF

	IF (all_ionise) THEN
	  WRITE (4,1700)
	ELSE
	  WRITE (4,1740) ph_here
	END IF

	OPEN (UNIT=1, NAME=molecule_coords(1:imc)//'_esH.pdb',
     1	      TYPE='OLD')
	nrec			= 0
200	nrec			= nrec + 1
	IF (nrec .GT. 100000) STOP 'nrec > 100000'
	READ (1,1100,END=300) pdb_store (nrec)
	GOTO 200

300	n_pdb_store		= nrec - 1
	CLOSE (UNIT=1)
	n_move_pdb		= 0
	DO 500 nrec		= 1, n_pdb_store
	  READ (pdb_store (nrec),1150) atom
	  IF ((atom.EQ.'ATOM').OR.(atom.EQ.'HETA')) THEN
	    READ (pdb_store (nrec),1300) atyp,res,chain,nres,xyz
	    move_here		= .FALSE.
	    IF     ((res.EQ.'SER ').AND.(atyp.EQ.'HG  ')) THEN
	      res_root		= 'SER '
	      atyph		= 'HG  '
	      atyp1_root	= 'OG  '
	      atyp2_root	= 'CB  '
	      atyp3_root	= 'CA  '
	      move_here		= .TRUE.
	    ELSE IF ((res.EQ.'THR ').AND.(atyp.EQ.'HG1 ')) THEN
	      res_root		= 'THR '
	      atyph		= 'HG1 '
	      atyp1_root	= 'OG1 '
	      atyp2_root	= 'CB  '
	      atyp3_root	= 'CA  '
	      move_here		= .TRUE.
	    ELSE IF ((res.EQ.'TYR ').AND.(atyp.EQ.'HH  ')) THEN
	      res_root		= 'TYR '
	      atyph		= 'HH  '
	      atyp1_root	= 'OH  '
	      atyp2_root	= 'CZ  '
	      atyp3_root	= 'CE2 '
	      move_here		= .TRUE.
	    ELSE IF ((res.EQ.'TYRH').AND.(atyp.EQ.'HH  ')) THEN
	      res_root		= 'TYRH'
	      atyph		= 'HH  '
	      atyp1_root	= 'OH  '
	      atyp2_root	= 'CZ  '
	      atyp3_root	= 'CE2 '
	      move_here		= .TRUE.
	    END IF
	    IF (move_here) THEN
	      nrec_here		= nrec
	      nres_root		= nres
	      chain_root	= chain
	      DO 310 n		= 1, 3
		xyzh (n)	= xyz (n)
310	      CONTINUE						! End n
	      root1_found	= .FALSE.
	      root2_found	= .FALSE.
	      root3_found	= .FALSE.
	      DO 400 jrec	= 1, n_pdb_store
	       READ (pdb_store (jrec),1150) atomj
	       IF ((atomj.EQ.'ATOM').OR.(atomj.EQ.'HETA')) THEN
		READ (pdb_store (jrec),1300) atyp,res,chain,
     1					     nres,xyz
		IF ((chain.EQ.chain_root).AND.(res.EQ.res_root).AND.
     1		    (nres.EQ.nres_root)) THEN
		  IF      (atyp.EQ.atyp1_root) THEN
		    DO 320 n	= 1, 3
		      xyz1 (n)	= xyz (n)
320		    CONTINUE					! End n
		    root1_found	= .TRUE.
		  ELSE IF (atyp.EQ.atyp2_root) THEN
		    DO 330 n	= 1, 3
		      xyz2 (n)	= xyz (n)
330		    CONTINUE					! End n
		    root2_found	= .TRUE.
		  ELSE IF (atyp.EQ.atyp3_root) THEN
		    DO 340 n	= 1, 3
		      xyz3 (n)	= xyz (n)
340		    CONTINUE					! End n
		    root3_found	= .TRUE.
		  END IF
		END IF
	       END IF
400	      CONTINUE						! End jrec
	      IF (root1_found.AND.root2_found.AND.root3_found) THEN
		n_move_pdb		= n_move_pdb + 1
		IF (n_move_pdb.GT.20000) STOP 'n_move_pdb > 20000'
		i			= n_move_pdb
		move_pdb_res (i)	= res_root
		move_pdb_nres (i)	= nres_root
		move_pdb_chain (i)	= chain_root
		move_pdb_ind (i)	= nrec_here
		move_pdb_atyp (i,1)	= atyph
		move_pdb_atyp (i,2)	= atyp1_root
		move_pdb_atyp (i,3)	= atyp2_root
		move_pdb_atyp (i,4)	= atyp3_root
		DO 420 n		= 1, 3
		  move_pdb_xyz (i,1,n)	= xyzh (n)
		  move_pdb_xyz (i,2,n)	= xyz1 (n)
		  move_pdb_xyz (i,3,n)	= xyz2 (n)
		  move_pdb_xyz (i,4,n)	= xyz3 (n)
420		CONTINUE					! End n
	      ELSE
		WRITE (4,1500) chain_root, nres_root, res_root
	      END IF
	    END IF
	  END IF
500	CONTINUE						! End nrec

	RETURN
	END


	SUBROUTINE	read_polh_charges
C	**********	*****************

C Read in the charges - assume that we have just a single charge set
C (es_1abc_all.qop) which is not overwritten through the calculation.

	REAL		qr_buf (7), xyz (3)
	CHARACTER*4	qa4_buf (3)
	INTEGER		qi_buf (3)
	LOGICAL		ql_buf (2)
	CHARACTER*1	qc_buf
	INTEGER		which_VdW_es
	LOGICAL		macinc, itrate, pkq
	REAL		Q_xyz (100000,3), Q (100000)
	INTEGER		Q_nres (100000), Q_stat (100000)
	CHARACTER*4	Q_res (100000), Q_atyp (100000)
	CHARACTER*1	Q_chain (100000)
	CHARACTER*80	pdb_store (100000)
	CHARACTER*60	molecule_coords, ph, hbond_lim
	LOGICAL		move_here, ok_vs_bg, all_ionise, remove_oh
	REAL		move_pdb_xyz (20000,5,3),
     1			move_pdb_rot (20000,30,6), move_pdb_Q (20000)
	INTEGER		move_pdb_nres (20000), move_pdb_ind (20000)
	CHARACTER*4	move_pdb_res (20000), move_pdb_atyp (20000,4)
	CHARACTER*1	move_pdb_chain (20000)
	LOGICAL		move_clash (20000,30)

	COMMON	/env/		molecule_coords, imc, ph, iph,
     1				ph_here, hbond_lim, ihb, hbond,
     2				all_ionise, remove_oh
	COMMON	/pdb/		pdb_store, n_pdb_store
	COMMON	/move_pdb/	n_move_pdb, move_pdb_xyz,
     1				move_pdb_res, move_pdb_nres,
     2				move_pdb_chain, move_pdb_ind,
     3				move_pdb_atyp, move_pdb_rot,
     4				move_pdb_Q, move_clash
	COMMON	/charges/	num_Q, Q_chain, Q_nres, Q_res,
     1				Q_atyp, Q_xyz, Q, Q_stat

1200	FORMAT (' ','Num Qs / root Qs / (O)H Qs = ',3I6)

C For each charge, we'll see what relation it has to the movers:

C	Any root atom will be flagged for switching
C	charge off in bg charge calculation (1 in Q_stat array).
C	Any mover will be flagged for xyz look-up from moving arrays (2).

	n_Qstat1		= 0
	n_Qstat2		= 0
	DO 180 nm		= 1, n_move_pdb
	  move_pdb_Q (nm)	= 999.
180	CONTINUE						! End nm
	OPEN (UNIT=3,NAME='es_'//molecule_coords(1:imc)//'_all.qop',
     1	      TYPE='OLD', FORM='UNFORMATTED')
	READ (3) nfil
	DO 200 ifil		= 1, nfil
	  READ (3)						! pdbfil
	  READ (3)						! qfil
200	CONTINUE						! ifil
	READ (3) nq_here, macinc, itrate, i_qset, which_VdW_es, ncal
	DO 300 iq		= 1, nq_here
	  READ (3) (qr_buf (m), m=1,7), (qa4_buf (m), m=1,3),
     1		   (qi_buf (m), m=1,3), (ql_buf (m), m=1,2),
     2		    qc_buf
	  Q_xyz (iq,1)		= qr_buf (1)
	  Q_xyz (iq,2)		= qr_buf (2)
	  Q_xyz (iq,3)		= qr_buf (3)
	  Q_chain (iq)		= qc_buf
	  Q_nres (iq)		= qi_buf (1)
	  Q_res (iq)		= qa4_buf (1)
	  Q_atyp (iq)		= qa4_buf (2)
	  IF (ql_buf (1)) THEN
	    qmag		= qr_buf (4)
	    pkq			= ql_buf (1)
	    pk			= qr_buf (5)
	    CALL	q_calc (qmag,pkq,pk,ph_here,q_here,all_ionise)
	    Q (iq)		= q_here
	  ELSE
	    Q (iq)		= qr_buf (4)
	  END IF

	  Q_stat (iq)		= 0
	  DO 240 nm		= 1, n_move_pdb		! flag mover/root at
	    IF ((Q_chain(iq).EQ.move_pdb_chain(nm)).AND.
     1	        (Q_nres (iq).EQ.move_pdb_nres (nm)).AND.
     2          (Q_res  (iq).EQ.move_pdb_res  (nm))) THEN
	      IF (Q_atyp(iq).EQ.move_pdb_atyp(nm,1)) THEN
		Q_stat (iq)	= 2			! H atom
		move_pdb_Q (nm)	= Q (iq)
		n_Qstat2	= n_Qstat2 + 1
		GOTO 300
	      END IF
	      DO 220 na		= 2, 4
		IF (Q_atyp(iq).EQ.move_pdb_atyp(nm,na)) THEN
		  Q_stat (iq)	= 1			! Root atom
		  n_Qstat1	= n_Qstat1 + 1
		  GOTO 300
		END IF
220	      CONTINUE						! End na
	    END IF
240	  CONTINUE						! End nm

300	CONTINUE						! End iq
	num_Q			= nq_here
	CLOSE (UNIT=3)
	WRITE (4,1200) num_Q, n_Qstat1, n_Qstat2

C Now populate charge interaction slots 4/5 for the move_pdb_rot array:
C	slot 4 is bg charge interactions
C	slot 5 is intn with all moving atom roots, except its own root
C	slot 6 will come if we iterate - intn from last cycle movers

	dist_check		= hbond - 1.0
	DO 400 nm		= 1, n_move_pdb
	  IF (move_pdb_Q (nm) .EQ. 999.) THEN
	    WRITE (6,*) 'moving charge unset ',
     1      move_pdb_chain(nm),move_pdb_nres(nm),move_pdb_res(nm)
	    STOP
	  END IF
	  DO 380 nr		= 1, 30
	    DO 360 n		= 1, 3
	      xyz (n)		= move_pdb_rot (nm,nr,n)
360	    CONTINUE						! End n
	    nm_here		= nm
	    CALL	DH_int_prelim (xyz, nm_here, val1, val2,
     1				      dist_check, ok_vs_bg)
	    move_pdb_rot (nm,nr,4) = val1
	    move_pdb_rot (nm,nr,5) = val2
	    IF (ok_vs_bg) THEN
	      move_clash (nm,nr)	= .FALSE.
	    ELSE
	      move_clash (nm,nr)	= .TRUE.
	    END IF

380	  CONTINUE						! End nr
400	CONTINUE						! End nm

	RETURN
	END


	SUBROUTINE	q_calc (qmag,pkq,pk,ph_here,q_here,all_ionise)
C	**********	******

C With (pk,ph) adjustment, fraction of charged/uncharged = 10**ff*(ph-pk),
C where ff=-1 for a base, ff=+1 for an acid.

C We have the all_ionise flag in to just take ionisable gps charged
C regardless of pH - if not set, go with the passed pH.

	LOGICAL		pkq, all_ionise

9000	FORMAT ('0','*** ERROR *** problem with q_here in q_calc')

	IF (pkq) THEN
	  ff		= 1.0
	  IF (qmag .GT. 0.) ff = -1.0
	  exp		= ff * (ph_here-pk)
	  IF (exp .GT.  60.) exp =  60.
	  IF (exp .LT. -60.) exp = -60.
	  fracq		= 10. ** exp
	  frac		= 1. / (1. + 1./fracq)

	  IF (all_ionise) THEN
	    q_here	= qmag
	  ELSE
	    q_here	= frac * qmag
	  END IF

	ELSE
	  q_here	= qmag
	END IF

	IF (ABS (q_here) .GT. ABS (qmag)) GOTO 900
	IF (q_here * qmag .LT. 0.)	  GOTO 900
	RETURN

900	WRITE (6,9000)
	CALL	EXIT
	END


	SUBROUTINE	calc_best_hyds
C	**********	**************

C Get the best MF oH-hyd positions.
C ** AT PRESENT JUST LOOK IN BG CHARGES AND DONT ITERATE **

	REAL		Q_xyz (100000,3), Q (100000)
	INTEGER		Q_nres (100000), Q_stat (100000)
	CHARACTER*4	Q_res (100000), Q_atyp (100000)
	CHARACTER*1	Q_chain (100000)
	REAL		move_pdb_xyz (20000,5,3),
     1			move_pdb_rot (20000,30,6), move_pdb_Q (20000)
	INTEGER		move_pdb_nres (20000), move_pdb_ind (20000)
	CHARACTER*4	move_pdb_res (20000), move_pdb_atyp (20000,4)
	CHARACTER*1	move_pdb_chain (20000)
	LOGICAL		move_clash (20000,30)
	REAL		a_ax (3), b_ax (3), new_axes (3,3),
     1			DC_1 (3,3), DC_2 (3,3), rot_mat (3,3),
     2			xyz (3), xyz_new (3), b_axu (3)
	LOGICAL		singular
	REAL		sc_xyz (4,3)

	COMMON	/rot_mats/	DC_1, DC_2, rot_mat
	COMMON	/move_pdb/	n_move_pdb, move_pdb_xyz,
     1				move_pdb_res, move_pdb_nres,
     2				move_pdb_chain, move_pdb_ind,
     3				move_pdb_atyp, move_pdb_rot,
     4				move_pdb_Q, move_clash
	COMMON	/charges/	num_Q, Q_chain, Q_nres, Q_res,
     1				Q_atyp, Q_xyz, Q, Q_stat

C Reminder:
C	slot 4 is bg charge interactions
C	slot 5 is intn with all moving atom roots, except its own root
C	slot 6 will come if we iterate - intn from last cycle movers

C Energies in kJ/mole hopefully / put RT = 2.5 kJ/mole in stat. wting.
C BUT don't use stat. wting - just take the most favourable angle -
C (April 03) BUT NOT if closer than our hbond i/p criterion.

	DO 400 nm		= 1, n_move_pdb

C??	  ang_deg_av		= 0.
C??	  sum_bolt		= 0.
	  en_best		= 9999.
	  ang_best		= 0.
	  DO 300 nr		= 1, 30
	    ang_deg		= 12.*FLOAT(nr-1)
	    IF (move_clash (nm,nr)) GOTO 300
	    IF (move_pdb_rot (nm,nr,4) .LT. en_best) THEN
	      en_best		= move_pdb_rot (nm,nr,4)
	      ang_best		= ang_deg
	    END IF
C??	    bolt_here		= 2.718**(-move_pdb_rot(nm,nr,4)/2.5)
C??	    ang_deg_av		= ang_deg_av + ang_deg*bolt_here
C??	    sum_bolt		= sum_bolt + bolt_here
300	  CONTINUE						! End nr
C??	  ang_deg		= ang_deg_av / sum_bolt

C??
C??	IF (en_best .EQ. 9999.) THEN
C??	WRITE (6,*) move_pdb_nres (nm), move_pdb_res (nm)
C??	END IF
C??	  IF (en_best .EQ. 9999.) STOP 'POLH_MOVE: too much clash'
C?? Very occasionally we have a problem here - do not drop out.
	  IF (en_best .EQ. 9999.) THEN
	    ang_best = 0.
	    WRITE (4,*) '*** A polh_move clash overriden - cludge ***'
	  END IF

	  DO 310 n		= 1, 3
	    sc_xyz (1,n)	= move_pdb_xyz (nm,4,n)
	    sc_xyz (2,n)	= move_pdb_xyz (nm,3,n)
	    sc_xyz (3,n)	= move_pdb_xyz (nm,2,n)
	    sc_xyz (4,n)	= move_pdb_xyz (nm,1,n)
310	  CONTINUE						! End n
	  DO 320 n		= 1, 3
	    a_ax (n)		= sc_xyz (2,n)
	    b_ax (n)		= sc_xyz (3,n) - sc_xyz (2,n)
	    CALL	vec_normalise (b_ax, b_axm, b_axu)
320	  CONTINUE						! End n
C Apply the chi rotation about r_axis = a_ax + lambda*b_axu.
C Get the direction cosine matrices that depend upon the roll-axis.
	  CALL new_axes_gen (b_axu, new_axes)
	  CALL DC_get (new_axes)
	  CALL INVERT3 (DC_1, DC_2, determinant, singular)
	  IF ( singular ) STOP 'singular DC inverse'
	  DO 340 n		= 1, 3
	    xyz (n)		= sc_xyz (4,n)
340	  CONTINUE						! End n
C??	  CALL rot_mat_x (ang_deg, rot_mat)
	  CALL rot_mat_x (ang_best, rot_mat)
	  CALL roll_point (xyz, xyz_new, a_ax)
	  DO 360 n		= 1, 3
	    move_pdb_xyz (nm,5,n) = xyz_new (n)
360	  CONTINUE						! End n

400	CONTINUE						! End nm

	RETURN
	END


	SUBROUTINE	DH_int_prelim (xyz, nm_here, val1, val2,
C	**********	*************
     1				       dist_check, ok_vs_bg)

C Work out DH interactions for this location - split according to root/
C non-root atoms, and ignore self-root of course.

	REAL		Q_xyz (100000,3), Q (100000), xyz (3)
	INTEGER		Q_nres (100000), Q_stat (100000)
	CHARACTER*4	Q_res (100000), Q_atyp (100000)
	CHARACTER*1	Q_chain (100000)
	REAL		move_pdb_xyz (20000,5,3),
     1			move_pdb_rot (20000,30,6), move_pdb_Q (20000)
	INTEGER		move_pdb_nres (20000), move_pdb_ind (20000)
	CHARACTER*4	move_pdb_res (20000), move_pdb_atyp (20000,4)
	CHARACTER*1	move_pdb_chain (20000)
	LOGICAL		move_clash (20000,30)
	LOGICAL		ok_vs_bg
	CHARACTER*4	atyp_here

	COMMON	/move_pdb/	n_move_pdb, move_pdb_xyz,
     1				move_pdb_res, move_pdb_nres,
     2				move_pdb_chain, move_pdb_ind,
     3				move_pdb_atyp, move_pdb_rot,
     4				move_pdb_Q, move_clash
	COMMON	/charges/	num_Q, Q_chain, Q_nres, Q_res,
     1				Q_atyp, Q_xyz, Q, Q_stat

C April 03 - also check the Rij vs BG chagres for being outside our limit,
C which is hbond - 1.0 (the polar hyd radius).

	val1			= 0.
	val2			= 0.
	ok_vs_bg		= .TRUE.
	DO 300 nq		= 1, num_Q
	  Rij			= SQRT ((Q_xyz(nq,1)-xyz(1))**2+
     1					(Q_xyz(nq,2)-xyz(2))**2+
     2					(Q_xyz(nq,3)-xyz(3))**2)
	  IF      (Q_stat (nq) .EQ. 0) THEN	! add bg charge in
C??	    val1		= val1 + (Q(nq)/Rij)*2.718**(-Rij/8.)
	    val1		= val1 + (Q(nq)/Rij)
	    atyp_here		= Q_atyp (nq)
	    IF (atyp_here (1:1) .NE. 'H') THEN
	      IF (Rij .LE. dist_check) ok_vs_bg = .FALSE.
	    END IF
	  ELSE IF (Q_stat (nq) .EQ. 1) THEN	! add root if not this res
	    IF ((Q_chain(nq).NE.move_pdb_chain(nm_here)).OR.
     1	        (Q_nres (nq).NE.move_pdb_nres (nm_here))) THEN
C??	      val2		= val2 + (Q(nq)/Rij)*2.718**(-Rij/8.)
	      val2		= val2 + (Q(nq)/Rij)
	    END IF
	  END IF				! skip Q_stat=2 (H atom)
300	CONTINUE						! End nq
	IF (move_pdb_Q (nm_here) .EQ. 999.) THEN
	  STOP 'moving atom Q not set'
	ELSE
	  Qi			= move_pdb_Q (nm_here)
	  val1			= val1 * Qi * 17.363
	  val2			= val2 * Qi * 17.363
	END IF

C?? rooney - check the values coming out
C??	WRITE (6,*) move_pdb_chain(nm_here),move_pdb_nres(nm_here),
C??     1		    move_pdb_res(nm_here), Qi, val1, val2

	RETURN
	END


	SUBROUTINE	hyds_torsion
C	**********	************

C Make the moving hydrogen alternate locations with routines modified
C from sc_freedom/rotamerise.

	REAL		sc_xyz (4,3)
	REAL		move_pdb_xyz (20000,5,3),
     1			move_pdb_rot (20000,30,6), move_pdb_Q (20000)
	INTEGER		move_pdb_nres (20000), move_pdb_ind (20000)
	CHARACTER*4	move_pdb_res (20000), move_pdb_atyp (20000,4)
	CHARACTER*1	move_pdb_chain (20000)
	LOGICAL		move_clash (20000,30)
	REAL		a_ax (3), b_ax (3), new_axes (3,3),
     1			DC_1 (3,3), DC_2 (3,3), rot_mat (3,3),
     2			xyz (3), xyz_new (3), b_axu (3)
	LOGICAL		singular

	COMMON	/rot_mats/	DC_1, DC_2, rot_mat
	COMMON	/move_pdb/	n_move_pdb, move_pdb_xyz,
     1				move_pdb_res, move_pdb_nres,
     2				move_pdb_chain, move_pdb_ind,
     3				move_pdb_atyp, move_pdb_rot,
     4				move_pdb_Q, move_clash

C Following sc_freedom, we get the start chi, and then just increment this
C in steps of 12 degs to get our 30 locations per OH.

	DO 400 nm		= 1, n_move_pdb
	  DO 220 n		= 1, 3
	    sc_xyz (1,n)	= move_pdb_xyz (nm,4,n)
	    sc_xyz (2,n)	= move_pdb_xyz (nm,3,n)
	    sc_xyz (3,n)	= move_pdb_xyz (nm,2,n)
	    sc_xyz (4,n)	= move_pdb_xyz (nm,1,n)
220	  CONTINUE						! End n
	  DO 240 n		= 1, 3
	    a_ax (n)		= sc_xyz (2,n)
	    b_ax (n)		= sc_xyz (3,n) - sc_xyz (2,n)
	    CALL	vec_normalise (b_ax, b_axm, b_axu)
240	  CONTINUE						! End n
C Apply the chi rotation about r_axis = a_ax + lambda*b_axu.
C Get the direction cosine matrices that depend upon the roll-axis.
	  CALL new_axes_gen (b_axu, new_axes)
	  CALL DC_get (new_axes)
	  CALL INVERT3 (DC_1, DC_2, determinant, singular)
	  IF ( singular ) STOP 'singular DC inverse'
	  DO 260 n		= 1, 3
	    xyz (n)		= sc_xyz (4,n)
260	  CONTINUE						! End n
	  DO 300 na		= 1, 30
	    ang_deg		= 12. * FLOAT (na-1)
	    CALL rot_mat_x (ang_deg, rot_mat)
	    CALL roll_point (xyz, xyz_new, a_ax)
	    DO 280 n		= 1, 3
	      move_pdb_rot (nm,na,n) = xyz_new (n)
280	    CONTINUE						! End n
300	  CONTINUE						! End na
400	CONTINUE						! End nm

	RETURN
	END


	SUBROUTINE	new_axes_gen (R, new_axes)
C	**********	************

C This routine works out a r-h axis system s.t. the point R on regular
C (xyz) system is transformed to (mod(R),0,0) in the new system.
C new_axes (3,3) records the components of the new system in (xyz).
C ?? See JW notes.

	REAL		R (3), new_axes (3,3),
     1			mu_jw, lambda_jw, A (3), B (3), C (3)

C First the new x axis simply is the normalised R vector on old (xyz).
C b_ax/R is already a unit vector - skip vec_normalise.

C	CALL vec_normalise (R, mag_vec, unit_vec)
	DO 200 n = 1, 3
C	  new_axes (1,n)	= unit_vec (n)
	  new_axes (1,n)	= R (n)
200	CONTINUE						! End n

C Now we choose the new y to be coplanar with new x and old y, (see JW notes).

	mu_jw		= SQRT (1./(1.-new_axes(1,2)*new_axes(1,2)))
	lambda_jw	= -mu_jw * new_axes (1,2)

	new_axes (2,1)	= lambda_jw * new_axes (1,1)
	new_axes (2,2)	= lambda_jw * new_axes (1,2) + mu_jw
	new_axes (2,3)	= lambda_jw * new_axes (1,3)

C And the new z is generated so as to give a right-handed set.

	DO 400 n = 1, 3
	  A (n)			= new_axes (1,n)
	  B (n)			= new_axes (2,n)
400	CONTINUE						! End n
	CALL cross_product (A, B, C)
	DO 500 n = 1, 3
	  new_axes (3,n)	= C (n)
500	CONTINUE						! End n
	RETURN
	END


	SUBROUTINE	vec_normalise (vec, mag_vec, unit_vec)
C	**********	*************

C This routine returns magnitude and normalised vector.

	REAL		vec (3), mag_vec, unit_vec (3)

	mag_vec		= SQRT (vec (1)*vec (1) + vec (2)*vec (2) +
     1				vec (3)*vec (3) )
	DO 200 n = 1, 3
	  unit_vec (n)	= vec (n) / mag_vec
200	CONTINUE						! End n
	RETURN
	END


	SUBROUTINE	cross_product (A, B, C)
C	**********	*************

C Routine returns C = A ^ B vector cross product.

	REAL		A (3), B (3), C (3)

	C (1)	= A (2)*B (3) - A (3)*B (2)
	C (2)	= A (3)*B (1) - A (1)*B (3)
	C (3)	= A (1)*B (2) - A (2)*B (1)
	RETURN
	END


	SUBROUTINE	DC_get (new_axes)
C	**********	******

C Routine returns the direction cosine matrix (DC) that transforms coords
C between normal (xyz) system and axial system with components on normal
C (xyz) given in new_axes array.

	REAL		new_axes (3,3), DC_1 (3,3), new_ax (3),
     1			old_axes (3,3), old_ax (3), DC_2 (3,3),
     2			rot_mat (3,3)

	COMMON	/rot_mats/	DC_1, DC_2, rot_mat

	DATA	old_axes(1,1),old_axes(1,2),old_axes(1,3) /1.,0.,0./
	DATA	old_axes(2,1),old_axes(2,2),old_axes(2,3) /0.,1.,0./
	DATA	old_axes(3,1),old_axes(3,2),old_axes(3,3) /0.,0.,1./

	DO 300 l = 1, 3
	  DO 200 n = 1, 3
	    new_ax (n)		= new_axes (l,n)
200	  CONTINUE						! End n
	  DO 300 m = 1, 3
	    DO 250 n = 1, 3
	      old_ax (n)	= old_axes (m,n)
250	    CONTINUE						! End n
	    CALL dot_prod (new_ax, old_ax, dot)
	    DC_1 (l,m)		= dot
300	CONTINUE						! End m,l
	RETURN
	END


	SUBROUTINE	dot_prod (a, b, dot_product)
C	**********	********

C Routine returns the vector dot product, a.b

	REAL		a (3), b (3)

	dot_product	= a(1)*b(1) + a(2)*b(2) + a(3)*b(3)
	RETURN
	END


	SUBROUTINE	mat_transpose (A, A_T)
C	**********	*************

C Routine returns 3x3 matrix transpose of A, A_T.

	REAL		A (3,3), A_T (3,3)

	DO 300 l = 1, 3
	  DO 300 m = 1, 3
	    A_T (m,l)	= A (l,m)
300	CONTINUE						! End m,l
	RETURN
	END


	SUBROUTINE INVERT3 ( A, AINV, ADET, SING )
C	********** *******

C@@@@@@  Test this Routine  LATER !! @@@@@@

C This routine returns the matrix inverse of A in AINV, the determinant of
C A (ADET), and sets SING = .true. if A is singular.
C See J.W., p.C3, Mathematics for Chemists.

C n.b. This is programmed for a 3*3 matrix.

	REAL A (3,3), AINV (3,3), T (3,3,3), DET (3)
	LOGICAL		SING

	DO 210 I = 1, 3
	    DO 200 J = 1, 3
		T (1,I,J) = A (I,J)
200	    CONTINUE						! End J
210	CONTINUE						! End I

	DO 550 I1 = 1, 3
	    IF ( I1 .EQ. 1 ) DET (1) = 0.
	    DO 500 J1 = 1, 3
		CALL STRIKE ( T, I1, J1, 1, 3 )
		DET (2) = 0.
		DO 450 J2 = 1, 2
		    CALL STRIKE ( T, 1, J2, 2, 2 )
		    DET (3) = T (3,1,1)
		    ISN = 1
		    IF ( MOD (1+J2,2) .NE. 0 ) ISN = -1
		    SN = FLOAT (ISN)
		    DET (2) = DET (2) + SN * T (2,1,J2) * DET (3)
450		CONTINUE					! End J2
		ISN = 1
		IF ( MOD (I1+J1,2) .NE. 0 ) ISN = -1
		SN = FLOAT (ISN)
		IF ( I1 .EQ. 1 )
     1          DET (1) = DET (1) + SN * T (1,I1,J1) * DET (2)
		AINV (J1,I1) = SN * DET (2)
500	    CONTINUE						! End J1
550	CONTINUE						! End I1

	ADET = DET (1)
	SING = .FALSE.
	IF ( ADET .EQ. 0. ) SING = .TRUE.
	IF ( SING ) RETURN

	DO 610 I = 1, 3
	    DO 600 J = 1, 3
		AINV (I,J) = AINV (I,J) / ADET
600	    CONTINUE						! End J
610	CONTINUE						! End I

	RETURN
	END


	SUBROUTINE STRIKE ( T, I, J, L, N )
C	********** ******

C This routine strikes out the i row and j column of the matrix (of order N)
C in the L level of T, and inserts the result (order N-1) in the L+1 level
C of T.

	REAL T (3,3,3)

	IF ( L+1 .GT. 3 ) STOP 'T array l+1 .gt. 3'
	IVV = 0
	DO 210 IV = 1, N
	    IF ( IV .EQ. I ) GOTO 210
	    IVV = IVV + 1
	    JVV = 0
	    DO 200 JV = 1, N
		IF ( JV .EQ. J ) GOTO 200
		JVV = JVV + 1
		T (L+1,IVV,JVV) = T (L,IV,JV)
200	    CONTINUE						! End Jv
210	CONTINUE						! End Iv

	RETURN
	END


	SUBROUTINE	rot_mat_x (ang_deg, rot_mat)
C	**********	*********

C Routine returns rotation matrix for rotn of ang_deg about x axis.
C ?? Patriots : in a hurry : look this up : just definition anyway

	REAL		rot_mat (3,3)

	angle		= ang_deg
	angle_rads	= (angle*3.14159)/180.
	c_angle		= COS (angle_rads)
	s_angle		= SIN (angle_rads)
Clnx	c_angle		= COSD (angle)
Clnx	s_angle		= SIND (angle)

	rot_mat (1,1)	= 1.
	rot_mat (1,2)	= 0.
	rot_mat (1,3)	= 0.
	rot_mat (2,1)	= 0.
	rot_mat (2,2)	=  c_angle
	rot_mat (2,3)	= -s_angle
	rot_mat (3,1)	= 0.
	rot_mat (3,2)	=  s_angle
	rot_mat (3,3)	=  c_angle
	RETURN
	END


	SUBROUTINE	roll_point (xyz, xyz_new, a_ax)
C	**********	**********

C Routine performs rotation about axis that has been encoded into the
C matrices passed.
C See JW notes for the transformation, involving the dirn cosine matrices.

	REAL		xyz (3), xyz_new (3), a_ax (3), DC_1 (3,3),
     1			DC_2 (3,3), rot_mat (3,3), tmp_1 (3),
     2			tmp_2 (3), tmp_3 (3), tmp_4 (3)

	COMMON	/rot_mats/	DC_1, DC_2, rot_mat

C ?? Patriots : can we not concatenate these matrix operations?

	sign	= -1.
	CALL vec_trn		(xyz, a_ax, tmp_1, sign)
	CALL mat_vec_mult	(DC_1, tmp_1, tmp_2)
	CALL mat_vec_mult	(rot_mat, tmp_2, tmp_3)
	CALL mat_vec_mult	(DC_2, tmp_3, tmp_4)
	sign	=  1.
	CALL vec_trn		(tmp_4, a_ax, xyz_new, sign)
	RETURN
	END


	SUBROUTINE	vec_trn (A, B, C, sign)
C	**********	*******

C Vector translation, C = A + sign*B

	REAL		A (3), B (3), C (3)

	DO 200 n = 1, 3
	  C (n)		= A (n) + sign * B (n)
200	CONTINUE						! End n
	RETURN
	END


	SUBROUTINE	mat_vec_mult (A, B, C)
C	**********	************

C Routine returns C (vec) = A (mat) * B (vec)

	REAL		A (3,3), B (3), C (3)

	DO 300 l = 1, 3
	  C (l)		= 0.
	  DO 300 m = 1, 3
	    C (l)	= C (l) + A (l,m) * B (m)
300	CONTINUE						! End m,l
	RETURN
	END



	SUBROUTINE	rot_mat_z (ang_deg, rot_mat)
C	**********	*********

C Routine returns rotation matrix for rotn of ang_deg about z axis.
C ?? Patriots : in a hurry : look this up : just definition anyway

	REAL		rot_mat (3,3)

	angle		= ang_deg
Clnx	c_angle		= COSD (angle)
Clnx	s_angle		= SIND (angle)
	angle_rads	= (angle*3.14159)/180.
	c_angle		= COS (angle_rads)
	s_angle		= SIN (angle_rads)

	rot_mat (1,1)	= c_angle
	rot_mat (1,2)	= -s_angle
	rot_mat (1,3)	= 0.
	rot_mat (2,1)	= s_angle
	rot_mat (2,2)	= c_angle
	rot_mat (2,3)	= 0.
	rot_mat (3,1)	= 0.
	rot_mat (3,2)	= 0.
	rot_mat (3,3)	= 1.

	RETURN
	END


	SUBROUTINE	rot_mat_y (ang_deg, rot_mat)
C	**********	*********

C Routine returns rotation matrix for rotn of ang_deg about y axis.
C ?? Patriots : in a hurry : look this up : just definition anyway

	REAL		rot_mat (3,3)

	angle		= ang_deg
Clnx	c_angle		= COSD (angle)
Clnx	s_angle		= SIND (angle)
	angle_rads	= (angle*3.14159)/180.
	c_angle		= COS (angle_rads)
	s_angle		= SIN (angle_rads)

	rot_mat (1,1)	=  c_angle
	rot_mat (1,2)	=  0.
	rot_mat (1,3)	=  s_angle
	rot_mat (2,1)	=  0.
	rot_mat (2,2)	=  1.
	rot_mat (2,3)	=  0.
	rot_mat (3,1)	= -s_angle
	rot_mat (3,2)	=  0.
	rot_mat (3,3)	=  c_angle

	RETURN
	END


	SUBROUTINE	write_best
C	**********	**********

	CHARACTER*80	line, pdb_store (100000)
	CHARACTER*60	molecule_coords, ph, hbond_lim
	CHARACTER*4	res, atyp, atom
	CHARACTER*1	chain
	REAL		xyz_new (3)
	LOGICAL		move_here, all_ionise, write_here,
     1			remove_oh
	REAL		move_pdb_xyz (20000,5,3),
     1			move_pdb_rot (20000,30,6), move_pdb_Q (20000)
	INTEGER		move_pdb_nres (20000), move_pdb_ind (20000)
	CHARACTER*4	move_pdb_res (20000), move_pdb_atyp (20000,4)
	CHARACTER*1	move_pdb_chain (20000)
	LOGICAL		move_clash (20000,30)

	COMMON	/env/		molecule_coords, imc, ph, iph,
     1				ph_here, hbond_lim, ihb, hbond,
     2				all_ionise, remove_oh
	COMMON	/pdb/		pdb_store, n_pdb_store
	COMMON	/move_pdb/	n_move_pdb, move_pdb_xyz,
     1				move_pdb_res, move_pdb_nres,
     2				move_pdb_chain, move_pdb_ind,
     3				move_pdb_atyp, move_pdb_rot,
     4				move_pdb_Q, move_clash

1100	FORMAT (A80)
1150	FORMAT (A4)
1300	FORMAT (13X, A4, A4)
1400	FORMAT (A30,3F8.3,A26)
1800	FORMAT (/,'*** SKIP ALL OTHER polh_move comments ***',/,
     1		  I5,' OH gp atoms REMOVED (OH_REMOVE env set)')

	DO 200 nm		= 1, n_move_pdb
	  nrec			= move_pdb_ind (nm)
	  line (1:80)		= pdb_store (nrec)
	  DO 180 n		= 1, 3
	    xyz_new (n)		= move_pdb_xyz (nm,5,n)
180	  CONTINUE						! End n
	  WRITE (pdb_store (nrec),1400) line (1:30), xyz_new,
     1					line (55:80)
200	CONTINUE						! End nm

	OPEN (UNIT=2, NAME=molecule_coords(1:imc)//'_esH_best.pdb',
     1	      TYPE='NEW')
	IF (remove_oh) n_remove_oh = 0
	DO 300 nrec		= 1, n_pdb_store

	  IF (remove_oh) THEN

	    READ (pdb_store (nrec),1150) atom
	    IF ((atom.EQ.'ATOM').OR.(atom.EQ.'HETA')) THEN

	      READ (pdb_store (nrec),1300) atyp, res
	      write_here		= .TRUE.
	      IF      ((res.EQ.'SER ')) THEN
		IF ((atyp.EQ.'HG  ').OR.(atyp.EQ.'OG  ').OR.
     1		    (atyp.EQ.'CB  ')) THEN
		  write_here	= .FALSE.
	        END IF
	      ELSE IF ((res.EQ.'THR ')) THEN
		IF ((atyp.EQ.'HG1 ').OR.(atyp.EQ.'OG1 ').OR.
     1		    (atyp.EQ.'CB  ')) THEN
		  write_here	= .FALSE.
	        END IF
	      ELSE IF ((res.EQ.'TYRH')) THEN
	        IF ((atyp.EQ.'HH  ').OR.(atyp.EQ.'OH  ').OR.
     1		    (atyp.EQ.'CZ  ')) THEN
		  write_here	= .FALSE.
	        END IF
	      END IF
	      IF (write_here) THEN
	        WRITE (2,1100) pdb_store (nrec)
	      ELSE
	        n_remove_oh	= n_remove_oh + 1
	      END IF

	    ELSE
	      WRITE (2,1100) pdb_store (nrec)
	    END IF

	  ELSE

	    WRITE (2,1100) pdb_store (nrec)

	  END IF

300	CONTINUE						! End nrec
	CLOSE (UNIT=2)

	WRITE (4,*) 'Calculated new locations for ', n_move_pdb,
     1 ' polar (O)H groups'

	IF (remove_oh) WRITE (4,1800) n_remove_oh

	CLOSE (UNIT=4)

C ---------------------------------------------------------------------
C ?? Temporary writing of a coord file with the torsioned locs - for check.
C??	OPEN (UNIT=2, NAME='polh_move_check.pdb', TYPE='NEW')
C??	WRITE (2,1100) pdb_store (1)
C??	DO 800 nm		= 1, n_move_pdb
C??	  nrec			= move_pdb_ind (nm)
C??	  line (1:80)		= pdb_store (nrec)
C??	  DO 780 na		= 1, 30
C??	    DO 760 n		= 1, 3
C??	       xyz_new (n)	= move_pdb_rot (nm,na,n)
C??760	    CONTINUE						! End n
C??	    WRITE (2,1400) line (1:30), xyz_new,
C??     1					  line (55:80)
C??780	  CONTINUE						! End na
C??800	CONTINUE						! End nm
C??	CLOSE (UNIT=2)
C ---------------------------------------------------------------------

	RETURN
	END


	SUBROUTINE	leader (char_array, itop)
C	**********	******

	CHARACTER*60	char_array

	itop	= 1
	DO WHILE (char_array (itop:itop) .NE. ' ')
	  itop	= itop + 1
	END DO
	itop	= itop - 1

	RETURN
	END

