
C ?? - REMEMBER - to make array dimensioning easier to change - flexibly.

C	Version		S C _ G E N	of
C 	Version		S C F _ C X X X       of
C	Program		S C _ F R E E D O M

C sc_gen	Feb 03	to generate missing scs according to highest
C			scf probs.
C NB: since our rotamer definitions have no bond lengths or
C angles, they work by rotating a given sc - so for new scs we need to 
C move a std sc set of coordinates into position.

C Assess freedom of sc's on a fixed backbone, using a rotamer library
C (e.g. Tuffery et al. 1997), and combined probabilities that looks like
C the Koehl and DelaRue (1995) mean field expression, but without a
C force-field othen allowed or disallowed (clashing).
C Jim Warwicker, Glasgow, June 1999.

C Modified incrementally at UMIST:
C - notably to use getenv routine to look up the $DATA environment variable
C - for path to atoms_es.dat and rotamers data files.

C UPDATE AUGUST 2005
C We have kept sc_gen going because it is easier to update it, than to
C combine it with sc_freedom, for now.
C INCL_EXPT (overall) is replaced with flag env variable as follows:
C FIX_SCS	= "COMPLETE"	fix any complete sidechain
C FIX_SCS	= "LISTED"	fix listed sidechains (scf_alter.in)
C FIX_SCS	= "NONE"	fix none (i.e. all sidechains move)
C - controls the fix_type char*4 flag
C - default (FIX_SCS not set) - fix complete sidechains (backwards compat)

C UPDATE OCTOBER 2005
C Again we have a new env variable (FIX_WHEN) to allow for fixing scs
C either by highest probability when all are packed (="END") or by the
C the highest probability of each sidechain when it is first allowed
C (="ASWEGO") i.e. as we are decrementing VdW.
C Again the default without specification is backwards compatible ("END").
C We came upon this from Fay's modelling work, which indicates that the
C high VdW_dec required to fit some sc replacements also allow some others
C (that may be close to fnal site) to have nasty overlaps - so have the
C option now to fix these as go along.
C Note that the way we are implementing this has still got ambiguity -
C rather than fix any happy sidechain for subsequent 'moving' calculations,
C we allow movement of multiple rotamers - we just revisit the first/best
C rotamers at the end of the VdW_dec calculation.  Our thinking is that if
C we don't allow such movement, the ultimate VdW stress will be even worse.
C nb the old collect_clash variable is set to zero in sc_gen - needs to be

C Update SEPTEMBER 2006
C We want to allow scs that are being 'moved' but did have complete orginals
C (e.g. with FIX_SCS = 'NONE' to use the originals as well as rots).  Then
C we can see how well we pack back to original.  Do this by comparing with
C sc_freedom (which uses the old incl_expt variable).
C Also we add another env variable (PROB_HIGHLOW), where 'HIGH' is the default
C and matches previous practice (o/p (moving) rotamer with highest prob), and
C 'LOW' is the new option - may get more buried scs this way - we'll look.

C Update JANUARY 2007
C Well, in general terms PROB_HIGHLOW seems to be working, BUT the 'LOW'
C setting gives clashes (that we don't normally see with 'HIGH').  Add
C REMOVE_CLASHES env variable, feeding into clash_relief flag - default is to
C not apply.  If applied, it tries to work back to rotamers (either up or down
C on prob according to PROB_HIGHLOW) that pack better.  Only one rotamer of
C a pair is 'busted' out of the calculation at a time (for the residue with
C more allowed rotamers).

C Update JUNE 2009 aa_vdw etc to give residue-specific info on vdw tol at
C packing solution, IF env variable AA_VDW is set.

	CHARACTER*60	fix_scs, fix_when, prob_highlow,clash_inf,
     1			remove_clashes, rel_to_val, targ_val,
     2			aa_vdw
	CHARACTER*4	fix_type, res_alt (1000)
	INTEGER		nres_alt (1000)
	CHARACTER*1	chn_alt (1000)
	LOGICAL		rem_oddmc, fix_aswego, prob_high, clash_relief,
     1			relax_to_val, clash_info, aa_output

	COMMON	/experiment/	fix_type, rem_oddmc
	COMMON	/alt_list/	nalt, res_alt, nres_alt, chn_alt
	COMMON	/fix_time/	fix_aswego
	COMMON	/probability/	prob_high, clash_relief
	COMMON	/relax_target/	relax_to_val, target_val, aa_output
	COMMON	/clash_verbose/	clash_info

1200	FORMAT (/,' We are FIXING sidechain conformations for: ',A4,/)
1250	FORMAT ('  Number sidechains fixed from sc_alter.in: ',I4,/)
1400	FORMAT (A1,2X,A4,1X,I5)

	OPEN (UNIT=4, NAME='sc_gen.inf', TYPE='NEW')

	CALL	getenv ('PROB_HIGHLOW', prob_highlow)
	CALL	leader (prob_highlow, iph)
	IF      (prob_highlow (1:iph) .EQ. 'HIGH') THEN
	  prob_high	= .TRUE.
	ELSE IF (prob_highlow (1:iph) .EQ. 'LOW') THEN
	  prob_high	= .FALSE.
	ELSE
	  prob_high	= .TRUE.
	END IF

	CALL	getenv ('FIX_WHEN', fix_when)
	CALL	leader (fix_when, ifw)
	IF      (fix_when (1:ifw) .EQ. 'END') THEN
	  fix_aswego	= .FALSE.
	ELSE IF (fix_when (1:ifw) .EQ. 'ASWEGO') THEN
	  fix_aswego	= .TRUE.
	ELSE
	  fix_aswego	= .FALSE.
	END IF

C Since we are not forcing convergence - we may not have exptal rots -
C RELAX_TO_VAL is just about continuing if clashes are ok earlier.

	CALL	getenv ('RELAX_TO_VAL', rel_to_val)
	CALL	leader (rel_to_val, irv)
	IF      ( (rel_to_val (1:irv) .EQ. 'YES') .OR.
     1		  (rel_to_val (1:irv) .EQ. 'yes') ) THEN
	  relax_to_val	= .TRUE.
	ELSE
	  relax_to_val	= .FALSE.
	END IF

	IF (relax_to_val) THEN
	  CALL	getenv ('TARGET_VAL', targ_val)
	  CALL	leader (targ_val, itv)
	  IF (itv .NE. 0) THEN
	    READ (targ_val(1:itv),*) target_val
	    IF (ABS(AMOD(target_val+0.001,0.1)).GE.0.01) STOP 'targ_val 1 dp'
	  ELSE
	    STOP 'relax_val - target_val not set'
	  END IF
	END IF

	CALL	getenv ('REMOVE_CLASHES', remove_clashes)
	CALL	leader (remove_clashes, irc)
	IF      ( (remove_clashes (1:irc) .EQ. 'YES') .OR.
     1		  (remove_clashes (1:irc) .EQ. 'yes') ) THEN
	  clash_relief	= .TRUE.
	ELSE
	  clash_relief	= .FALSE.
	END IF

	CALL	getenv ('CLASH_INFO', clash_inf)
	CALL	leader (clash_inf, izz)
	IF      ( (clash_inf (1:izz) .EQ. 'YES') .OR.
     1		  (clash_inf (1:izz) .EQ. 'yes') ) THEN
	  clash_info	= .TRUE.
	ELSE
	  clash_info	= .FALSE.
	END IF

	CALL	getenv ('AA_VDW', aa_vdw)
	CALL	leader (aa_vdw, iav)
	IF      ( (aa_vdw (1:iav) .EQ. 'YES') .OR.
     1		  (aa_vdw (1:iav) .EQ. 'yes') ) THEN
	  aa_output	= .TRUE.
	ELSE
	  aa_output	= .FALSE.
	END IF

	CALL	getenv ('FIX_SCS', fix_scs)
	CALL	leader (fix_scs, ifs)
	IF      (fix_scs (1:ifs) .EQ. 'COMPLETE') THEN
	  fix_type	= 'COMP'
	ELSE IF (fix_scs (1:ifs) .EQ. 'LISTED') THEN
	  fix_type	= 'LIST'

	  OPEN (UNIT=3, NAME='scf_alter.in', TYPE='OLD')
	  nalt		= 0
140	  nalt		= nalt + 1
	  IF (nalt .GT. 1000) STOP 'nalt > 1000'
	  READ (3,1400,END=160) chn_alt(nalt), res_alt(nalt),
     1				nres_alt(nalt)
	  GOTO 140

160	  nalt		= nalt - 1
	  CLOSE (UNIT=3)

	ELSE IF (fix_scs (1:ifs) .EQ. 'NONE') THEN
	  fix_type	= 'NONE'
	ELSE
	  fix_type	= 'COMP'
	END IF
	WRITE (4,1200) fix_type
	IF (fix_type .EQ. 'LIST') WRITE (4,1250) nalt

	CALL	sc_params	! parameter setting
	CALL	read_atoms_es	! link to atoms_dat routines
	CALL	rots_set	! call early - naming sc atoms for rotamers

C Read in sc templates (the rotamer library only has torsions/no lengths).

	CALL	sc_templates

C Read in the coordinates, and fill up with sc templates as required.

	CALL	coords_read

C Read in the rotamer library (torsions defining rotamers from template).

	CALL	rots_read

	CALL	rots_gen	! generate coords for all aa sc rotamers
	CALL	rots_prob	! mean-field calculation or rot probs

	CLOSE (UNIT=4)

	CALL	EXIT
	END


	SUBROUTINE	sc_params
C	**********	*********

C Set parameters.

	CHARACTER*8	rot_source
	CHARACTER*4	fix_type
	INTEGER		which_vdw
	LOGICAL		brick_atoms, rot_coords, collect_clash,
     1			rem_oddmc
	REAL		p_new (10000,50), p_old (10000,50)

	COMMON	/ctr_stuff/	which_vdw, rot_source
	COMMON	/ctr_clash/	brick_atoms, vdw_dec, vdw_relax,
     1				conv_frac, collect_clash
	COMMON	/testit/	rot_coords, nres_coords
	COMMON	/probs/		p_new, p_old, frac_new
	COMMON	/experiment/	fix_type, rem_oddmc

	which_vdw	= 1
	rot_source	= 'tuffery '
	brick_atoms	= .TRUE.	! brick ats for quicker clash tests
	vdw_dec	= 0.1			! VdW tolerance dec to clash-free
	collect_clash	= .FALSE.	! dont want detailed clash info
	frac_new	= 1.0		! under-relaxation factor
	conv_frac	= 0.00001	! converegence limit
	rot_coords	= .FALSE.	! dont write rots for specific res

C nb - no relaxn to value used here (unlike sc_freedom)
C If rem_oddmc is set, we remove entire aa for which N, CA, or C absent.
C?? get this from env at some point

	rem_oddmc	= .TRUE.

	RETURN
	END


	SUBROUTINE	coords_read
C	**********	***********

C Get the PDB coordinates, with a single header record.
C ?? Currently don't want any hydrogens in here (fixed or moving) ??

	REAL		xyz (3), at_VdW (100,10),
     1			at_chrg (2000,4), at_pK (2000)
	INTEGER		n_res_ats (200), nat_pter (200,2),
     1			at_class (2000), common_pter (100),
     2			which_vdw
	CHARACTER*4	common_atyp (100), at_prtner (2000),
     1			res_type (200), at_type (2000)
	CHARACTER*80	header
	CHARACTER*60	data_path
	CHARACTER*34	at_info (100)
	CHARACTER*8	rot_source
	CHARACTER*4	atom, atyp, res, atm, fix_type
	CHARACTER*5	atyp_buf
	CHARACTER*1	chain
	LOGICAL		common_at, not_here, got_calpha,
     1			complete, got_cbeta, listed_here
	LOGICAL*1	at_anyq (2000), at_ionise (2000)

	REAL		ats_xyz (70000,3), ats_rad (70000)
	INTEGER		ats_nres (70000), ats_class (70000),
     1			residue (10000,2)
	CHARACTER*4	ats_atyp (70000), ats_res (70000)
	CHARACTER*4	res_here, ats_atom (70000)
	CHARACTER*1	ats_chain (70000)
	LOGICAL*1	ats_comm (70000)

	INTEGER		n_sc_ats (20), chi_ats (20,5,4), sc_chi (20)
	CHARACTER*4	sc_ats (20,10), chi_start (20,3)
	INTEGER*2	sc_rank (20,10)

	REAL		res_chi (20,50,5)
	INTEGER		res_nchi (20), res_natm (20), res_nrot (20)
	CHARACTER*4	res_tag (20)

	REAL		sc_geom_xyz (20,12,3)
	CHARACTER*4	sc_geom_atyp (20,12)

	LOGICAL		end_of_file, got_n, got_c
	CHARACTER*4	buf_atyp (100,2), buf_res (100,2),
     1			buf_atom (100,2)
	REAL		buf_xyz (100,3,2)
	INTEGER		buf_nres (100,2), n_buf (2)
	CHARACTER*1	buf_chain (100,2)
	CHARACTER*80	atom_record
	LOGICAL		first_res, first_atom, rem_oddmc,
     1			fix_residue (10000), comp_residue (10000)

	CHARACTER*4	res_alt (1000)
	INTEGER		nres_alt (1000)
	CHARACTER*1	chn_alt (1000)

	COMMON	/coord_buf/	buf_atyp, buf_res, buf_xyz,
     1				buf_atom, buf_nres, first_res,
     2				atom_record, n_buf, first_atom,
     3				buf_chain
	COMMON	/rot_dict/	res_nchi, res_natm, res_chi,
     1				res_nrot, res_tag
	COMMON	/ctr_stuff/	which_vdw, rot_source
	COMMON	/common_atoms/	common_atyp, common_pter, n_common
	COMMON	/atoms_es_type/ at_info, n_at_types, at_VdW, n_VdW
	COMMON	/atoms_es_resi/	n_res_types, n_chrg, n_all_at,
     1				res_type, n_res_ats, nat_pter,
     2				at_type, at_class, at_chrg,
     3				at_anyq, at_ionise, at_pK,
     4				at_prtner
	COMMON	/ats/		num_ats, ats_xyz, ats_res,
     1				ats_nres, ats_chain, ats_atyp,
     2				ats_rad, ats_class, residue,
     3				num_res, ats_comm, ats_atom,
     4				fix_residue, comp_residue
	COMMON	/rot_scs/	n_sc_ats, sc_ats, chi_ats, sc_chi,
     1				chi_start, sc_rank
	COMMON	/data_loc/	data_path, idp
	COMMON	/experiment/	fix_type, rem_oddmc
	COMMON	/numats/	numats_in, numats_out, numats_add,
     1				numats_ovr, numcb_add, numats_rem
	COMMON	/alt_list/	nalt, res_alt, nres_alt, chn_alt

	DATA	numats_in, numats_out, numats_add / 0, 0, 0 /
	DATA	numats_ovr, numcb_add, numats_rem / 0, 0, 0 /

1100	FORMAT (A80)
1200	FORMAT (A4, 9X, A4, A4, 1X, I4, 4X, 3F8.3)
1220	FORMAT (A4, 8X, A5, A4, A1, I4, 4X, 3F8.3)
2000	FORMAT ('           SC_GEN OUTPUT FILE: input coords ->')
2100	FORMAT (/,A80,/)
2300	FORMAT (A1)
2400	FORMAT (' ','residue ',A1,1X,I4,1X,A4,' missing mc - excluded')

C *** Note that coordinates coming into sc_gen should have been 'cleaned'
C *** by pdb_es.
C Here we read by residue:
C	check an amino acid (otherwise just store as is).
C	for aa, if fix_type='COMP', check complete - if so store/fix
C				- if not replace sc w/ complete template/move
C	for aa, if fix_type-'LIST' - check against scf_alter.in list
C	(for aa), if fix_type='NONE' - replace sc w/ complete template/move
C	but (Sep 06) also add in the original as a rot possibility

	OPEN (UNIT=1, NAME='sc_gen.in', TYPE='OLD')
	READ (1,1100) header
	WRITE (4,2000)
	WRITE (4,2100) header
	WRITE (4,*) '*** LIST of residues modified by sc_gen follows'
	end_of_file	= .FALSE.
	i_ats		= 0
	num_res		= 0
200	num_res		= num_res + 1
	IF (num_res .GT. 10000) STOP 'num_res'
	CALL	read_residue (end_of_file)
	res		= buf_res (1,2)
	nres		= buf_nres (1,2)
	chain		= buf_chain (1,2)
	CALL	res_id (res, irt, ia_1, ia_2, na)
	CALL	res_assign (res, ind_res, not_here)
	numats_in	= numats_in + n_buf (2)

C Leave out entirely any aa that has not got N,CA,C (if rem_oddmc set).
C For Ala we could be listed as complete but not have a CB - check for this.

	IF (.NOT. not_here) THEN	! we have an aa here
	  CALL	check_various (ind_res, got_calpha, got_cbeta,
     1			       got_n, got_c, complete)
	  IF (got_calpha) THEN	! on a potl sc containing aa (not eg NT/CT)
	    IF (res .EQ. 'GLY ') THEN	! fixed since no sc
	      comp_residue (num_res)	= .TRUE.
	      fix_residue (num_res)	= .TRUE.
	    ELSE			! other than Gly
	      IF (.NOT.got_cbeta) THEN  ! will need N,CA,C
		IF ((.NOT.got_n).OR.(.NOT.got_calpha).OR.
     1		    (.NOT.got_c)) THEN
		  IF (rem_oddmc) THEN
	            num_res	= num_res - 1	! leave this amino acid out
	            nres	= buf_nres (1,2)
	            chain	= buf_chain (1,2)
	            WRITE (4,2400) chain, nres, res
	            numats_rem	= numats_rem + n_buf (2)
	            GOTO 410
	          END IF
	        END IF
	      END IF

	      listed_here		= .FALSE.
	      IF (res .EQ. 'ALA ') THEN	! fixed but check for CB
	        comp_residue (num_res)	= .TRUE.
	        fix_residue (num_res)	= .TRUE.
	        IF (.NOT. got_cbeta) THEN
		  CALL	add_sc_atoms (res, ind_res)	! gen whole sc
	        END IF
	      ELSE
		IF (fix_type .EQ. 'LIST') THEN		! check the list
		  listed_here		= .FALSE.
		  DO 220 ialt		= 1, nalt
		    IF ((chain .EQ. chn_alt (ialt)) .AND.
     1		        (res .EQ. res_alt (ialt)) .AND.
     2		        (nres .EQ. nres_alt (ialt))) THEN
		      listed_here	= .TRUE.
		      GOTO 230
		    END IF
220		  CONTINUE				! End ialt
		END IF

230		IF (complete) THEN
		  comp_residue (num_res)	= .TRUE.
		  IF     (fix_type .EQ. 'LIST') THEN
		    IF (listed_here) THEN
		      fix_residue (num_res)	= .TRUE.	! fix list sc
C??tempaug05
C	WRITE (6,*) "fixing listed residue ", chain, nres, res

		    ELSE
		      fix_residue (num_res)	= .FALSE.
		    END IF
		  ELSE IF (fix_type .EQ. 'COMP') THEN
		    fix_residue (num_res)	= .TRUE.	! fix comp sc
		  ELSE IF (fix_type .EQ. 'NONE') THEN
		    fix_residue (num_res)	= .FALSE.	! fix none
		  ELSE
		    STOP 'fix_type undefined'
		  END IF
		ELSE				! incomp - add rest and move
		  IF (listed_here) STOP 'incomplete listed sc'
	          comp_residue (num_res)	= .FALSE.
	          fix_residue (num_res)		= .FALSE.
	          CALL	add_sc_atoms (res, ind_res)	! gen whole sc
		END IF
	      END IF						! end if ALA

	    END IF						! end if GLY
	  ELSE				! not potl sc - store and fix
	    comp_residue (num_res)	= .TRUE.
	    fix_residue (num_res)	= .TRUE.
	  END IF						! end if gotCA
	ELSE				! not protein - store and move on
	  comp_residue (num_res)	= .TRUE.
	  fix_residue (num_res)		= .TRUE.
	END IF							! end if aahere

	DO 400 ia	= 1, n_buf (2)	! store atoms info for this res
	  i_ats		= i_ats + 1
	  IF (i_ats .GT. 70000) STOP 'nat/70000'
	  atyp		= buf_atyp (ia,2)
	  common_at	= .FALSE.
	  ats_comm (i_ats) = .FALSE.
	  DO 360 i_com	= 1, n_common
	    IF (atyp .EQ. common_atyp (i_com)) THEN
	      iat		= common_pter (i_com)
	      common_at		= .TRUE.
	      ats_comm (i_ats)	= .TRUE.
	    END IF
360	  CONTINUE						! End i_com
	  IF (.NOT. common_at) THEN
	    CALL	atom_id (atyp, iat, ia_1, ia_2)
	  END IF
	  ic			= at_class (iat)
	  ats_class (i_ats)	= ic
	  ats_rad (i_ats)	= at_VdW (ic,which_vdw)
	  IF (ats_rad (i_ats) .LT. 0.55) STOP '0.55/rad'
	  DO 380 n		= 1, 3
	    ats_xyz (i_ats,n)	= buf_xyz (ia,n,2)
380	  CONTINUE						! End n
	  ats_res (i_ats)	= res
	  ats_nres (i_ats)	= buf_nres (ia,2)
	  ats_atom (i_ats)	= buf_atom (ia,2)
	  ats_atyp (i_ats)	= atyp
	  ats_chain (i_ats)	= buf_chain (ia,2)
400	CONTINUE						! End ia
	residue (num_res,2)	= i_ats		! res inds in ats arrays
	residue (num_res,1)	= i_ats - n_buf (2) + 1
410	IF (first_res) first_res =.FALSE.
	IF (end_of_file) THEN
	  GOTO 500
	ELSE
	  GOTO 200
	END IF

500	num_ats		= i_ats
	numats_out	= num_ats
	CLOSE (UNIT=1)


	WRITE (4,*) 'END LIST of residues to be modified by sc_gen'

	RETURN
	END


	SUBROUTINE	check_various (ind_res,got_calpha,got_cbeta,
C	**********	*************
     1				       got_n,got_c,complete)

	INTEGER		n_sc_ats (20), chi_ats (20,5,4), sc_chi (20)
	CHARACTER*4	sc_ats (20,10), chi_start (20,3), atyp
	INTEGER*2	sc_rank (20,10)

	CHARACTER*4	buf_atyp (100,2), buf_res (100,2),
     1			buf_atom (100,2)
	REAL		buf_xyz (100,3,2)
	INTEGER		buf_nres (100,2), n_buf (2)
	CHARACTER*1	buf_chain (100,2)
	CHARACTER*80	atom_record
	LOGICAL		first_res, first_atom, got_cbeta,
     1			got_calpha, complete, got_n, got_c

	COMMON	/coord_buf/	buf_atyp, buf_res, buf_xyz,
     1				buf_atom, buf_nres, first_res,
     2				atom_record, n_buf, first_atom,
     3				buf_chain
	COMMON	/rot_scs/	n_sc_ats, sc_ats, chi_ats, sc_chi,
     1				chi_start, sc_rank

	got_calpha	= .FALSE.
	got_cbeta	= .FALSE.
	got_n		= .FALSE.
	got_c		= .FALSE.
	complete	= .FALSE.

	DO 200 ia	= 1, n_buf (2)
	  atyp		= buf_atyp (ia,2)
	  IF (atyp .EQ. 'CA  ') got_calpha = .TRUE.
	  IF (atyp .EQ. 'CB  ') got_cbeta = .TRUE.
	  IF ((atyp.EQ.'N   ').OR.(atyp.EQ.'NT  ')) got_n = .TRUE.
	  IF ((atyp.EQ.'C   ').OR.(atyp.EQ.'CT  ')) got_c = .TRUE.
200	CONTINUE						! End ia

	IF (.NOT. got_n) THEN		! try to look back for N/NT
	  IF (.NOT. first_res) THEN
	    DO 230 ia		= 1, n_buf (1)
	      IF (buf_atyp (ia,is) .EQ. 'NT  ') got_n = .TRUE.
230	    CONTINUE						! End ia
	  END IF
	END IF
C??	  IF (.NOT. got_c) THEN
C??giggs - how to look ahead?
C??	    DO 240 ia		= 1, n_buf (??)
C??	      IF (buf_atyp (ia,??) .EQ. 'CT  ') got_c = .TRUE.
C??240	    CONTINUE						! End ia
C??	  END IF

	IF ((ind_res.EQ.1) .OR. (ind_res.EQ.6)) THEN	! no sc ats G,A
	  complete	= .TRUE.
	ELSE
	  DO 400 ja	= 1, n_sc_ats (ind_res)
	    DO 360 ka	= 1, n_buf (2)
	      IF (buf_atyp(ka,2) .EQ. sc_ats(ind_res,ja)) GOTO 400
360	    CONTINUE						! End ka
	    RETURN
400	  CONTINUE						! End ja
	  complete	= .TRUE.
	END IF

	RETURN
	END


	SUBROUTINE	add_sc_atoms (res, ind_res)
C	**********	************

C We're here for an incomplete aa sc - not just add - overwrite entire sc
C from the template.

	REAL		xyz (3), ca_targ (3), cb_targ (3),
     1			cg_targ (3), cd_targ (3)
	CHARACTER*4	atyp, res
	CHARACTER*1	chain

	INTEGER		n_sc_ats (20), chi_ats (20,5,4), sc_chi (20)
	CHARACTER*4	sc_ats (20,10), chi_start (20,3)
	INTEGER*2	sc_rank (20,10)

	REAL		res_chi (20,50,5)
	INTEGER		res_nchi (20), res_natm (20), res_nrot (20)
	CHARACTER*4	res_tag (20)

	REAL		sc_geom_xyz (20,12,3)
	CHARACTER*4	sc_geom_atyp (20,12)

	CHARACTER*4	buf_atyp (100,2), buf_res (100,2),
     1			buf_atom (100,2)
	REAL		buf_xyz (100,3,2)
	INTEGER		buf_nres (100,2), n_buf (2)
	CHARACTER*1	buf_chain (100,2)
	CHARACTER*80	atom_record
	LOGICAL		first_res, first_atom, ca_no, cb_no, n_no,
     1			c_no

	REAL		cg_mult (3), cd_mult (3), n_targ (3),
     1			c_targ (3), cb_mult (3)
	REAL		cb_ca_targ(3),cb_ca(3),cb_this(3),cb_tran(3),
     1			tmp_vec(3), roll_ax(3), roll_unit(3),
     2			xyz_new (3), a_ax (3), DC_1 (3,3),
     3			DC_2 (3,3), rot_mat (3,3), new_axes (3,3)
	LOGICAL		singular, end_of_file

	COMMON	/rot_dict/	res_nchi, res_natm, res_chi,
     1				res_nrot, res_tag
	COMMON	/rot_scs/	n_sc_ats, sc_ats, chi_ats, sc_chi,
     1				chi_start, sc_rank
	COMMON	/rot_mats/	DC_1, DC_2, rot_mat
	COMMON	/sc_templ/	sc_geom_xyz, sc_geom_atyp,
     1				cg_mult, cd_mult, cb_mult
	COMMON	/coord_buf/	buf_atyp, buf_res, buf_xyz,
     1				buf_atom, buf_nres, first_res,
     2				atom_record, n_buf, first_atom,
     3				buf_chain
	COMMON	/numats/	numats_in, numats_out, numats_add,
     1				numats_ovr, numcb_add, numats_rem

2200	FORMAT (' ','residue ',A1,1X,I4,1X,A4,' CB atom added')
2400	FORMAT (' ','residue ',A1,1X,I4,1X,A4,' adding/replacing sc')

C Should not be here for Gly (but may be for Ala if missing a CB).
C For other scs, translate sc onto CB, then rotate to align CB-CA, using
C roll routines - roll axis perpendicular to i/p and target CB-CA vectors
C (cross prod); roll angle between these vectors.  Should be ok other than
C PRO, where we generate coords on an ad-hoc basis (see pro_adhoc routine).

C If we do not have CB, try digging back to N and C and generating it - then
C carry on as above, using the CB to generate rest of sc.

	IF (res .EQ. 'GLY ') STOP 'Gly add_sc'

	ca_no		= .TRUE.
	cb_no		= .TRUE.
	n_no		= .TRUE.
	c_no		= .TRUE.
	DO 250 ia	= 1, n_buf (2)
	  atyp		= buf_atyp (ia,2)
	  IF (atyp.EQ.'CB  ') THEN
	    DO 242 n		= 1, 3
	      cb_targ (n)	= buf_xyz (ia,n,2)
242	    CONTINUE						! End n
	    cb_no		= .FALSE.
	  END IF
	  IF (atyp.EQ.'CA  ') THEN
	    DO 244 n		= 1, 3
	      ca_targ (n)	= buf_xyz (ia,n,2)
244	    CONTINUE						! End n
	    ca_no		= .FALSE.
	  END IF
	  IF (atyp.EQ.'N   ') THEN
	    DO 245 n		= 1, 3
	      n_targ (n)	= buf_xyz (ia,n,2)
245	    CONTINUE						! End n
	    n_no		= .FALSE.
	  END IF
	  IF (atyp.EQ.'C   ') THEN
	    DO 247 n		= 1, 3
	      c_targ (n)	= buf_xyz (ia,n,2)
247	    CONTINUE						! End n
	    c_no		= .FALSE.
	  END IF
250	CONTINUE						! End ia

C Look for NT (nres/nres-1) or CT (nres/nres+1) for any res, if N, C absent.

	IF (n_no) THEN
	  DO 300 is		= 2, 1, -1
	    DO 290 ia		= 1, n_buf (is)
	      IF (buf_atyp (ia,is) .EQ. 'NT  ') THEN
		DO 280 n	= 1, 3
		  n_targ (n)	= buf_xyz (ia,n,is)
280		CONTINUE					! End n
		n_no		= .FALSE.
C??giggs - maybe put a reportable counter on this NT event
		GOTO 302
	      END IF
290	    CONTINUE						! End ia
300	  CONTINUE						! End is
	END IF

302	IF (c_no) THEN
C??giggs - wrong - need to search up on nres for CT - currently missing CT
C??giggs  DO 310 is		= 2, 1, -1
	  DO 310 is		= 2, 2
	    DO 306 ia		= 1, n_buf (is)
	      IF (buf_atyp (ia,is) .EQ. 'CT  ') THEN
		DO 304 n	= 1, 3
		  c_targ (n)	= buf_xyz (ia,n,is)
304		CONTINUE					! End n
		c_no		= .FALSE.
C??giggs - maybe put a reportable counter on this CT event
		GOTO 320
	      END IF
306	    CONTINUE						! End ia
310	  CONTINUE						! End is
	END IF

320	CONTINUE
	IF (cb_no) THEN				! try to generate CB
	  IF (n_no .OR. ca_no .OR. c_no) THEN	! need mc atoms to gen CB
	    STOP 'CB gen needs N,CA,C'
	  ELSE
	    CALL cb_make (n_targ, ca_targ, c_targ, cb_mult, cb_targ)
	    nn			= n_buf (2) + 1
	    IF (nn .LT. 2) STOP 'nn < 2 in cb add on'
	    buf_res (nn,2)	= res
	    buf_atyp (nn,2)	= 'CB  '
	    buf_nres (nn,2)	= buf_nres (nn-1,2)
	    buf_atom (nn,2)	= buf_atom (nn-1,2)
	    buf_chain (nn,2)	= buf_chain (nn-1,2)
	    DO 340 n		= 1, 3
	      buf_xyz (nn,n,2) 	= cb_targ (n)
340	    CONTINUE						! End n
	    numats_add		= numats_add + 1
	    numcb_add		= numcb_add + 1
	    n_buf (2)		= nn
	    WRITE (4,2200) buf_chain (nn,2), buf_nres (nn,2), res
	    IF (res .EQ. 'ALA ') THEN	! Ala - only here if no CB?
	      IF (.NOT. cb_no) STOP 'cb_no false for Ala in add_sc'
	      cb_no		= .FALSE.
	      RETURN			! Ala - return after adding CB
	    ELSE
	      cb_no		= .FALSE.
	    END IF
	  END IF
	END IF

	IF (res .EQ. 'PRO ') THEN
	  IF (ca_no .OR. cb_no .OR. n_no) THEN	! only need n_targ for PRO
	    STOP 'PRO : ca/cb/n targ not found'
	  END IF
	ELSE
	  IF (ca_no .OR. cb_no) THEN
	    STOP 'non PRO : ca/cb targ not found'
	  END IF
	END IF

	IF (res.EQ.'PRO ') THEN		! generate/replace PRO

	  CALL pro_make (n_targ,ca_targ,cb_targ,cg_mult,cd_mult,
     1			 cg_targ,cd_targ)

	  nn			= n_buf (2)
	  DO 440 ie		= 1, n_sc_ats (ind_res)		! sc atoms
	    atyp		= sc_ats (ind_res,ie)
	    DO 420 n		= 1, 3
	      IF      (atyp .EQ. 'CG  ') THEN
		xyz_new (n)	= cg_targ (n)
	      ELSE IF (atyp .EQ. 'CD  ') THEN
		xyz_new (n)	= cd_targ (n)
	      ELSE
		STOP 'Not CG/CD on PRO'
	      END IF
420	    CONTINUE						! End n
	    DO 430 je		= 1, n_buf (2)	! overwrite or add on
	      IF (buf_atyp (je,2) .EQ. atyp) THEN	! overwrite xyz
		DO 424 n	= 1, 3
		  buf_xyz (je,n,2) = xyz_new (n)
424		CONTINUE					! End n
		numats_ovr	= numats_ovr + 1
		GOTO 440
	      END IF
430	    CONTINUE						! End je
	    nn			= nn + 1		! add on new atom
	    IF (nn .LT. 2) STOP '**** nn < 2'
	    buf_res (nn,2)	= res
	    buf_atyp (nn,2)	= atyp
	    buf_nres (nn,2)	= buf_nres (nn-1,2)
	    buf_atom (nn,2)	= buf_atom (nn-1,2)
	    buf_chain (nn,2)	= buf_chain (nn-1,2)
	    DO 426 n		= 1, 3
	      buf_xyz (nn,n,2) 	= xyz_new (n)
426	    CONTINUE						! End n
	    numats_add		= numats_add + 1
440	  CONTINUE						! End ie
	  WRITE (4,2400) buf_chain (nn,2), buf_nres (nn,2), res
	  n_buf (2)		= nn

	ELSE				! generate/replace non-PRO

	  DO 500 n		= 1, 3
	    cb_ca_targ (n)	= ca_targ (n) - cb_targ (n)
	    cb_ca (n)		= sc_geom_xyz (ind_res,1,n) -
     1				  sc_geom_xyz (ind_res,2,n)
	    cb_tran (n) = cb_targ(n) - sc_geom_xyz(ind_res,2,n)
500	  CONTINUE						! End n
	  CALL cross_product (cb_ca, cb_ca_targ, roll_ax)	! vec sign?
	  CALL vec_normalise (roll_ax, roll_mag, roll_unit)
	  CALL vec_normalise (cb_ca_targ, tmp1, tmp_vec)
	  CALL vec_normalise (cb_ca, tmp2, tmp_vec)
	  CALL dot_prod (cb_ca_targ, cb_ca, tmp3)
	  cos_ang		= tmp3 / (tmp1*tmp2)
	  IF (cos_ang .LT. -1.0) THEN	! return 0->180, arg -1 to + 1
	    IF (cos_ang .LT. -1.00001) THEN
	      STOP 'cos_ang/-1 sc_gen'
	    ELSE
	      cos_ang		= -1.0
	    END IF
	  END IF
	  IF (cos_ang .GT. 1.0) THEN
	    IF (cos_ang .GT. 1.00001) THEN
	      STOP 'cos_ang/+1 sc_gen'
	    ELSE
	      cos_ang	= 1.0
	    END IF
	  END IF
	  ang_roll_rads		= ACOS (cos_ang)
	  ang_roll		= (ang_roll_rads*180.)/3.14159
	  CALL new_axes_gen (roll_unit, new_axes)
	  CALL DC_get (new_axes)
	  CALL INVERT3 (DC_1, DC_2, determinant, singular)
	  IF ( singular ) STOP 'singular DC inverse'
	  CALL rot_mat_x (ang_roll, rot_mat)

C Rotate the Cb-atom vector in dictionary copy about roll axis placed on Cb
C of target - that should do the job: (roll axis from Cb-Ca perpendicular).

	  nn			= n_buf (2)
	  DO 540 ie		= 1, n_sc_ats (ind_res)		! sc atoms
	    atyp		= sc_ats (ind_res,ie)
	    DO 514 n		= 1, 3
	      cb_this (n)	= sc_geom_xyz (ind_res,2+ie,n) -
     1				  sc_geom_xyz (ind_res,2,n)+cb_targ(n)
514	    CONTINUE						! End n
	    CALL roll_point (cb_this,xyz_new,cb_targ)

	    DO 530 je		= 1, n_buf (2)	! overwrite or add on
	      IF (buf_atyp (je,2) .EQ. atyp) THEN	! overwrite xyz
		DO 524 n	= 1, 3
		  buf_xyz (je,n,2) = xyz_new (n)
524		CONTINUE					! End n
		numats_ovr	= numats_ovr + 1
		GOTO 540
	      END IF
530	    CONTINUE						! End je
	    nn			= nn + 1		! add on new atom
	    IF (nn .LT. 2) STOP '**** nn < 2'
	    buf_res (nn,2)	= res
	    buf_atyp (nn,2)	= atyp
	    buf_nres (nn,2)	= buf_nres (nn-1,2)
	    buf_atom (nn,2)	= buf_atom (nn-1,2)
	    buf_chain (nn,2)	= buf_chain (nn-1,2)
	    DO 526 n		= 1, 3
	      buf_xyz (nn,n,2) 	= xyz_new (n)
526	    CONTINUE						! End n
	    numats_add		= numats_add + 1
540	  CONTINUE						! End ie
	  WRITE (4,2400) buf_chain (nn,2), buf_nres (nn,2), res
	  n_buf (2)		= nn

	END IF						! End PRO/non-PRO

	RETURN
	END


	SUBROUTINE	rots_read
C	**********	*********

C Read in rotamer chi tables from the specified dictionary.

	CHARACTER*80	rec
	CHARACTER*8	check, rot_source
	CHARACTER*4	res
	CHARACTER*1	nchi_test
	INTEGER		which_vdw
	LOGICAL		not_here
	INTEGER		n_sc_ats (20), chi_ats (20,5,4), sc_chi (20)
	CHARACTER*4	sc_ats (20,10), chi_start (20,3)
	INTEGER*2	sc_rank (20,10)

	REAL		res_chi (20,50,5)
	INTEGER		res_nchi (20), res_natm (20), res_nrot (20)
	CHARACTER*4	res_tag (20)
	CHARACTER*60	data_path

	COMMON	/ctr_stuff/	which_vdw, rot_source
	COMMON	/rot_dict/	res_nchi, res_natm, res_chi,
     2				res_nrot, res_tag
	COMMON	/rot_scs/	n_sc_ats, sc_ats, chi_ats, sc_chi,
     1				chi_start, sc_rank
	COMMON	/data_loc/	data_path, idp

1100	FORMAT (A80)
1200	FORMAT (7X,A4)
1240	FORMAT (19X,A1)
1300	FORMAT (13X,I2,4X,I1,3X,I2)
1340	FORMAT (13X,I2)
1500	FORMAT (27X, (1X,F6.1))
1600	FORMAT (27X,2(1X,F6.1))
1700	FORMAT (27X,3(1X,F6.1))
1800	FORMAT (27X,4(1X,F6.1))
1900	FORMAT (27X,5(1X,F6.1))
2000	FORMAT (/,' With the rotamer library of ',A8,' we have
     1 ',I4,' total rotamers as follows:')
2100	FORMAT (/,'index  res  natm  nchi  nrot',/)
2200	FORMAT (' ',1X,I2,3X,A4,3X,I2,3X,I1,4X,I2)

C Actually, we have modified (reduced) the tuffery file a little - where
C the final 1 or 2 chi's correspond to hydrogen positions, we have removed
C them, and we assume that Gly, Pro and Ala are all fixed at i/p coords.
C We retain the 180 chi_5 for Arg.
C ?? BUT NOTE the crossover between using a chi to place hyds and our
C ?? general polar hyd 'optimisation' problem.

	IF (rot_source .EQ. 'tuffery ') THEN
	  OPEN (UNIT=2, NAME=data_path(1:idp-1)//'rots_tuffery.dat',
     1		TYPE='OLD')
	  READ (2,*)
	  READ (2,*)
	  DO 180 ind_res	= 1, 20
	    res_nchi (ind_res)	= -1
	    res_nrot (ind_res)	= 0
180	  CONTINUE						! End ind_res
	  i_tot			= 0
200	  i_tot			= i_tot + 1
	  READ (2,1100,END=400) rec
	  READ (rec,1200) res
	  IF (res .EQ. '----') THEN
	    i_tot		= i_tot - 1
	    GOTO 200
	  ELSE
	    CALL res_assign (res, ind_res, not_here)
	    IF (not_here) STOP 'not_here'
	    IF (res_nchi (ind_res) .EQ. -1) THEN
	      READ (rec,1240) nchi_test
	      IF (nchi_test .EQ. ' ') THEN
		STOP 'nchi_test'
	      ELSE
		READ (rec,1300) num, nchi, natm
		IF (num .NE. 1) STOP 'num .ne. 1'
		res_natm (ind_res)	= natm
		res_nchi (ind_res)	= nchi
		IF (nchi .EQ. 0) THEN
		  i_tot		= i_tot - 1
		  GOTO 200
		END IF
	      END IF
	    END IF
	    nchi		= res_nchi (ind_res)
	    res_nrot (ind_res)	= res_nrot (ind_res) + 1
	    READ (rec,1340) num
	    IF (num .NE. res_nrot (ind_res)) STOP 'num/res_nrot'
	    IF (nchi .EQ. 0) THEN
	      STOP '?? nchi=0 - recode'
	    ELSE IF (nchi .EQ. 1) THEN
	      READ (rec,1500)  res_chi (ind_res,num,1)
	    ELSE IF (nchi .EQ. 2) THEN
	      READ (rec,1600) (res_chi (ind_res,num,m),m=1,2)
	    ELSE IF (nchi .EQ. 3) THEN
	      READ (rec,1700) (res_chi (ind_res,num,m),m=1,3)
	    ELSE IF (nchi .EQ. 4) THEN
	      READ (rec,1800) (res_chi (ind_res,num,m),m=1,4)
	    ELSE IF (nchi .EQ. 5) THEN
	      READ (rec,1900) (res_chi (ind_res,num,m),m=1,5)
	    ELSE
	      STOP 'nchi .gt. 5'
	    END IF
	    GOTO 200
	  END IF

400	  n_tot		= i_tot - 1
	  CLOSE (UNIT=2)
	  DO 430 n	= 1, 20
	    IF (sc_chi (n) .NE. res_nchi (n)) STOP 'chi mism'
430	  CONTINUE						! End n
	ELSE
	  STOP 'rot_source'
	END IF

C Write some information then.

	WRITE (4,2000) rot_source, n_tot
	WRITE (4,2100)
	DO 500 ind_res		= 1, 20
	  WRITE (4,2200) ind_res,res_tag(ind_res),res_natm(ind_res),
     1			 res_nchi(ind_res),res_nrot(ind_res)
500	CONTINUE						! End ind_res

	RETURN
	END


	SUBROUTINE	rots_set
C	**********	********

C Yes, I know that this is ugly, thankyou.
C When we come to the rotamer generation, the sc atoms are mapped to their
C corresponding array indices in sc_ats.  Each chi operation is defined by
C 4 atoms, the middle 2 defining the rotation axis, the first giving the
C chi origin, and the last being moved.  All sc atoms from this last atom
C to the end of the sc are moved in this chi operation.

C The three atoms that we need to start dihedral definitions, but which are
C not moved are stored in the chi_start array, are labelled -2,-1 and 0 in the
C chi_ats array.
C For amino acids, 0 is CB, -1 is CA, -2 is N   [Biochem (1970) 9, 3471]
C sc_rank stores which 'moving' atoms are within 2 atoms of the fixed point,
C with +1 nn to fixed pt, +2 second nn to fixed pt, 0 otherwise.
C ?? Actually not using rank derived things at present.

	INTEGER		n_sc_ats (20), chi_ats (20,5,4), sc_chi (20)
	CHARACTER*4	sc_ats (20,10), chi_start (20,3)
	INTEGER*2	sc_rank (20,10)

	COMMON	/rot_scs/	n_sc_ats, sc_ats, chi_ats, sc_chi,
     1				chi_start, sc_rank

	DATA	n_sc_ats (1),sc_chi (1)	/ 0, 0 /		! Ala
	DATA	(sc_ats (1,m),m=1,10)	/ 10*'    ' /
	DATA	(sc_rank (1,m),m=1,10)	/ 10*0 /
	DATA	(chi_start(1,m),m=1,3)	/ 'N   ','CA  ','CB  ' /

	DATA	n_sc_ats (2),sc_chi (2) / 1, 1 /		! Cys
	DATA	(sc_ats (2,m),m=1,10)	/ 'SG  ', 9*'    ' /
	DATA	(sc_rank (2,m),m=1,10)	/ 1, 9*0 /
	DATA	(chi_ats (2,1,n),n=1,4)	/ -2,-1, 0, 1 /
	DATA	(chi_start(2,m),m=1,3)	/ 'N   ','CA  ','CB  ' /

	DATA	n_sc_ats (3),sc_chi (3)	/ 3, 2 /		! Asp
	DATA	(sc_ats (3,m),m=1,10)	/ 'CG  ','OD1 ','OD2 ',	
     1					7*'    ' /
	DATA	(sc_rank (3,m),m=1,10)	/ 1, 2, 2, 7*0 /
	DATA	(chi_ats (3,1,n),n=1,4)	/ -2,-1, 0, 1 /
	DATA	(chi_ats (3,2,n),n=1,4)	/ -1, 0, 1, 2 /
	DATA	(chi_start(3,m),m=1,3)	/ 'N   ','CA  ','CB  ' /

	DATA	n_sc_ats (4),sc_chi (4)	/ 4, 3 /		! Glu
	DATA	(sc_ats (4,m),m=1,10)	/ 'CG  ','CD  ','OE1 ',
     1					  'OE2 ',6*'    ' /
	DATA	(sc_rank (4,m),m=1,10)	/ 1, 2, 8*0 /
	DATA	(chi_ats (4,1,n),n=1,4)	/ -2,-1, 0, 1 /
	DATA	(chi_ats (4,2,n),n=1,4)	/ -1, 0, 1, 2 /
	DATA	(chi_ats (4,3,n),n=1,4)	/  0, 1, 2, 3 /
	DATA	(chi_start(4,m),m=1,3)	/ 'N   ','CA  ','CB  ' /

	DATA	n_sc_ats (5),sc_chi (5)	/ 6, 2 /		! Phe
	DATA	(sc_ats (5,m),m=1,10)	/ 'CG  ','CD1 ','CD2 ',
     1					  'CE1 ','CE2 ','CZ  ',
     2					4*'    ' /
	DATA	(sc_rank (5,m),m=1,10)	/ 1, 2, 2, 7*0 /
	DATA	(chi_ats (5,1,n),n=1,4)	/ -2,-1, 0, 1 /
	DATA	(chi_ats (5,2,n),n=1,4)	/ -1, 0, 1, 2 /
	DATA	(chi_start(5,m),m=1,3)	/ 'N   ','CA  ','CB  ' /

	DATA	n_sc_ats (6),sc_chi (6)	/ 0, 0 /		! Gly
	DATA	(sc_ats (6,m),m=1,10)	/ 10*'    ' /
	DATA	(sc_rank (6,m),m=1,10)	/ 10*0 /
	DATA	(chi_start(6,m),m=1,3)	/ 'N   ','CA  ','CB  ' /

	DATA	n_sc_ats (7),sc_chi (7)	/ 5, 2 /		! His
	DATA	(sc_ats (7,m),m=1,10)	/ 'CG  ','ND1 ','CD2 ',
     1					  'CE1 ','NE2 ',
     2					5*'    ' /
	DATA	(sc_rank (7,m),m=1,10)	/ 1, 2, 2, 7*0 /
	DATA	(chi_ats (7,1,n),n=1,4)	/ -2,-1, 0, 1 /
	DATA	(chi_ats (7,2,n),n=1,4)	/ -1, 0, 1, 2 /
	DATA	(chi_start(7,m),m=1,3)	/ 'N   ','CA  ','CB  ' /

	DATA	n_sc_ats (8),sc_chi (8)	/ 3, 2 /		! Ile
	DATA	(sc_ats (8,m),m=1,10)	/ 'CG1 ','CG2 ','CD1 ',
     1					7*'    ' /
	DATA	(sc_rank (8,m),m=1,10)	/ 1, 1, 2, 7*0 /
	DATA	(chi_ats (8,1,n),n=1,4)	/ -2,-1, 0, 1 /
	DATA	(chi_ats (8,2,n),n=1,4)	/ -1, 0, 1, 3 /
	DATA	(chi_start(8,m),m=1,3)	/ 'N   ','CA  ','CB  ' /

	DATA	n_sc_ats (9),sc_chi (9)	/ 4, 4 /		! Lys
	DATA	(sc_ats (9,m),m=1,10)	/ 'CG  ','CD  ','CE  ',
     1					  'NZ  ',6*'    ' /
	DATA	(sc_rank (9,m),m=1,10)	/ 1, 2, 8*0 /
	DATA	(chi_ats (9,1,n),n=1,4)	/ -2,-1, 0, 1 /
	DATA	(chi_ats (9,2,n),n=1,4)	/ -1, 0, 1, 2 /
	DATA	(chi_ats (9,3,n),n=1,4)	/  0, 1, 2, 3 /
	DATA	(chi_ats (9,4,n),n=1,4)	/  1, 2, 3, 4 /
	DATA	(chi_start(9,m),m=1,3)	/ 'N   ','CA  ','CB  ' /

	DATA	n_sc_ats(10),sc_chi(10)	/ 3, 2 /		! Leu
	DATA	(sc_ats(10,m),m=1,10)	/ 'CG  ','CD1 ','CD2 ',
     1					7*'    ' /
	DATA	(sc_rank(10,m),m=1,10)	/ 1, 2, 2, 7*0 /
	DATA	(chi_ats(10,1,n),n=1,4)	/ -2,-1, 0, 1 /
	DATA	(chi_ats(10,2,n),n=1,4)	/ -1, 0, 1, 2 /
	DATA	(chi_start(10,m),m=1,3)	/ 'N   ','CA  ','CB  ' /

	DATA	n_sc_ats(11),sc_chi(11)	/ 3, 3 /		! Met
	DATA	(sc_ats(11,m),m=1,10)	/ 'CG  ','SD  ','CE  ',
     1					7*'    ' /
	DATA	(sc_rank(11,m),m=1,10)	/ 1, 2, 8*0 /
	DATA	(chi_ats(11,1,n),n=1,4)	/ -2,-1, 0, 1 /
	DATA	(chi_ats(11,2,n),n=1,4)	/ -1, 0, 1, 2 /
	DATA	(chi_ats(11,3,n),n=1,4)	/  0, 1, 2, 3 /
	DATA	(chi_start(11,m),m=1,3)	/ 'N   ','CA  ','CB  ' /

	DATA	n_sc_ats(12),sc_chi(12)	/ 3, 2 /		! Asn
	DATA	(sc_ats(12,m),m=1,10)	/ 'CG  ','OD1 ','ND2 ',
     1					7*'    ' /
	DATA	(sc_rank(12,m),m=1,10)	/ 1, 2, 2, 7*0 /
	DATA	(chi_ats(12,1,n),n=1,4)	/ -2,-1, 0, 1 /
	DATA	(chi_ats(12,2,n),n=1,4)	/ -1, 0, 1, 2 /
	DATA	(chi_start(12,m),m=1,3)	/ 'N   ','CA  ','CB  ' /

	DATA	n_sc_ats(13),sc_chi(13)	/ 2, 0 /		! Pro
C hender ?? check this out
	DATA	(sc_ats(13,m),m=1,10)	/'CG  ','CD  ',
     1					8*'    ' /
	DATA	(sc_rank(13,m),m=1,10)	/ 10*0 /
	DATA	(chi_start(13,m),m=1,3)	/ 'N   ','CA  ','CB  ' /

	DATA	n_sc_ats(14),sc_chi(14)	/ 4, 3 /		! Gln
	DATA	(sc_ats(14,m),m=1,10)	/'CG  ','CD  ','OE1 ',
     1					 'NE2 ',6*'    ' /
	DATA	(sc_rank(14,m),m=1,10)	/ 1, 2, 8*0 /
	DATA	(chi_ats(14,1,n),n=1,4)	/ -2,-1, 0, 1 /
	DATA	(chi_ats(14,2,n),n=1,4)	/ -1, 0, 1, 2 /
	DATA	(chi_ats(14,3,n),n=1,4)	/  0, 1, 2, 3 /
	DATA	(chi_start(14,m),m=1,3)	/ 'N   ','CA  ','CB  ' /

	DATA	n_sc_ats(15),sc_chi(15)	/ 6, 5 /		! Arg
	DATA	(sc_ats(15,m),m=1,10)	/'CG  ','CD  ','NE  ',
     1					 'CZ  ','NH1 ','NH2 ',
     2					4*'    ' /
	DATA	(sc_rank(15,m),m=1,10)	/ 1, 2, 8*0 /
	DATA	(chi_ats(15,1,n),n=1,4)	/ -2,-1, 0, 1 /
	DATA	(chi_ats(15,2,n),n=1,4)	/ -1, 0, 1, 2 /
	DATA	(chi_ats(15,3,n),n=1,4)	/  0, 1, 2, 3 /
	DATA	(chi_ats(15,4,n),n=1,4)	/  1, 2, 3, 4 /
	DATA	(chi_ats(15,5,n),n=1,4)	/  2, 3, 4, 5 /
	DATA	(chi_start(15,m),m=1,3)	/ 'N   ','CA  ','CB  ' /

	DATA	n_sc_ats(16),sc_chi(16)	/ 1, 1 /		! Ser
	DATA	(sc_ats(16,m),m=1,10)	/ 'OG  ',9*'    ' /
	DATA	(sc_rank(16,m),m=1,10)	/ 1, 9*0 /
	DATA	(chi_ats(16,1,n),n=1,4)	/ -2,-1, 0, 1/
	DATA	(chi_start(16,m),m=1,3)	/ 'N   ','CA  ','CB  ' /

	DATA	n_sc_ats(17),sc_chi(17)	/ 2, 1 /		! Thr
	DATA	(sc_ats(17,m),m=1,10)	/ 'OG1 ','CG2 ',
     1					8*'    ' /
	DATA	(sc_rank(17,m),m=1,10)	/ 1, 1, 8*0 /
	DATA	(chi_ats(17,1,n),n=1,4)	/ -2,-1, 0, 1/
	DATA	(chi_start(17,m),m=1,3)	/ 'N   ','CA  ','CB  ' /

	DATA	n_sc_ats(18),sc_chi(18)	/ 2, 1 /		! Val
	DATA	(sc_ats(18,m),m=1,10)	/ 'CG1 ','CG2 ',
     1					8*'    ' /
	DATA	(sc_rank(18,m),m=1,10)	/ 1, 1, 8*0 /
	DATA	(chi_ats(18,1,n),n=1,4)	/ -2,-1, 0, 1/
	DATA	(chi_start(18,m),m=1,3)	/ 'N   ','CA  ','CB  ' /

	DATA	n_sc_ats(19),sc_chi(19)	/ 9, 2 /		! Trp
	DATA	(sc_ats(19,m),m=1,10)	/ 'CG  ','CD1 ','CD2 ',
     1					  'NE1 ','CE2 ','CE3 ',
     2					  'CZ2 ','CZ3 ','CH2 ',
     3					  '    ' /
	DATA	(sc_rank(19,m),m=1,10)	/ 1, 2, 2, 7*0 /
	DATA	(chi_ats(19,1,n),n=1,4)	/ -2,-1, 0, 1 /
	DATA	(chi_ats(19,2,n),n=1,4)	/ -1, 0, 1, 2 /
	DATA	(chi_start(19,m),m=1,3)	/ 'N   ','CA  ','CB  ' /

	DATA	n_sc_ats(20),sc_chi(20)	/ 7, 2 /		! Tyr
	DATA	(sc_ats(20,m),m=1,10)	/ 'CG  ','CD1 ','CD2 ',
     1					  'CE1 ','CE2 ','CZ  ',
     2					  'OH  ',3*'    ' /
	DATA	(sc_rank(20,m),m=1,10)	/ 1, 2, 2, 7*0 /
	DATA	(chi_ats(20,1,n),n=1,4)	/ -2,-1, 0, 1 /
	DATA	(chi_ats(20,2,n),n=1,4)	/ -1, 0, 1, 2 /
	DATA	(chi_start(20,m),m=1,3)	/ 'N   ','CA  ','CB  ' /

	RETURN
	END


	SUBROUTINE	res_assign (res, ind_res, not_here)
C	**********	**********

	REAL		res_chi (20,50,5)
	INTEGER		res_nchi (20), res_natm (20), res_nrot (20)
	CHARACTER*4	res, res_tag (20), res_tag_here
	LOGICAL		not_here

	COMMON	/rot_dict/	res_nchi, res_natm, res_chi,
     1				res_nrot, res_tag

	DATA	res_tag	/ 'ALA ','CYS ','ASP ','GLU ','PHE ',
     1			  'GLY ','HIS ','ILE ','LYS ','LEU ',
     2			  'MET ','ASN ','PRO ','GLN ','ARG ',
     3			  'SER ','THR ','VAL ','TRP ','TYR ' /

	not_here	= .FALSE.
	DO 200 i	= 1, 20
C??	  IF (res .EQ. res_tag (i)) THEN
	  res_tag_here	= res_tag (i)
	  IF (res (1:3) .EQ. res_tag_here (1:3)) THEN
	    ind_res	= i
	    RETURN
	  END IF
200	CONTINUE						! End i
	not_here	= .TRUE.

	RETURN
	END


	SUBROUTINE	rots_gen
C	**********	********

C Get xyz, in an indexed array, for all rotamers to be studied.

	REAL		ats_xyz (70000,3), ats_rad (70000)
	INTEGER		ats_nres (70000), ats_class (70000),
     1			residue (10000,2)
	CHARACTER*4	ats_atyp (70000), ats_res (70000)
	CHARACTER*1	ats_chain (70000)
	LOGICAL*1	ats_comm (70000)
	REAL		res_chi (20,50,5)
	INTEGER		res_nchi (20), res_natm (20), res_nrot (20)
	CHARACTER*4	res_tag (20)
	CHARACTER*4	fix_type, res, ats_atom (70000)
	LOGICAL		not_here, rot_coords
	INTEGER		n_sc_ats (20), chi_ats (20,5,4), sc_chi (20)
	CHARACTER*4	sc_ats (20,10), chi_start (20,3)
	INTEGER*2	sc_rank (20,10)
	LOGICAL		rem_oddmc, incl_exp (10000),
     1			fix_residue (10000), comp_residue (10000)

	REAL		sc_xyz (15,3), sc_rad (15)
	INTEGER		sc_num, sc_ia (15)

	REAL		fixed (70000,4), moved (700000,4)
	INTEGER		res_rots_ind (10000,4)
	INTEGER*2	moved_ind (700000,3), fixed_ind (70000,2)
	CHARACTER*1	chain

	COMMON	/ats/		num_ats, ats_xyz, ats_res,
     1				ats_nres, ats_chain, ats_atyp,
     2				ats_rad, ats_class, residue,
     3				num_res, ats_comm, ats_atom,
     4				fix_residue, comp_residue
	COMMON	/rot_dict/	res_nchi, res_natm, res_chi,
     1				res_nrot, res_tag
	COMMON	/rot_scs/	n_sc_ats, sc_ats, chi_ats, sc_chi,
     1				chi_start, sc_rank
	COMMON	/ats_loads/	fixed, fixed_ind, moved, moved_ind,
     1				if, im, res_rots_ind, n_fixed,n_moved,
     2				num_residues
	COMMON	/testit/	rot_coords, nres_coords
	COMMON	/experiment/	fix_type, rem_oddmc
	COMMON	/incl_exprot/	incl_exp

2100	FORMAT (/,' processing ',I6,' residues')
2400	FORMAT (' ','Rotamers for nres = ',I4,' - ',I2,' rotamers')
2420	FORMAT ('ATOM',9X,'ALA ','CA  ',1X,I4,4X,3F8.3)
2500	FORMAT (' sc atom missing for ',A4,2X,I4,2X,A4)

C Rotamer generation gives us the fixed and moved arrays:
C fixed, with if counter, gives xyz and rad for a fixed atom
C fixed_ind (if,1) gives the ir/jr residue index for a fixed atom
C fixed_ind (if,2) is -1,-2,-3 for the 1st,2nd,3rd nn atoms to 'moved' ats
C                  and zero otherwise (e.g. CB/-1,CA/-2,NandC/-3)
C moved, with im counter, gives xyz and rad for a rotameric atom
C moved_ind (im,1) gives the ir/jr residue index for a moved atom
C moved_ind (im,2) gives irot/jrot rotamer index for a moved atom
C moved_ind (im,3) gives the sc_rank (1,2,0) for moved at (rel to fixed ats)
C res_rots_ind (ir) points to the moved array for each residue:
C		    slot 1 gives no. of rots for res ir (none if -999)
C		    slot 2 gives no. of ats in moved for each rotamer
C		    slot 3 gives index of first at belonging to ir in moved
C		    slot 4 gives index of last at belonging to ir in moved

C Go through by residue, checking against the set to be left fixed.
C Remember that we currently have this ugly 'common' atom notation of NT and
C CT,OT1,OT2 for peptide terminal gps, and that we should first leave these
C fixed if encountered (don't bother torsioning the C-t carboxyl at present),
C and two may need to assign as chi_1 defining atoms for their sidechains that
C may be in a different nres (ugly indeed).

	if			= 0
	im			= 0
	num_residues		= num_res
	DO 500 ir		= 1, num_res
	  res_rots_ind (ir,1)	= -999
	  incl_exp (ir)		= .FALSE.
	  i1			= residue (ir,1)
	  i2			= residue (ir,2)
	  res			= ats_res (i1)
	  nres			= ats_nres (i1)
	  chain			= ats_chain (i1)
	  indr		= 1
	  CALL	res_assign (res, indr, not_here)
C??giggs - dont need this we may well have non-aa residue
C??giggs  IF (not_here) STOP 'not_here/rots_gen'
C??	  IF ((excl_this).OR.(not_here).OR.(sc_chi(indr).EQ.0)) THEN
	  IF ((fix_residue(ir)) .OR. (sc_chi(indr).EQ.0)) THEN	! fixed

	    DO 300 ia		= i1, i2
	      if		= if + 1
	      DO 280 n		= 1, 3
		fixed (if,n)	= ats_xyz (ia,n)
280	      CONTINUE						! End n
	      fixed (if,4)	= ats_rad (ia)
	      fixed_ind (if,1)	= ir
	      fixed_ind (if,2)	= 0	! no moving parts/rot anchor atoms
300	    CONTINUE						! End ia

	  ELSE							! moving ats

	    DO 340 inda		= 1, n_sc_ats (indr)
	      DO 330 ia		= i1, i2
		IF (sc_ats(indr,inda).EQ.ats_atyp(ia)) THEN
		  DO 320 n	= 1, 3
		    sc_xyz (inda+3,n)	= ats_xyz (ia,n)
320		  CONTINUE					! End n
		  sc_rad (inda+3)	= ats_rad (ia)
		  sc_ia (inda+3)	= ia
		  GOTO 340
		END IF
330	      CONTINUE						! End ia

C ?? this loop above is the key (I think) wrt requiring all atoms in file
C ?? - which should be ok in sc_gen, since we have added them - hopefully

	      WRITE (4,2500) res, nres, sc_ats (indr,inda)
	      STOP 'sc atom missing/rots_gen'
340	    CONTINUE						! End inda
	    DO 380 ia		= i1, i2
	      DO 360 inda	= 1, n_sc_ats (indr)
		IF (ats_atyp(ia).EQ.sc_ats(indr,inda)) GOTO 380
360	      CONTINUE						! End inda
	      if		= if + 1
	      DO 370 n		= 1, 3
		fixed (if,n)	= ats_xyz (ia,n)
370	      CONTINUE						! End n
	      fixed (if,4)	= ats_rad (ia)
	      fixed_ind (if,1)	= ir
C ?? look vs chi_start(indr,?) slots - assume this is fixed link to rots
	      IF      (ats_atyp (ia) .EQ. chi_start (indr,3)) THEN
		fixed_ind (if,2) = -1
	      ELSE IF (ats_atyp (ia) .EQ. chi_start (indr,2)) THEN
		fixed_ind (if,2) = -2
	      ELSE IF (ats_atyp (ia) .EQ. chi_start (indr,1)) THEN
		fixed_ind (if,2) = -3
C ?? - fix to put the carbonyl C in - NOT O which can move under phi,psi
	      ELSE IF (ats_atyp (ia) .EQ. 'C   ') THEN
		fixed_ind (if,2) = -3
	      ELSE
		fixed_ind (if,2) = 0
	      END IF
380	    CONTINUE						! End ia
	    DO 400 is		= 1, 3
	      DO 390 ia		= i1, i2
		IF (chi_start(indr,is).EQ.ats_atyp(ia)) THEN
		  DO 386 n	= 1, 3
		    sc_xyz (is,n) = ats_xyz (ia,n)
386		  CONTINUE					! End n
		  sc_rad (is)	= ats_rad (ia)
		  sc_ia (is)	= ia
		  GOTO 400
		END IF
390	      CONTINUE						! End ia
C ?? For amino acids assume that we may be looking for is=1 in NT of res-1 ??
	      IF (is .EQ. 1) THEN
		IF (ir .EQ. 1) THEN
		  STOP 'is=1/ir=1'
		ELSE
		  DO 394 iap	= residue(ir-1,1), residue(ir-1,2)
		    IF (ats_atyp (iap) .EQ. 'NT  ') THEN
		      DO 392 n	= 1, 3
			sc_xyz (is,n) = ats_xyz (iap,n)
392		      CONTINUE					! End n
		      sc_rad (is) = ats_rad (iap)
		      sc_ia (is)  = ia
		      GOTO 400
		    END IF
394		  CONTINUE
		  STOP 'is.eq.1 not matched'
		END IF
	      ELSE
		STOP 'is.ne.1 not matched'
	      END IF
400	    CONTINUE						! End is
	    sc_num		= 3 + n_sc_ats (indr)
	    res_rots_ind (ir,1)	= res_nrot (indr)
	    res_rots_ind (ir,2)	= n_sc_ats (indr)
	    res_rots_ind (ir,3)	= im + 1
	    CALL	rotamerise (sc_xyz, sc_rad, sc_num, sc_ia,
     1				    ir, indr, comp_residue)
	    res_rots_ind (ir,4)	= im
	    n_test1	= res_rots_ind(ir,4) - res_rots_ind(ir,3) + 1
	    IF (n_test1 .LT. 1) STOP 'n_test'
	    IF (incl_exp (ir)) THEN
	      n_test2	= (1+res_rots_ind(ir,1))*res_rots_ind(ir,2)
	    ELSE
	      n_test2	=    res_rots_ind(ir,1)*res_rots_ind(ir,2)
	    END IF
	    IF (n_test1 .NE. n_test2) STOP 'ntest1/2'
	    imh			= res_rots_ind (ir,3) - 1
	    IF (incl_exp (ir)) THEN
	      nroth		= 1+res_rots_ind (ir,1)
	    ELSE
	      nroth		=   res_rots_ind (ir,1)
	    END IF
	    imh			= res_rots_ind (ir,3) - 1
	    DO 440 iroth	= 1, nroth
	      DO 430 iath	= 1, res_rots_ind (ir,2)
		imh		= imh + 1
		IF (imh .GT. res_rots_ind(ir,4)) STOP 'imh'
		moved_ind (imh,1) = ir
		moved_ind (imh,2) = iroth
		moved_ind (imh,3) = sc_rank (indr,iath)
430	      CONTINUE						! End iath
440	    CONTINUE						! End iroth

	  END IF
C?? henman - no idea if above will work for cb setting sc atoms (beyond Cb)

500	CONTINUE						! End ir

	n_fixed		= if
	n_moved		= im

C??giggs - aa only? remove?
C ?? Just want to check the fixed_ind array?

	IF (n_fixed .EQ. 0) STOP 'n_fixed.eq.0'
	ir_last		= fixed_ind (1,1)
	itot		= fixed_ind (1,2)
	DO 540 ifh	= 2, n_fixed
	  ir		= fixed_ind (ifh,1)
	  iflag		= fixed_ind (ifh,2)
	  IF (ir .EQ. ir_last) THEN
	    itot	= itot + iflag
	  ELSE
	    IF (res_rots_ind (ir_last,1) .EQ. -999) THEN
	      IF (itot .NE. 0) STOP 'itot.ne.0'
	    ELSE
	      IF ((itot.NE.-6).AND.(itot.NE.-3).AND.
     1		  (itot.NE.-9)) STOP 'itot.ne./-9-6/-3'
	    END IF
	    ir_last	= ir
	    itot	= iflag
	  END IF
540	CONTINUE						! End ifh
	IF (res_rots_ind (ir_last,1) .EQ. -999) THEN
	  IF (itot .NE. 0) STOP 'itot.ne.0'
	ELSE
	  IF ((itot.NE.-6).AND.(itot.NE.-3).AND.
     1	      (itot.NE.-9)) STOP 'itot.ne./-9-6/-3'
	END IF

C Option to write out rotamers for one residue, with nres adjusted to
C give original nres and rotamer number.
C??henman - is this what we want writing out? - not really

	IF (rot_coords) THEN
	  DO 640 ir		= 1, num_res
	    i1			= residue (ir,1)
	    nres		= ats_nres (i1)
	    IF (nres .EQ. nres_coords) THEN
	      im1		= res_rots_ind (ir,3)
	      im2		= res_rots_ind (ir,4)
	      IF (incl_exp (ir)) THEN
		nrotsh		= 1 + res_rots_ind (ir,1)
	      ELSE
		nrotsh		= res_rots_ind (ir,1)
	      END IF
	      natsh		= res_rots_ind (ir,2)
	      OPEN (UNIT=1, NAME='sc_test1.pdb', TYPE='NEW')
	      WRITE (1,2400) nres_coords, nrotsh
	      DO 620 ifh	= 1, n_fixed
		irh		= fixed_ind (ifh,1)
		IF (irh.EQ.ir) THEN
		  WRITE (1,2420) 1, (fixed (ifh,n),n=1,3)
		END IF
620	      CONTINUE						! End ifh
	      DO 630 iroth	= 1, nrotsh
		DO 625 iath	= 1, natsh
		  imh		= im1 + natsh*(iroth-1) + iath-1
		  IF ((imh.LT.im1).OR.(imh.GT.im2)) STOP 'imh oub'
		  WRITE (1,2420) iroth, (moved (imh,n),n=1,3)
625		CONTINUE					! End iath
630	      CONTINUE						! End iroth
	      GOTO 650
	    END IF
640	  CONTINUE						! End ir
	  STOP 'nres_coords nf'
	END IF

650	CLOSE (UNIT=1)

	RETURN
	END


	SUBROUTINE	rotamerise (sc_xyz, sc_rad, sc_num, sc_ia,
C	**********	**********
     1				    ir, indr, comp_residue)

C Make the rotamers, and add to the moving atom arrays.

	REAL		res_chi (20,50,5)
	INTEGER		res_nchi (20), res_natm (20), res_nrot (20)
	CHARACTER*4	fix_type, res_tag (20)

	INTEGER		n_sc_ats (20), chi_ats (20,5,4), sc_chi (20)
	CHARACTER*4	sc_ats (20,10), chi_start (20,3)
	INTEGER*2	sc_rank (20,10)

	REAL		sc_xyz (15,3), sc_rad (15)
	INTEGER		sc_num, sc_ia (15)

	REAL		fixed (70000,4), moved (700000,4)
	INTEGER		res_rots_ind (10000,4)
	INTEGER*2	moved_ind (700000,3), fixed_ind (70000,2)

	INTEGER		m (4)
	REAL		a_ax (3), b_ax (3), new_axes (3,3),
     1			DC_1 (3,3), DC_2 (3,3), rot_mat (3,3),
     2			xyz (3), xyz_new (3), b_axu (3),
     3			sc_xyzp (15,3)
	LOGICAL		singular, rem_oddmc, comp_residue (10000),
     1			incl_exp (10000)

	COMMON	/rot_dict/	res_nchi, res_natm, res_chi,
     1				res_nrot, res_tag
	COMMON	/rot_scs/	n_sc_ats, sc_ats, chi_ats, sc_chi,
     1				chi_start, sc_rank
	COMMON	/ats_loads/	fixed, fixed_ind, moved, moved_ind,
     1				if, im, res_rots_ind, n_fixed,n_moved,
     2				num_residues
	COMMON	/rot_mats/	DC_1, DC_2, rot_mat
	COMMON	/experiment/	fix_type, rem_oddmc
	COMMON	/incl_exprot/	incl_exp

C The first three atoms of sc_xyz/sc_rad have already been stored as fixed.
C Atoms 4->sc_num will be multiplied by sc_chi (=res_nchi) chi angles to give
C res_nrot rotamers, which will be stored in the moved arrays.
C sc_ia gives the index in the original atoms arrays of each sc atom.
C ir is the residue index from the original coords read.
C indr is the residue type index in the rotamer arrays.
C For each rotamer use a separate sc_xyz(p) array, so that each rotamer
C starts fresh with the original coords - avoiding cumulative errors.

C ?? The roll_point code should be checked - but prelim. look shows that
C ?? we should be getting clockwise rotn looking out along sc, for +ve
C ?? chi_move - e.g. consider if b_ax were already along x.

	IF (res_nrot (indr) .EQ. 0) STOP 'res_nrot 0'
	DO 500 irot		= 1, res_nrot (indr)
	  IF (sc_chi (indr) .EQ. 0) STOP 'sc chi 0'
	  IF (sc_chi(indr) .NE. res_nchi(indr)) STOP 'chi no mism'
	  DO 205 i		= 1, 15
	    DO 205 n		= 1, 3
	      sc_xyzp (i,n)	= sc_xyz (i,n)
205	  CONTINUE						! End n,i

	  DO 400 ichi		= 1, res_nchi (indr)
	    DO 210 i		= 1, 4
	      m (i)		= 3 + chi_ats (indr,ichi,i)
	      IF (m (i) .GT. sc_num) STOP 'm/sc_num'
210	    CONTINUE						! End i

	    CALL	start_chi (sc_xyzp, m, chi_curr)
	    chi_ang		= res_chi (indr,irot,ichi)
	    chi_move		= chi_ang - chi_curr

C ?? nb chi_move can be bigger than 180 - don't think this is a problem
	    DO 220 n		= 1, 3
	      a_ax (n)		= sc_xyzp(m(2),n)
	      b_ax (n)		= sc_xyzp(m(3),n)-sc_xyzp(m(2),n)
	      CALL	vec_normalise (b_ax, b_axm, b_axu)
220	    CONTINUE						! End n

C Apply the chi rotation about r_axis = a_ax + lambda*b_axu.
C Get the direction cosine matrices that depend upon the roll-axis.

	    CALL new_axes_gen (b_axu, new_axes)
	    CALL DC_get (new_axes)
C ?? Patriots : for DC matrix, inverse is the transpose ?
C ?? Patriots : I do not think so : use the inversion routine
C ??	    CALL mat_transpose
	    CALL INVERT3 (DC_1, DC_2, determinant, singular)
	    IF ( singular ) STOP 'singular DC inverse'
	    ang_deg		= chi_move
	    CALL rot_mat_x (ang_deg, rot_mat)

	    DO 300 iat		= m (4), sc_num
	      DO 240 n		= 1, 3
		xyz (n)		= sc_xyzp (iat,n)
240	      CONTINUE						! End n
	      CALL roll_point (xyz,xyz_new,a_ax)
	      DO 260 n		= 1, 3
		sc_xyzp (iat,n)	= xyz_new (n)
260	      CONTINUE						! End n
300	    CONTINUE						! End iat
400	  CONTINUE						! End ichi

C ?? - CHECK - WE HAD THIS WRONG - OK NOW ? - AND iat ALWAYS start at 4 ?
	  DO 460 iat		= 4, sc_num
	    im			= im + 1
	    IF (im .GT. 700000) STOP 'im/700000'
	    DO 440 n		= 1, 3
	      moved (im,n)	= sc_xyzp (iat,n)
440	    CONTINUE						! End n
	    moved (im,4)	= sc_rad (iat)
460	  CONTINUE						! End iat

500	CONTINUE						! End irot

C Add in the coord input rotamer, if the res was complete originally.
C ?? I think that sc_xyz should be the original ats xyz for a complete sc.

	IF (comp_residue (ir)) THEN
	  incl_exp (ir)		= .TRUE.
	  DO 540 iat		= 4, sc_num
	    im			= im + 1
	    IF (im .GT. 700000) STOP 'im/700000'
	    DO 520 n		= 1, 3
	      moved (im,n)	= sc_xyz (iat,n)
520	    CONTINUE						! End n
	    moved (im,4)	= sc_rad (iat)
540	  CONTINUE						! End iat
	END IF


	RETURN
	END


	SUBROUTINE	start_chi (sc_xyzp, m, chi_curr)
C	**********	*********

C Work out the current chi value by projecting the end bonds onto a plane
C perpendicular to the central bond of the dihedral - see notes.

C Aug 2001 - we seem to have a problem here that may be from extended chain
C chi's where we get -180 and 180 - check for this.

	REAL		sc_xyzp (15,3), r_12 (3), r_23 (3),
     1			r_23u (3), r_34 (3), r_y (3), r_2y (3),
     2			r_x (3), r_3x (3), r_2yu (3), r_3xu (3),
     3			cross (3)
	REAL		mr_12 (3), mr_23 (3), norm_123 (3),
     1			norm_234 (3), norm_123u (3), norm_234u (3),
     2			norm_123m, norm_234m
	INTEGER		m (4)

	DO 200 n	= 1, 3
	  r_12 (n)	= sc_xyzp (m(2),n) - sc_xyzp (m(1),n)
	  r_23 (n)	= sc_xyzp (m(3),n) - sc_xyzp (m(2),n)
	  r_34 (n)	= sc_xyzp (m(4),n) - sc_xyzp (m(3),n)
200	CONTINUE						! End n
	CALL	dot_prod (r_12, r_23, dot_12_23)
	CALL	dot_prod (r_23, r_34, dot_23_34)
	CALL	vec_normalise (r_23, r_23m, r_23u)
	DO 300 n	= 1, 3
	  r_y (n)	= sc_xyzp(m(1),n)+(dot_12_23/r_23m)*r_23u(n)
	  r_2y (n)	= r_y (n) - sc_xyzp (m(2),n)
	  r_x (n)	= sc_xyzp(m(4),n)-(dot_23_34/r_23m)*r_23u(n)
	  r_3x (n)	= r_x (n) - sc_xyzp (m(3),n)
300	CONTINUE						! End n
	CALL	dot_prod (r_2y, r_3x, dot_2y_3x)
	CALL	vec_normalise (r_2y, r_2ym, r_2yu)
	CALL	vec_normalise (r_3x, r_3xm, r_3xu)
	cos_ang		= dot_2y_3x / (r_2ym*r_3xm)

C We want ang back between -pi and +pi, positive for 3x clockwise from 2y.
C Do this by comparing an y*x cross-product with r_23.
C ACOSD will return just 0 to 180 degrees, and arg -1 -> +1 of course.

	IF (cos_ang .LT. -1.0) THEN
	  IF (cos_ang .LT. -1.00001) THEN
	    STOP 'cos_ang/-1'
	  ELSE
	    cos_ang	= -1.0
	  END IF
	END IF
	IF (cos_ang .GT. 1.0) THEN
	  IF (cos_ang .GT. 1.00001) THEN
	    STOP 'cos_ang/+1'
	  ELSE
	    cos_ang	= 1.0
	  END IF
	END IF

Clnx	ang		= ACOSD (cos_ang)
	ang_rads	= ACOS (cos_ang)
	ang		= (ang_rads*180.)/3.14159

	CALL	cross_product (r_2y, r_3x, cross)
	CALL	dot_prod (cross, r_23, dot_test)
	IF 	(dot_test .GT. 0.) THEN
	  chi_curr	=  ang
	ELSE IF (dot_test .LT. 0.) THEN
	  chi_curr	= -ang
	ELSE
C??	  IF      (ang .EQ. 0.) THEN
	  IF	  (ABS(ang) .LT. 0.1) THEN
	    chi_curr	= 0.
C??	  ELSE IF (ang .EQ. -180.) THEN
	  ELSE IF (ABS(ang+180.) .LT. 0.1) THEN
	    chi_curr	= -180.
C??	  ELSE IF (ang .EQ. 180.) THEN
	  ELSE IF (ABS(ang-180.) .LT. 0.1) THEN
	    chi_curr	= 180.
	  ELSE
	    STOP 'dot_test start_chi 1'
	  END IF
	END IF

C ?? I'm really sorry about this - for now, check 2 ways give same result.
C Just the angle between 2 planes, with an add-on direction test.

	DO 500 n	= 1, 3
	  mr_12 (n)	= -r_12 (n)
	  mr_23 (n)	= -r_23 (n)
500	CONTINUE						! End n
	CALL	cross_product (mr_12, r_23, norm_123)
	CALL	cross_product (mr_23, r_34, norm_234)
	CALL	dot_prod (norm_123, norm_234, dot_norms)
	CALL	vec_normalise (norm_123, norm_123m, norm_123u)
	CALL	vec_normalise (norm_234, norm_234m, norm_234u)
	cos_ang		= dot_norms / (norm_123m*norm_234m)
	IF (cos_ang .LT. -1.0) THEN
	  IF (cos_ang .LT. -1.00001) THEN
	    STOP 'cos_ang/-1'
	  ELSE
	    cos_ang	= -1.0
	  END IF
	END IF
	IF (cos_ang .GT. 1.0) THEN
	  IF (cos_ang .GT. 1.00001) THEN
	    STOP 'cos_ang/+1'
	  ELSE
	    cos_ang	= 1.0
	  END IF
	END IF
Clnx	ang		= ACOSD (cos_ang)
	ang_rads	= ACOS (cos_ang)
	ang		= (ang_rads*180.)/3.14159
	CALL	cross_product (norm_123, norm_234, cross)
	CALL	dot_prod (cross, r_23, dot_test)
	IF 	(dot_test .GT. 0.) THEN
	  chi_curr1	=  ang
	ELSE IF (dot_test .LT. 0.) THEN
	  chi_curr1	= -ang
	ELSE
C??	  IF      (ang .EQ. 0.) THEN
	  IF 	  (ABS (ang) .LT. 0.1) THEN
	    chi_curr1	= 0.
C??	  ELSE IF (ang .EQ. -180.) THEN
	  ELSE IF (ABS(ang+180.) .LT. 0.1) THEN
	    chi_curr1	= -180.
C??	  ELSE IF (ang .EQ. 180.) THEN
	  ELSE IF (ABS(ang-180.) .LT. 0.1) THEN
	    chi_curr1	= 180.
	  ELSE
C??	WRITE (4,3478) ang
C??3478	FORMAT (' ang (not 0, +/- 180) = ',E14.6)
	    STOP 'dot_test start_chi 2'
	  END IF
	END IF

	IF (ABS(chi_curr-chi_curr1) .GT. 0.1) THEN
	  chi_tmp = chi_curr - 360.
	  IF (ABS(chi_tmp-chi_curr1) .GT. 0.1) THEN
	    chi_tmp = chi_curr + 360.
	    IF (ABS(chi_tmp-chi_curr1) .GT. 0.1) THEN
	      WRITE (4,1212) chi_curr, chi_curr1
1212	      FORMAT (' ',' chi_curr, chi_curr1 = ',2F6.1)
	      STOP 'chi"s out'
	    END IF
	  END IF
	END IF

	RETURN
	END


	SUBROUTINE	new_axes_gen (R, new_axes)
C	**********	************

C This routine works out a r-h axis system s.t. the point R on regular
C (xyz) system is transformed to (mod(R),0,0) in the new system.
C new_axes (3,3) records the components of the new system in (xyz).
C ?? See JW notes.

	REAL		R (3), new_axes (3,3),
     1			mu_jw, lambda_jw, A (3), B (3), C (3)

C First the new x axis simply is the normalised R vector on old (xyz).
C b_ax/R is already a unit vector - skip vec_normalise.

C	CALL vec_normalise (R, mag_vec, unit_vec)
	DO 200 n = 1, 3
C	  new_axes (1,n)	= unit_vec (n)
	  new_axes (1,n)	= R (n)
200	CONTINUE						! End n

C Now we choose the new y to be coplanar with new x and old y, (see JW notes).

	mu_jw		= SQRT (1./(1.-new_axes(1,2)*new_axes(1,2)))
	lambda_jw	= -mu_jw * new_axes (1,2)

	new_axes (2,1)	= lambda_jw * new_axes (1,1)
	new_axes (2,2)	= lambda_jw * new_axes (1,2) + mu_jw
	new_axes (2,3)	= lambda_jw * new_axes (1,3)

C And the new z is generated so as to give a right-handed set.

	DO 400 n = 1, 3
	  A (n)			= new_axes (1,n)
	  B (n)			= new_axes (2,n)
400	CONTINUE						! End n
	CALL cross_product (A, B, C)
	DO 500 n = 1, 3
	  new_axes (3,n)	= C (n)
500	CONTINUE						! End n
	RETURN
	END


	SUBROUTINE	vec_normalise (vec, mag_vec, unit_vec)
C	**********	*************

C This routine returns magnitude and normalised vector.

	REAL		vec (3), mag_vec, unit_vec (3)

	mag_vec		= SQRT (vec (1)*vec (1) + vec (2)*vec (2) +
     1				vec (3)*vec (3) )
	DO 200 n = 1, 3
	  unit_vec (n)	= vec (n) / mag_vec
200	CONTINUE						! End n
	RETURN
	END


	SUBROUTINE	cross_product (A, B, C)
C	**********	*************

C Routine returns C = A ^ B vector cross product.

	REAL		A (3), B (3), C (3)

	C (1)	= A (2)*B (3) - A (3)*B (2)
	C (2)	= A (3)*B (1) - A (1)*B (3)
	C (3)	= A (1)*B (2) - A (2)*B (1)
	RETURN
	END


	SUBROUTINE	DC_get (new_axes)
C	**********	******

C Routine returns the direction cosine matrix (DC) that transforms coords
C between normal (xyz) system and axial system with components on normal
C (xyz) given in new_axes array.

	REAL		new_axes (3,3), DC_1 (3,3), new_ax (3),
     1			old_axes (3,3), old_ax (3), DC_2 (3,3),
     2			rot_mat (3,3)

	COMMON	/rot_mats/	DC_1, DC_2, rot_mat

	DATA	old_axes(1,1),old_axes(1,2),old_axes(1,3) /1.,0.,0./
	DATA	old_axes(2,1),old_axes(2,2),old_axes(2,3) /0.,1.,0./
	DATA	old_axes(3,1),old_axes(3,2),old_axes(3,3) /0.,0.,1./

	DO 300 l = 1, 3
	  DO 200 n = 1, 3
	    new_ax (n)		= new_axes (l,n)
200	  CONTINUE						! End n
	  DO 300 m = 1, 3
	    DO 250 n = 1, 3
	      old_ax (n)	= old_axes (m,n)
250	    CONTINUE						! End n
	    CALL dot_prod (new_ax, old_ax, dot)
	    DC_1 (l,m)		= dot
300	CONTINUE						! End m,l
	RETURN
	END


	SUBROUTINE	dot_prod (a, b, dot_product)
C	**********	********

C Routine returns the vector dot product, a.b

	REAL		a (3), b (3)

	dot_product	= a(1)*b(1) + a(2)*b(2) + a(3)*b(3)
	RETURN
	END


	SUBROUTINE	mat_transpose (A, A_T)
C	**********	*************

C Routine returns 3x3 matrix transpose of A, A_T.

	REAL		A (3,3), A_T (3,3)

	DO 300 l = 1, 3
	  DO 300 m = 1, 3
	    A_T (m,l)	= A (l,m)
300	CONTINUE						! End m,l
	RETURN
	END


	SUBROUTINE INVERT3 ( A, AINV, ADET, SING )
C	********** *******

C@@@@@@  Test this Routine  LATER !! @@@@@@

C This routine returns the matrix inverse of A in AINV, the determinant of
C A (ADET), and sets SING = .true. if A is singular.
C See J.W., p.C3, Mathematics for Chemists.

C n.b. This is programmed for a 3*3 matrix.

	REAL A (3,3), AINV (3,3), T (3,3,3), DET (3)
	LOGICAL		SING

	DO 210 I = 1, 3
	    DO 200 J = 1, 3
		T (1,I,J) = A (I,J)
200	    CONTINUE						! End J
210	CONTINUE						! End I

	DO 550 I1 = 1, 3
	    IF ( I1 .EQ. 1 ) DET (1) = 0.
	    DO 500 J1 = 1, 3
		CALL STRIKE ( T, I1, J1, 1, 3 )
		DET (2) = 0.
		DO 450 J2 = 1, 2
		    CALL STRIKE ( T, 1, J2, 2, 2 )
		    DET (3) = T (3,1,1)
		    ISN = 1
		    IF ( MOD (1+J2,2) .NE. 0 ) ISN = -1
		    SN = FLOAT (ISN)
		    DET (2) = DET (2) + SN * T (2,1,J2) * DET (3)
450		CONTINUE					! End J2
		ISN = 1
		IF ( MOD (I1+J1,2) .NE. 0 ) ISN = -1
		SN = FLOAT (ISN)
		IF ( I1 .EQ. 1 )
     1          DET (1) = DET (1) + SN * T (1,I1,J1) * DET (2)
		AINV (J1,I1) = SN * DET (2)
500	    CONTINUE						! End J1
550	CONTINUE						! End I1

	ADET = DET (1)
	SING = .FALSE.
	IF ( ADET .EQ. 0. ) SING = .TRUE.
	IF ( SING ) RETURN

	DO 610 I = 1, 3
	    DO 600 J = 1, 3
		AINV (I,J) = AINV (I,J) / ADET
600	    CONTINUE						! End J
610	CONTINUE						! End I

	RETURN
	END


	SUBROUTINE STRIKE ( T, I, J, L, N )
C	********** ******

C This routine strikes out the i row and j column of the matrix (of order N)
C in the L level of T, and inserts the result (order N-1) in the L+1 level
C of T.

	REAL T (3,3,3)

	IF ( L+1 .GT. 3 ) STOP 'T array l+1 .gt. 3'
	IVV = 0
	DO 210 IV = 1, N
	    IF ( IV .EQ. I ) GOTO 210
	    IVV = IVV + 1
	    JVV = 0
	    DO 200 JV = 1, N
		IF ( JV .EQ. J ) GOTO 200
		JVV = JVV + 1
		T (L+1,IVV,JVV) = T (L,IV,JV)
200	    CONTINUE						! End Jv
210	CONTINUE						! End Iv

	RETURN
	END


	SUBROUTINE	rot_mat_x (ang_deg, rot_mat)
C	**********	*********

C Routine returns rotation matrix for rotn of ang_deg about x axis.
C ?? Patriots : in a hurry : look this up : just definition anyway

	REAL		rot_mat (3,3)

	angle		= ang_deg
	angle_rads	= (angle*3.14159)/180.
	c_angle		= COS (angle_rads)
	s_angle		= SIN (angle_rads)
Clnx	c_angle		= COSD (angle)
Clnx	s_angle		= SIND (angle)

	rot_mat (1,1)	= 1.
	rot_mat (1,2)	= 0.
	rot_mat (1,3)	= 0.
	rot_mat (2,1)	= 0.
	rot_mat (2,2)	=  c_angle
	rot_mat (2,3)	= -s_angle
	rot_mat (3,1)	= 0.
	rot_mat (3,2)	=  s_angle
	rot_mat (3,3)	=  c_angle
	RETURN
	END


	SUBROUTINE	roll_point (xyz, xyz_new, a_ax)
C	**********	**********

C Routine performs rotation about axis that has been encoded into the
C matrices passed.
C See JW notes for the transformation, involving the dirn cosine matrices.

	REAL		xyz (3), xyz_new (3), a_ax (3), DC_1 (3,3),
     1			DC_2 (3,3), rot_mat (3,3), tmp_1 (3),
     2			tmp_2 (3), tmp_3 (3), tmp_4 (3)

	COMMON	/rot_mats/	DC_1, DC_2, rot_mat

C ?? Patriots : can we not concatenate these matrix operations?

	sign	= -1.
	CALL vec_trn		(xyz, a_ax, tmp_1, sign)
	CALL mat_vec_mult	(DC_1, tmp_1, tmp_2)
	CALL mat_vec_mult	(rot_mat, tmp_2, tmp_3)
	CALL mat_vec_mult	(DC_2, tmp_3, tmp_4)
	sign	=  1.
	CALL vec_trn		(tmp_4, a_ax, xyz_new, sign)
	RETURN
	END


	SUBROUTINE	vec_trn (A, B, C, sign)
C	**********	*******

C Vector translation, C = A + sign*B

	REAL		A (3), B (3), C (3)

	DO 200 n = 1, 3
	  C (n)		= A (n) + sign * B (n)
200	CONTINUE						! End n
	RETURN
	END


	SUBROUTINE	mat_vec_mult (A, B, C)
C	**********	************

C Routine returns C (vec) = A (mat) * B (vec)

	REAL		A (3,3), B (3), C (3)

	DO 300 l = 1, 3
	  C (l)		= 0.
	  DO 300 m = 1, 3
	    C (l)	= C (l) + A (l,m) * B (m)
300	CONTINUE						! End m,l
	RETURN
	END



	SUBROUTINE	rot_mat_z (ang_deg, rot_mat)
C	**********	*********

C Routine returns rotation matrix for rotn of ang_deg about z axis.
C ?? Patriots : in a hurry : look this up : just definition anyway

	REAL		rot_mat (3,3)

	angle		= ang_deg
Clnx	c_angle		= COSD (angle)
Clnx	s_angle		= SIND (angle)
	angle_rads	= (angle*3.14159)/180.
	c_angle		= COS (angle_rads)
	s_angle		= SIN (angle_rads)

	rot_mat (1,1)	= c_angle
	rot_mat (1,2)	= -s_angle
	rot_mat (1,3)	= 0.
	rot_mat (2,1)	= s_angle
	rot_mat (2,2)	= c_angle
	rot_mat (2,3)	= 0.
	rot_mat (3,1)	= 0.
	rot_mat (3,2)	= 0.
	rot_mat (3,3)	= 1.

	RETURN
	END


	SUBROUTINE	rot_mat_y (ang_deg, rot_mat)
C	**********	*********

C Routine returns rotation matrix for rotn of ang_deg about y axis.
C ?? Patriots : in a hurry : look this up : just definition anyway

	REAL		rot_mat (3,3)

	angle		= ang_deg
Clnx	c_angle		= COSD (angle)
Clnx	s_angle		= SIND (angle)
	angle_rads	= (angle*3.14159)/180.
	c_angle		= COS (angle_rads)
	s_angle		= SIN (angle_rads)

	rot_mat (1,1)	=  c_angle
	rot_mat (1,2)	=  0.
	rot_mat (1,3)	=  s_angle
	rot_mat (2,1)	=  0.
	rot_mat (2,2)	=  1.
	rot_mat (2,3)	=  0.
	rot_mat (3,1)	= -s_angle
	rot_mat (3,2)	=  0.
	rot_mat (3,3)	=  c_angle

	RETURN
	END


	SUBROUTINE	rots_prob
C	**********	*********

C Derive probabilities for rotamers, based on sc clashes - I think
C equivalent to a mean field approach in which a clash is strictly
C disallowed.  The cost of not using a (soft) potl fn is that we must devise
C a method to ensure that no single residue has no allowed rotamers, since
C then all probabilities go to zero in the following analysis for residue i
C and rotamer k: (where the vs results are 1-allowed or 0-clash)

C p(i,k) = {(i,k) vs fixed} * PI(over j.ne.i)[SUM[p(j,l)*{(i,k) vs (j,l)}]

C Have an iterative relaxation of vdw clashing until we get a 'solution'.

	REAL		fixed (70000,4), moved (700000,4)
	INTEGER		res_rots_ind (10000,4)
	INTEGER*2	moved_ind (700000,3), fixed_ind (70000,2)
	REAL		p_new (10000,50), p_old (10000,50),
     1			p_aswego (10000,50)
	REAL		pinc_hold (10000,50)
	REAL		xyz1 (15,3), rad1 (15), xyz (3)
	REAL		rad_out (70000), xyz_out (70000,3)
	INTEGER		ir_out (70000), iroth_out (70000),
     1			nres_out (70000)
	INTEGER		num_allowed (10000)
	CHARACTER*4	atyp_out (70000), res_out (70000)
	CHARACTER*3	code_out (70000)
	CHARACTER*1	chain_out (70000)
	CHARACTER*54	line_tmp, line_buf (100)
	LOGICAL		busted (10000,50)
	LOGICAL		brick_atoms, rot_coords, collect_clash
	LOGICAL		un_converged, clashed, found_clash,
     1			write_once, past_fix_here, rem_oddmc
	LOGICAL*1	strike_out (10000,50)
	INTEGER		clash_store (10)
	INTEGER*2	rank (15)
	LOGICAL		fix_residue (10000), comp_residue (10000)
	CHARACTER*4	res, atyp, ats_atom (70000), atom_here,
     1			fix_type, res_hear
	CHARACTER*6	atom6, atom6_out (70000)
	CHARACTER*1	chain, chain_hear
	REAL		ats_xyz (70000,3), ats_rad (70000),
     1			res_vdw (10000)
	INTEGER		ats_nres (70000), ats_class (70000),
     1			residue (10000,2)
	CHARACTER*4	ats_atyp (70000), ats_res (70000)
	CHARACTER*1	ats_chain (70000)
	LOGICAL*1	ats_comm (70000)
	CHARACTER*3	filenum
	CHARACTER*1	digi (10)
	LOGICAL		fix_aswego, set_aswego (10000),
     1			psum_zero (10000), any_psum_zero, new_busted,
     2			incl_exp (10000), prob_high, clash_relief,
     3			relax_to_val, clash_info, exptal_i,exptal_j,
     4			on_expt (70000), on_expt_here, aa_output,
     5			res_sorted (10000)

	COMMON	/ats/		num_ats, ats_xyz, ats_res,
     1				ats_nres, ats_chain, ats_atyp,
     2				ats_rad, ats_class, residue,
     3				num_res, ats_comm, ats_atom,
     4				fix_residue, comp_residue
	COMMON	/ats_loads/	fixed, fixed_ind, moved, moved_ind,
     1				if, im, res_rots_ind, n_fixed,n_moved,
     2				num_residues
	COMMON	/ctr_clash/	brick_atoms, vdw_dec, vdw_relax,
     1				conv_frac, collect_clash
	COMMON	/testit/	rot_coords, nres_coords
	COMMON	/probs/		p_new, p_old, frac_new
	COMMON	/experiment/	fix_type, rem_oddmc
	COMMON	/numats/	numats_in, numats_out, numats_add,
     1				numats_ovr, numcb_add, numats_rem
	COMMON	/fix_time/	fix_aswego
	COMMON	/incl_exprot/	incl_exp
	COMMON	/probability/	prob_high, clash_relief
	COMMON	/relax_target/	relax_to_val, target_val, aa_output
	COMMON	/clash_verbose/	clash_info

	DATA	write_once	/ .TRUE. /
	DATA	digi	/'0','1','2','3','4','5','6','7','8','9'/

2200	FORMAT (/,' Into vdw_dec/iter loops for ',I5,' residues')
2220	FORMAT (/,'    vdw_dec loop: i_dec,vdw_relax = ',I3,2X,F6.2,/)
2300	FORMAT (' Unconverged - num_iter, prob_rmsd, n_osc = ',
     1 I5,2X,F12.7,2X,I5)
2360	FORMAT (' ','Dummy atoms rotamer file from sc_freedom')
2370	FORMAT ('ATOM',9X,A4,A4,A1,I4,4X,3F8.3)
2380	FORMAT (A6,7X,A4,A4,A1,I4,4X,3F8.3)
2400	FORMAT ('   Converged - num_iter,rmsd,conv = ',I5,2(2X,F12.7))
2420	FORMAT (/,' The VdW relaxation at convergence = ',F6.2,' A')
2510	FORMAT (' ','zero probs for ir"s = ',10(1X,I4))
2530	FORMAT (' ','zero probs for ',I4,' residues')
2600	FORMAT (/,' *** LIST - FIX residues from scf_alter.in',/)
2620	FORMAT (/,' *** COMP - FIX all complete sidechains',/)
2640	FORMAT (/,' *** NONE - FIX none',/)
2810	FORMAT ('*** clash-free at vdw_relax = ',F5.1,' but
     1 continue to vdw target = ',F5.1)
2820	FORMAT (/,' sc_gen.out written for ',I4,' residues, of which:',
     1 /,24X,I4,' were fixed',/,24X,I4,' were moving and',
     2 /,24X,I4,' of the movers were PAG - not really moving')
2840	FORMAT (/,'In terms of atoms we have ',I5,' in and ',I5,' out')
2860	FORMAT (' ','of o/p atoms ',I5,' were added ',I5,' ovrwritten')
2870	FORMAT (' ','noting that  ',I5,' were removed: CA/N/C missing')
2880	FORMAT (' ',' And ',I5,' CB atoms were added')
2900	FORMAT (' ','*** .GT. 200 iterations: skip to next i_dec ***')
3100	FORMAT (/,' *** SELECTED ROTAMERS: HIGH or LOW PROBABILITY
     1 WHEN FIRST PACKED')
3200	FORMAT (/,' *** SELECTED ROTAMERS: HIGH or LOW PROBABILITY
     1 AT FINAL PACKING')
4000	FORMAT (A1,1X,I4,1X,A4,' fails at vdw_relax/iter = ',F5.1,I3)
4100	FORMAT ('MODEL',I9)
4200	FORMAT ('ENDMDL')
4300	FORMAT ('RESIDUE ',A1,1X,I4,1X,A4)
C??prob4400	FORMAT (A4,1X,F7.5,1X,A4,A4,A1,I4,4X,3F8.3)
4400	FORMAT (A4,1X,7X,1X,A4,A4,A1,I4,4X,3F8.3)
4500	FORMAT (A54)
4600	FORMAT ('RESIDUE ',A1,1X,I4,1X,A4,' nrot/prb = ',I3,1X,F8.6)
4800	FORMAT ('> ',A1,2X,I5,2X,A4,2X,F6.2)

	IF (fix_aswego .AND. collect_clash) STOP 'fix_aswego/clash'
	IF (aa_output .AND. collect_clash) STOP 'aa_output/clash'

	IF (clash_info) THEN
	  OPEN (UNIT=7, NAME='scg_clashes.out', TYPE='NEW')
	END IF

	DO 120 ir		= 1, num_residues
	    IF (res_rots_ind (ir,1) .NE. -999) GOTO 130
120	CONTINUE						! End ir
	WRITE (4,*) '*** All scs COMPLETE - O/P coords same as I/P'
	GOTO 710

130	CONTINUE

C Brick all the fixed and moved atoms to aid the clashing calculation -
C ?? test how much time will this save (from ctr option to brick or not).

	IF (brick_atoms) THEN
	  CALL brick_em
	END IF

	IF (aa_output) THEN
	  DO 136 ir		= 1, num_residues
	    res_sorted (ir)	= .FALSE.
136	  CONTINUE						! End ir
	END IF

	IF (fix_aswego) THEN
	  WRITE (4,3100)
	  DO 150 ir		= 1, num_residues
	    set_aswego (ir)	= .FALSE.
	    IF (res_rots_ind (ir,1) .NE. -999) THEN
	      IF (incl_exp (ir)) THEN
		nrots		= 1 + res_rots_ind (ir,1)
	      ELSE
		nrots		=     res_rots_ind (ir,1)
	      END IF
	      IF ((nrots.GE.1).AND.(nrots.LE.50)) THEN
		DO 140 irot	= 1, nrots
		  p_aswego (ir,irot) = 0.
140		CONTINUE					! End irot
	      ELSE
		STOP 'n_here/rots'
	      END IF
	    END IF
150	  CONTINUE						! End ir
	ELSE
	  WRITE (4,3200)
	END IF

	IF      (fix_type .EQ. 'LIST') THEN
	  WRITE (4,2600)
	ELSE IF (fix_type .EQ. 'COMP') THEN
	  WRITE (4,2620)
	ELSE IF (fix_type .EQ. 'NONE') THEN
	  WRITE (4,2640)
	ELSE
	  STOP 'undefined fix_type'
	END IF

C For each value of the relaxation of vdw clashing parameter, cycle through
C prob array updating, until all rots at zero prob. residue is found (skip
C to relaxation) or convergence.  Initialise probs for each dec loop.

	IF (collect_clash) THEN
	  found_clash 	= .FALSE.
	  n_store	= 0
	END IF
	WRITE (4,2200) num_residues
	DO 600 i_dec		= 1, 100
	  vdw_relax		= vdw_dec*FLOAT(i_dec-1)
	  WRITE (4,2220) i_dec, vdw_relax

	  un_converged		= .TRUE.
	  num_iter		= 0
	  DO 200 ir		= 1, num_residues
	    IF (res_rots_ind (ir,1) .NE. -999) THEN
	      IF (incl_exp (ir)) THEN
		nrots		= 1 + res_rots_ind (ir,1)
	      ELSE
		nrots		=     res_rots_ind (ir,1)
	      END IF
	      IF ((nrots.GE.1).AND.(nrots.LE.50)) THEN
		DO 180 irot	= 1, nrots
		  p_old (ir,irot) = 1. / FLOAT (nrots)
180		CONTINUE					! End irot
	      ELSE
		STOP 'n_here/rots'
	      END IF
	    END IF
200	  CONTINUE						! End ir

	  DO WHILE (un_converged)

	    num_iter		= num_iter + 1

C?? July 2004 - problem with some runs not coverging - skip to next i_dec
	    IF (num_iter .GT. 200) THEN
	      WRITE (4,2900)
	      GOTO 600
	    END IF
C?? July 2004

	    n_osc		= 0
	    any_psum_zero	= .FALSE.
	    DO 500 ir		= 1, num_residues
	      IF (res_rots_ind (ir,1) .NE. -999) THEN
	        IF (incl_exp (ir)) THEN
		  nrots		= 1 + res_rots_ind (ir,1)
	        ELSE
		  nrots		=     res_rots_ind (ir,1)
	        END IF
		past_fix_here	= .FALSE.

		DO 400 irot	= 1, nrots
		  nats		= res_rots_ind (ir,2)
		  im1		= res_rots_ind (ir,3) + nats*(irot-1)
		  im2		= im1 + nats - 1
		  IF (irot .EQ. nrots) THEN
		    IF (im2 .NE. res_rots_ind (ir,4)) STOP 'im2.ne.4'
		  ELSE
		    IF (im2 .GE. res_rots_ind (ir,4)) STOP 'im2.ge.4'
		  END IF

		  DO 300 im	= im1, im2
		    ia		= im - im1 + 1
		    DO 230 n	= 1, 3
		      xyz1 (ia,n) = moved (im,n)
230		    CONTINUE					! End n
		    rad1 (ia)	= moved (im,4)
		    rank (ia)	= moved_ind (im,3)
300		  CONTINUE					! End im

		  CALL	vs_fixed (ir,xyz1,rad1,rank,nats, clashed)
		  IF (clashed) THEN
		    p_new (ir,irot)	= 0.
		  ELSE
		    past_fix_here	= .TRUE.
		    pmult		= 1.
		    CALL vs_moved (ir,xyz1,rad1,nats, strike_out)
		    DO 340 jr		= 1, num_residues
		      IF (jr .EQ. ir) GOTO 340
		      IF (incl_exp (jr)) THEN
			mrots		= 1 + res_rots_ind (jr,1)
		      ELSE
			mrots		=     res_rots_ind (jr,1)
		      END IF
		      IF (res_rots_ind (jr,1) .NE. -999) THEN
			psum		= 0.
			DO 320 jrot	= 1, mrots
			  IF (.NOT. strike_out (jr,jrot)) THEN
			    psum	= psum + p_old (jr,jrot)
			  END IF
320			CONTINUE				! End jrot
			pmult		= pmult*psum
		      END IF
340		    CONTINUE					! End jr
		    p_new (ir,irot)	= pmult
		  END IF
400		CONTINUE					! End irot

		psum_ir			= 0.
		DO 440 irot		= 1, nrots
		  psum_ir		= psum_ir+p_new (ir,irot)
440		CONTINUE					! End irot
		IF (psum_ir .EQ. 0.) THEN
C ?? - maybe we should have an option to keep psum=0 scs in somehow?
C ?? - rather than just heading for the next vdw decrement

		 IF (clash_info) THEN
		  i1_hear	= residue (ir,1)
		  nres_hear	= ats_nres (i1_hear)
		  chain_hear	= ats_chain (i1_hear)
		  res_hear	= ats_res (i1_hear)
		  WRITE (7,4000) chain_hear,nres_hear,res_hear,
     1				  vdw_relax, num_iter
		 END IF

		 psum_zero (ir)		= .TRUE.
		 any_psum_zero		= .TRUE.
		 IF (.NOT.fix_aswego) THEN
		  IF (.NOT.collect_clash) THEN
		   IF (.NOT. aa_output) THEN
		    GOTO 600
		   END IF
		  ELSE
		    found_clash		= .TRUE.
		    n_store		= n_store + 1
		    IF (n_store .LE. 10) THEN
C ?? store -ve for clash moving/fix and +ve for moving/moving
		      IF (past_fix_here) THEN
			clash_store (n_store) =  ir
		      ELSE
			clash_store (n_store) = -ir
		      END IF
		    END IF
		    DO 450 irot		= 1, nrots
		      p_new (ir,irot)	= 0.
450		    CONTINUE					! End irot
		  END IF
		 END IF						! End aswego
		ELSE
		  psum_zero (ir)	= .FALSE.
		  DO 460 irot		= 1, nrots
		    p_new (ir,irot)	= p_new (ir,irot) / psum_ir
460		  CONTINUE					! End irot
		END IF

	      END IF						! EndIf rots+
500	    CONTINUE						! End ir

	    IF (aa_output) THEN		! record VdW for aa first packing
	      DO 503 ir		= 1, num_residues
	        IF ( (.NOT.psum_zero(ir)).AND.
     1		     (.NOT.res_sorted(ir)) ) THEN
		  res_sorted (ir)	= .TRUE.
		  res_vdw (ir)		= vdw_relax
	        END IF
503	      CONTINUE						! End ir
	    END IF

C We may also be here with a clash, if collecting info. on numbers clashing.
C Above a certain vdw_relax level (1 A say), when we get to less than
C 10 clashed ir's, write out these rotamers - we shouldn't be here.
C We will not have collect_clash=T if fix_aswego=T.

	    IF (collect_clash) THEN
	      IF (found_clash) THEN
		IF (n_store .LE. 10) THEN
		  WRITE (4,2510) (clash_store(ist),ist=1,n_store)
		  IF (write_once) THEN
		    IF (vdw_relax .GT. 1.0) THEN
		      OPEN (UNIT=1,NAME='sc_nasties.pdb',TYPE='NEW')
		      WRITE (1,2360)
		      DO 530 i_store	= 1, n_store
			irh		= clash_store (i_store)
			imh1		= res_rots_ind (irh,3)
			imh2		= res_rots_ind (irh,4)
			DO 520 imh	= imh1, imh2
			  nresh		= irh
			  WRITE (1,2370) nresh,(moved(imh,n),n=1,3)
520			CONTINUE				! End imh
530		      CONTINUE					! End i_store
		      CLOSE (UNIT=1)
		      write_once	= .FALSE.
		    END IF
		  END IF
		ELSE
		  WRITE (4,2530) n_store
		END IF
		n_store		= 0
		found_clash	= .FALSE.
		GOTO 600
	      END IF
	    END IF

C We could be here (Oct 05) with psum=zero, but fixing final rots as we go.

	    IF (fix_aswego) THEN
	      DO 510 ir		= 1, num_residues
	       IF ( (.NOT.psum_zero(ir)).AND.
     1		    (.NOT.set_aswego(ir)) ) THEN
	        IF (res_rots_ind (ir,1) .NE. -999) THEN
		  IF (incl_exp (ir)) THEN
		    nrots	= 1 + res_rots_ind (ir,1)
		  ELSE
		    nrots	=     res_rots_ind (ir,1)
		  END IF
		  DO 506 irot	= 1, nrots
		    p_aswego (ir,irot) = p_new (ir,irot)
506		  CONTINUE					! End irot
	        END IF
		set_aswego (ir)	= .TRUE.
	       END IF
510	      CONTINUE						! End ir
	    END IF

C Now skip to VdW decrement if we have any sidechains with total probs=zero.

	    IF (any_psum_zero) GOTO 600

C Only update p_old after a complete pass - and check on convergence.

	    sum_sq		= 0.
	    sum_num		= 0.
	    DO 560 ir		= 1, num_residues
	      IF (res_rots_ind (ir,1) .NE. -999) THEN
		IF (incl_exp (ir)) THEN
		  nrots		= 1 + res_rots_ind (ir,1)
		ELSE
		  nrots		=     res_rots_ind (ir,1)
		END IF

		DO 540 irot	= 1, nrots
		  p_new (ir,irot) = frac_new*p_new (ir,irot) +
     1				    (1.-frac_new)*p_old(ir,irot)
		  pinc		= p_new(ir,irot)-p_old(ir,irot)
C ??
		  IF (num_iter .EQ. 1) pinc_hold (ir,irot) = pinc
		  IF (num_iter .GT. 1) THEN
		    IF ((pinc*pinc_hold(ir,irot)).LT.0.) THEN
		      abs_now		= ABS (pinc)
		      abs_then		= ABS (pinc_hold(ir,irot))
		      frac		= abs_now/abs_then
		      IF ((frac.GT.0.8).AND.(frac.LT.1.2)) THEN
			n_osc		= n_osc + 1
C ??			IF (num_iter .GT. 20) THEN
C ?? - smooth it above 20 iterations - how?
C ??			END IF
		      END IF
		    END IF
		    pinc_hold (ir,irot) = pinc
		  END IF
C ??
		  sum_sq	= sum_sq + pinc*pinc
		  sum_num	= sum_num + 1.
		  p_old (ir,irot) = p_new (ir,irot)
540		CONTINUE					! End irot
	      END IF

C We can re-route here to keep going to a val, if required.

	      IF (ir .EQ. num_residues) THEN
		IF (relax_to_val) THEN
	          IF (ABS(vdw_relax-target_val) .GE. 0.01) THEN
		    WRITE (4,2810) vdw_relax, target_val
		    GOTO 600
		  END IF
		END IF
	      END IF

560	    CONTINUE						! End ir
	    rmsd_here	= SQRT (sum_sq/sum_num)
	    WRITE (4,2300) num_iter, rmsd_here, n_osc
	    IF (rmsd_here .LT. conv_frac) GOTO 700

	  END DO

600	CONTINUE						! End i_dec

700	WRITE (4,2400) num_iter, rmsd_here, conv_frac
	WRITE (4,2420) vdw_relax

710	CONTINUE

	IF (clash_relief) THEN			! coords straight out
	  vdw_tol		= vdw_relax	! tol = what we conv at
	  number_relief		= 0
	  number_busted_total	= 0
	  DO 712 ir		= 1, num_residues
	    IF (res_rots_ind (ir,1) .NE. -999) THEN
	      IF (incl_exp (ir)) THEN
		nrots		= 1 + res_rots_ind (ir,1)
	      ELSE
		nrots		=     res_rots_ind (ir,1)
	      END IF
	      IF ((nrots.GE.1).AND.(nrots.LE.50)) THEN
	        num_allowed(ir)	= 0
		DO 711 irot	= 1, nrots
		  busted (ir,irot) = .FALSE.
	          IF (fix_aswego) THEN
	           IF (.NOT. set_aswego(ir)) STOP 'set_aswego NOT set'
	           p_here	= p_aswego (ir,irot)
	          ELSE
		   p_here	= p_old (ir,irot)
	          END IF
	          IF (p_here .GT. 0.00001) THEN
		    num_allowed(ir) = num_allowed(ir) + 1
		  END IF
711		CONTINUE					! End irot
	      ELSE
		STOP 'n_here/rots'
	      END IF
	    END IF
712	  CONTINUE						! End ir
	ELSE
	  OPEN (UNIT=2, NAME='sc_gen.out', TYPE='NEW')
	  WRITE (2,*) 'HEADER coords for es calc with aa sc completion'
	END IF

713	CONTINUE				! in here for recycling
	IF (clash_relief) THEN
	  ia_out		= 0
	  number_busted		= 0
	  number_relief		= number_relief + 1
	  dist_test_min		= 0.
	  WRITE (4,*) 'starting clash relief cycle ', number_relief
	END IF

	num_fout		= 0
	num_mout		= 0
	num_PAGout		= 0
	DO 740 ir		= 1, num_res
	  i1			= residue (ir,1)
	  i2			= residue (ir,2)
	  nres			= ats_nres (i1)
	  chain			= ats_chain (i1)
	  res			= ats_res (i1)
	  atom_here		= ats_atom (i1)
	  IF     (atom_here .EQ. 'ATOM') THEN
	    atom6		= 'ATOM  '
	  ELSE IF (atom_here .EQ. 'HETA') THEN
	    atom6		= 'HETATM'
	  ELSE
	    STOP 'ats_atom not ATOM/HETA'
	  END IF
	  IF (fix_residue (ir)) THEN		! fixed atoms back out

	    num_fout		= num_fout + 1
	    DO 716 i		= i1, i2
	      atyp		= ats_atyp (i)
	      DO 714 n		= 1, 3
		xyz (n)		= ats_xyz (i,n)
714	      CONTINUE						! End n
	      IF (clash_relief) THEN
	        ia_out			= ia_out + 1
		atom6_out (ia_out)	= atom6
		atyp_out  (ia_out) 	= atyp
		res_out   (ia_out) 	= res
		chain_out (ia_out) 	= chain
		rad_out   (ia_out)	= ats_rad (i)
		nres_out  (ia_out)	= nres
		DO 715 n		= 1, 3
		  xyz_out (ia_out,n)	= xyz (n)
715		CONTINUE					! End n
		code_out  (ia_out)	= 'fix'
		ir_out    (ia_out)	= ir
		iroth_out (ia_out)	= 999			! fixed
		on_expt   (ia_out)	= .FALSE.
	      ELSE
	        WRITE (2,2380) atom6, atyp, res, chain, nres, xyz
	      END IF
716	    CONTINUE						! End i

	  ELSE			! pick highest prob rotamer and write

	    num_mout		= num_mout + 1
	    num_tmp		= 0	! first: fixed ats for this res
	    DO 720 ifh		= 1, n_fixed
	      irh		= fixed_ind (ifh,1)
	      IF (irh.EQ.ir) THEN
C?? giggs/scf_cxxc - don't know if this atyp cludge will work
	       atyp		= ats_atyp (i1+num_tmp)
	      IF (clash_relief) THEN
	        ia_out			= ia_out + 1
		atom6_out (ia_out)	= atom6
		atyp_out  (ia_out) 	= atyp
		res_out   (ia_out) 	= res
		chain_out (ia_out) 	= chain
Coopsfeb07		rad_out   (ia_out)	= ats_rad (i)
		rad_out   (ia_out)	= fixed(ifh,4)
		nres_out  (ia_out)	= nres
		DO 718 n		= 1, 3
		  xyz_out (ia_out,n)	= fixed(ifh,n)
718		CONTINUE					! End n
		code_out  (ia_out)	= 'fix'
		ir_out    (ia_out)	= ir
		iroth_out (ia_out)	= 999			! fixed
		on_expt   (ia_out)	= .FALSE.
	      ELSE
	       WRITE (2,2380) atom6, atyp, res, chain, nres,
     1			      (fixed(ifh,n),n=1,3)
	      END IF
	       num_tmp		= num_tmp + 1
	      END IF
720	    CONTINUE						! End ifh

C?? giggs - check that G,A,P come out ok
	    IF ((res.EQ.'GLY ') .OR. (res.EQ.'ALA ') .OR.
     1		(res.EQ.'PRO ')) THEN		! no moving parts
	      num_PAGout	= num_PAGout + 1
	      GOTO 740
	    END IF

	    im1			= res_rots_ind (ir,3)
	    im2			= res_rots_ind (ir,4)
	    IF (incl_exp (ir)) THEN
	      nrotsh		= 1 + res_rots_ind (ir,1)
	    ELSE
	      nrotsh		=     res_rots_ind (ir,1)
	    END IF
	    natsh		= res_rots_ind (ir,2)
	    IF (prob_high) THEN
	      p_best		= 0.0
	    ELSE
	      p_best		= 999.0
	    END IF
	    i_best		= 0
	    DO 730 iroth	= 1, nrotsh
	      IF (busted (ir,iroth)) GOTO 730		! clashed out
	      IF (fix_aswego) THEN
	        IF (.NOT. set_aswego(ir)) STOP 'set_aswego NOT set'
	        p_here		= p_aswego (ir,iroth)
	      ELSE
		p_here		= p_old (ir,iroth)
	      END IF
	      IF (p_here .GT. 0.00001) THEN
	        IF (prob_high) THEN
		  IF (p_here .GT. p_best) THEN
		    p_best	= p_here
		    i_best	= iroth
		    IF (iroth .EQ. nrotsh) THEN
		      on_expt_here	= .TRUE.
		    ELSE
		      on_expt_here	= .FALSE.
		    END IF
		  END IF
		ELSE
		  IF (p_here .LT. p_best) THEN
		    p_best	= p_here
		    i_best	= iroth
		    IF (iroth .EQ. nrotsh) THEN
		      on_expt_here	= .TRUE.
		    ELSE
		      on_expt_here	= .FALSE.
		    END IF
		  END IF
		END IF
	      END IF
730	    CONTINUE						! End iroth
	    IF (i_best .EQ. 0) THEN
	      STOP 'all rots have zero prob'
	    ELSE
	      DO 734 iath	= 1, natsh
		imh		= im1 + natsh*(i_best-1) + iath-1
		IF ((imh.LT.im1).OR.(imh.GT.im2)) STOP 'imh oub'
		atyp		= ats_atyp (i1+iath+num_tmp-1)
	      IF (clash_relief) THEN
	        ia_out			= ia_out + 1
		atom6_out (ia_out)	= atom6
		atyp_out  (ia_out) 	= atyp
		res_out   (ia_out) 	= res
		chain_out (ia_out) 	= chain
		nres_out  (ia_out)	= nres
CoopsFeb07		rad_out   (ia_out)	= ats_rad (i)
		rad_out   (ia_out)	= moved(imh,4)
		DO 732 n		= 1, 3
		  xyz_out (ia_out,n)	= moved(imh,n)
732		CONTINUE					! End n
		code_out  (ia_out)	= 'mov'
		ir_out    (ia_out)	= ir
		iroth_out (ia_out)	= i_best		! fixed
		on_expt   (ia_out)	= on_expt_here
	      ELSE
		WRITE (2,2380) atom6, atyp, res, chain,nres,
     1		      (moved(imh,n),n=1,3)
	      END IF

734	      CONTINUE					! End iath
	    END IF
C??giggs - put some prob distn info out for each moving aa?

	  END IF
740	CONTINUE						! End ir

C ** Note that we should not have to check vs fixed atoms - should have been
C ** done already in the rots_prob code (fir given vdw_dec).
C It's important to note that we will only 'bust' one rotamer of a clashing
C pair at a time - and this is for the residue with more allowed rots,
C *** unless this residue is sitting on the exptal rotamer (i.e. bias to keep).
C When we run over the second (j) loop, take only from atom i+1. i.e. we
C should get the unique half of the interaction matrix, and therefore we
C only look once at all interacting pairs. (atoms in same res/rot skipped).
C Also, with dynamic busted array update, any subsequently found clashes will
C be skipped for this cycle - since they will be different when coords updated.

	IF (clash_relief) THEN
	  new_busted			= .FALSE.
C??	  DO 760 i			= 1, ia_out
	  DO 760 i			= 1, ia_out-1	! uniq half matrix
	    IF ((code_out(i).NE.'mov').AND.
     1          (code_out(i).NE.'fix')) STOP 'code_out not fix/mov'
	    IF (code_out(i) .EQ. 'mov') THEN
	      iri			= ir_out (i)
	      iroti			= iroth_out (i)
	      IF (iroti .EQ. 999) STOP 'not a moving atomi: CR/sc_gen'
C??	      DO 750 j			= 1, ia_out
	      DO 750 j			= i+1, ia_out	! uniq half matrix
		irj			= ir_out (j)
		irotj			= iroth_out (j)
	        IF ((iri.EQ.irj).AND.(code_out(j).EQ.'mov')) GOTO 750
		IF (code_out(j).EQ.'fix') GOTO 750		! **
	        IF (irotj.EQ.999) STOP 'not a moving atomj: CR/sc_gen'
	        IF (busted(irj,irotj)) GOTO 750	! maybe busted in this round
	        IF (busted(iri,iroti)) GOTO 750	! maybe busted in this round
		dist_here = SQRT ( (xyz_out(i,1)-xyz_out(j,1))**2 +
     1				   (xyz_out(i,2)-xyz_out(j,2))**2 +
     2				   (xyz_out(i,3)-xyz_out(j,3))**2 )
		vdw_dist  = rad_out(i) + rad_out(j)
		dist_test = dist_here - vdw_dist + vdw_tol
		IF (dist_test .LT. 0.) THEN
		  IF (on_expt(i)) THEN
		    exptal_i		= .TRUE.
		  ELSE
		    exptal_i		= .FALSE.
		  END IF
		  IF (on_expt(j)) THEN
		    exptal_j		= .TRUE.
		  ELSE
		    exptal_j		= .FALSE.
		  END IF
		  IF      (exptal_i .AND. (.NOT.exptal_j)) THEN
		    ir_here		= irj
		    irot_here		= irotj
		  ELSE IF (exptal_j .AND. (.NOT.exptal_i)) THEN
		    ir_here		= iri
		    irot_here		= iroti
		  ELSE
		    IF (num_allowed(iri).GE.num_allowed(irj)) THEN
		      ir_here		= iri
		      irot_here		= iroti
		    ELSE
		      ir_here		= irj
		      irot_here		= irotj
		    END IF
		  END IF
		  IF (num_allowed(ir_here).LT.2) THEN
		    WRITE (4,*) 'BUSTING last allowed rotamer for ',
     1			        ir_here
		  END IF
		  IF (busted(ir_here,irot_here)) STOP 'CR ?? busted'
		  busted (ir_here,irot_here)	= .TRUE.
		  new_busted		= .TRUE.
		  number_busted	= number_busted + 1
		  number_busted_total	= number_busted_total + 1
		  num_allowed(ir_here) = num_allowed(ir_here) - 1

	WRITE (4,*) 'busted I/J,dist,vdw_dist,violn = ',
     1	            nres_out(i),res_out(i),atyp_out(i),
     2              nres_out(j),res_out(j),atyp_out(j),
     3		    dist_here,vdw_dist,dist_test

		  IF (dist_test .LT. dist_test_min) THEN
		    dist_test_min	= dist_test
		  END IF
C??		  IF (code_out(j) .EQ. 'mov') THEN  ! j gets its own look as i
C??		    busted (irj,irotj)	= .TRUE.
C??		    number_busted	= number_busted + 1
C??		    number_busted_total	= number_busted_total + 1
C??		  END IF
		END IF
750	      CONTINUE						! End j
	    END IF
760	  CONTINUE						! End i
	  IF (new_busted) THEN
	    WRITE (4,*) 'clash relief cycle, number_busted = ',
     1 number_relief, number_busted
	    WRITE (4,*) 'worst violn of (vdw-vdw_tol) = ',dist_test_min
	    IF (number_busted_total .GT. 1000) THEN
	      WRITE (4,*)
	      WRITE (4,*) 'SC_GEN FAILED >1000 busted rotamers'
	      WRITE (4,*)
	      CALL	EXIT
	    END IF
	    GOTO 713
	  ELSE
	    WRITE (4,*) 'PASSED CLASH RELIEF, total busted = ',
     1 number_busted_total
	    WRITE (4,*) 'PASSED CLASH RELIEF for vdw_tol = ', vdw_tol
	    WRITE (4,*)
	  OPEN (UNIT=2, NAME='sc_gen.out', TYPE='NEW')
	  WRITE (2,*) 'HEADER coords for es calc with aa sc completion'
	    DO 780 i			= 1, ia_out
	      WRITE (2,2380) atom6_out(i),atyp_out(i),res_out(i),
     1		    chain_out(i),nres_out(i),(xyz_out(i,n),n=1,3)
780	    CONTINUE						! End i
	    CLOSE (UNIT=2)
	  END IF
	ELSE
	  CLOSE (UNIT=2)
	END IF
	WRITE (4,2820) num_res, num_fout, num_mout, num_PAGout
	WRITE (4,2840) numats_in, numats_out
	WRITE (4,2860) numats_add, numats_ovr
	WRITE (4,2870) numats_rem
	WRITE (4,2880) numcb_add

	IF (clash_info) THEN
	  CLOSE (UNIT=7)
	  OPEN (UNIT=7, NAME='scg_rots.pdb', TYPE='NEW')
	  OPEN (UNIT=8, NAME='scg_probs.txt', TYPE='NEW')
	  WRITE (7,*) 'Header : sc_gen rotamer xyz'
	  WRITE (8,*) 'Header : sc_gen rotamer probabilities'
	  DO 820 ir		= 1, num_residues
	    i1			= residue (ir,1)
	    i2			= residue (ir,2)
	    nres		= ats_nres (i1)
	    chain		= ats_chain (i1)
	    res			= ats_res (i1)
	    atom_here		= ats_atom (i1)
	    IF     (atom_here .EQ. 'ATOM') THEN
	      atom6		= 'ATOM  '
	    ELSE IF (atom_here .EQ. 'HETA') THEN
	      atom6		= 'HETATM'
	    ELSE
	      STOP 'ats_atom not ATOM/HETA'
	    END IF
	    WRITE (7,4300) chain, nres, res
	    IF (fix_residue(ir).OR.(res_rots_ind(ir,1).EQ.-999)) THEN  ! fixed

	      n_model		= 1
	      prb		= 1.
	      WRITE (8,4600) chain, nres, res, n_model, prb
C??prob	      WRITE (7,4100) n_model
	      DO 786 i		= i1, i2
	        atyp		= ats_atyp (i)
	        DO 784 n	= 1, 3
		  xyz (n)	= ats_xyz (i,n)
784	        CONTINUE			! End n
C??prob	        WRITE (7,4400) atom_here,prb,atyp,res,chain,nres,xyz
	        WRITE (7,4400) atom_here,atyp,res,chain,nres,xyz
786	      CONTINUE				! End i
C??prob	      WRITE (7,4200)

	    ELSE				! residue with moving atoms
C??prob	    IF (res_rots_ind (ir,1) .NE. -999) THEN

	      num_tmp			= 0	! first: fixed ats this res
	      prb			= 1.
	      DO 790 ifh		= 1, n_fixed
	        irh			= fixed_ind (ifh,1)
	        IF (irh.EQ.ir) THEN
C?? giggs/scf_cxxc - don't know if this atyp cludge will work
	          atyp		= ats_atyp (i1+num_tmp)
C??prob	      WRITE (line_tmp,4400) atom_here,prb,atyp,res,chain,nres,
C??prob     1			           (fixed(ifh,n),n=1,3)
	      WRITE (line_tmp,4400) atom_here,atyp,res,chain,nres,
     1			           (fixed(ifh,n),n=1,3)
	          num_tmp	= num_tmp + 1
		  IF (num_tmp .GT. 100) STOP 'num_tmp > 100'
		  line_buf (num_tmp)	= line_tmp
	        END IF
790	      CONTINUE						! End ifh

	      IF (incl_exp (ir)) THEN
		nrots		= 1 + res_rots_ind (ir,1)
	      ELSE
		nrots		=     res_rots_ind (ir,1)
	      END IF
	      IF ((nrots.GE.1).AND.(nrots.LE.50)) THEN
	        n_model		= 0
		DO 800 irot	= 1, nrots
	          IF ((res.EQ.'GLY ') .OR. (res.EQ.'ALA ') .OR.
     1		      (res.EQ.'PRO ')) THEN		! no moving parts
		    IF (num_tmp .NE. 0) THEN
		      DO 792 iath	= 1, num_tmp
		        WRITE (7,4500) line_buf (iath)
792		      CONTINUE				! End iath
		    END IF
	            GOTO 820
	          END IF
		  n_model	= n_model + 1
		  WRITE (7,4100) n_model
		  IF (num_tmp .NE. 0) THEN
		    DO 793 iath	= 1, num_tmp
		      WRITE (7,4500) line_buf (iath)
793		    CONTINUE				! End iath
		  END IF
	          im1		= res_rots_ind (ir,3)
	          im2		= res_rots_ind (ir,4)
	          natsh		= res_rots_ind (ir,2)
	          IF (fix_aswego) THEN
	            IF (.NOT. set_aswego(ir)) STOP 'set_aswego NOT set'
	            prb		= p_aswego (ir,irot)
	          ELSE
		    prb		= p_old (ir,irot)
	          END IF
	          DO 794 iath	= 1, natsh
		    imh		= im1 + natsh*(irot-1) + iath-1
		    IF ((imh.LT.im1).OR.(imh.GT.im2)) STOP 'imh oub'
C??prob - not sure about num_tmp here or in earlier 700+ code
		    atyp	= ats_atyp (i1+iath+num_tmp-1)
C??prob		    WRITE (7,4400) atom_here,prb,atyp,res,chain,nres,
C??prob     1		                  (moved(imh,n),n=1,3)
		    WRITE (7,4400) atom_here,atyp,res,chain,nres,
     1		                  (moved(imh,n),n=1,3)
794	          CONTINUE					! End iath
	          WRITE (8,4600) chain, nres, res, n_model, prb
		  WRITE (7,4200)
800		CONTINUE					! End irot
	      ELSE
		STOP 'n_here/rots at rots out'
	      END IF
	    END IF
820	  CONTINUE						! End ir
	END IF
	CLOSE (UNIT=7)
	CLOSE (UNIT=8)

	IF (aa_output) THEN	! output the vdw at which aa first packs
	  OPEN (UNIT=7, NAME='aa_vdw.txt', TYPE='NEW')
	  WRITE (7,*) 'Individual aa VdW tolerances at first packing'
	  DO 840 ir		= 1, num_residues
	    i1			= residue (ir,1)
	    nres		= ats_nres (i1)
	    chain		= ats_chain (i1)
	    res			= ats_res (i1)
	    WRITE (7,4800) chain, nres, res, res_vdw(ir)
840	  CONTINUE						! End ir
	  CLOSE (UNIT=7)
	END IF
C ?? wednesday

	RETURN
	END


	SUBROUTINE	brick_em
C	**********	********

	REAL		fixed (70000,4), moved (700000,4)
	INTEGER		res_rots_ind (10000,4)
	INTEGER*2	moved_ind (700000,3), fixed_ind (70000,2)
	REAL		xyz_lims (3,2), range (3), xyz (3)

	REAL		bg (3,2)
	INTEGER		bn (3), brk_ind (50,50,50,2),
     1			brk (770000)
	INTEGER		tally (50,50,50)

	COMMON	/ats_loads/	fixed, fixed_ind, moved, moved_ind,
     1				if, im, res_rots_ind, n_fixed,n_moved,
     2				num_residues
	COMMON	/bricks/	bg, bn, brk_ind, brk, step,
     1				vdw_max_fixed, vdw_max_moved

2500	FORMAT (/,' Bricking atoms, with bricks of ',F6.2,' A',/)
2520	FORMAT (' On ',A1,' low, high, top index = ',2F6.1,2X,I2)

C Get the molecule limits and the max assigned VdW radius.

	DO 160 n		= 1, 3
	  xyz_lims (n,1)	=  1.E20
	  xyz_lims (n,2)	= -1.E20
160	CONTINUE						! End n
	vdw_max_fixed		= 0.
	vdw_max_moved		= 0.

	DO 180 if		= 1, n_fixed
	  DO 170 n		= 1, 3
	    IF (fixed (if,n) .LT. xyz_lims (n,1)) THEN
	      xyz_lims (n,1)	= fixed (if,n)
	    END IF
	    IF (fixed (if,n) .GT. xyz_lims (n,2)) THEN
	      xyz_lims (n,2)	= fixed (if,n)
	    END IF
170	  CONTINUE						! End n
	  IF (fixed (if,4) .GT. vdw_max_fixed) THEN
	    vdw_max_fixed	= fixed (if,4)
	  END IF
180	CONTINUE						! End if
	DO 200 im		= 1, n_moved
	  DO 190 n		= 1, 3
	    IF (moved (im,n) .LT. xyz_lims (n,1)) THEN
	      xyz_lims (n,1)	= moved (im,n)
	    END IF
	    IF (moved (im,n) .GT. xyz_lims (n,2)) THEN
	      xyz_lims (n,2)	= moved (im,n)
	    END IF
190	  CONTINUE						! End n
	  IF (moved (im,4) .GT. vdw_max_moved) THEN
	    vdw_max_moved	= moved (im,4)
	  END IF
200	CONTINUE						! End im

C Work out a bricking array of 50 points in the longest direction.  Use
C a uniform step on x,y,z to ease distance screening with bricks.
C Store grid boundaries, grid dims, grid step: bg(3,2), bn(3), step.
C Actually each brick stores atoms 'above' lowest corner - last brk not used??

	biggest			= 0.
	DO 220 n		= 1, 3
	  range (n)		= xyz_lims (n,2) - xyz_lims (n,1)
	  IF (range (n) .GT. biggest) THEN
	    biggest		= range (n)
	    n_biggest		= n
	  END IF
220	CONTINUE						! End n
	step			= (biggest+0.2)/49.
	DO 230 n		= 1, 3
	  bg (n,1)		= xyz_lims (n,1) - 0.1
	  IF (n .EQ. n_biggest) THEN
	    bn (n)		= 50
	  ELSE
	    bn (n)		= 2 + INT ((range(n)+0.2)/step)
	    IF      (bn (n) .GT. 51) THEN
	      STOP 'bn/51'
	    ELSE IF (bn (n) .GT. 50) THEN
	      bn (n)		= 50
	    END IF
C??	    IF (bn (n) .GT. 50) STOP 'bn/50'
	  END IF
	  bg (n,2)		= bg (n,1) + step*FLOAT(bn(n)-1)
230	CONTINUE						! End n

C Brick atoms with brk_ind pointing to first and last positions in the
C linear brk array for brick (ijk).  In brk, negative indices point to
C atom location in fixed array, positive indices to atom in moved array.
C In set-up, we do 2 passes through atoms - count and then store.

	DO 250 k		= 1, bn (3)
	  DO 250 j		= 1, bn (2)
	    DO 250 i		= 1, bn (1)
	      brk_ind (i,j,k,2)	= 0
	      tally (i,j,k)	= 0
250	CONTINUE						! End i,j,k

	DO 500 i_pass		= 1, 2

	  IF (i_pass .EQ. 2) THEN
	    ib_tot		= 0
	  END IF

	  DO 300 if		= 1, n_fixed
	    DO 270 n		= 1, 3
	      xyz (n)		= fixed (if,n)
270	    CONTINUE						! End n
	    CALL	which_brk (xyz, i,j,k)
	    IF (i_pass .EQ. 1) THEN
	      brk_ind (i,j,k,2)	= brk_ind (i,j,k,2) + 1
	    ELSE
	      tally (i,j,k)	= tally (i,j,k) + 1
	      ib_tot		= ib_tot + 1
	      ib		= brk_ind(i,j,k,1)+tally(i,j,k)-1
	      brk (ib)		= -if
	    END IF
300	  CONTINUE						! End if
	  DO 400 im		= 1, n_moved
	    DO 320 n		= 1, 3
	      xyz (n)		= moved (im,n)
320	    CONTINUE						! End n
	    CALL	which_brk (xyz, i,j,k)
	    IF (i_pass .EQ. 1) THEN
	      brk_ind (i,j,k,2)	= brk_ind (i,j,k,2) + 1
	    ELSE
	      tally (i,j,k)	= tally (i,j,k) + 1
	      ib_tot		= ib_tot + 1
	      ib		= brk_ind(i,j,k,1)+tally(i,j,k)-1
	      brk (ib)		=  im
	    END IF
400	  CONTINUE						! End im
	  IF (i_pass .EQ. 1) THEN
	    n_sofar		= 0
	    DO 440 k		= 1, bn (3)
	      DO 440 j		= 1, bn (2)
		DO 440 i	= 1, bn (1)
		  IF (brk_ind (i,j,k,2) .NE. 0) THEN
		    brk_ind (i,j,k,1) = n_sofar + 1
		    n_here	= brk_ind (i,j,k,2)
		    brk_ind (i,j,k,2) = brk_ind (i,j,k,1) + n_here-1
		    n_sofar	= n_sofar + n_here
		  ELSE
		    brk_ind (i,j,k,1) = 0
		  END IF
440	    CONTINUE						! End i,j,k
	    IF (n_sofar .NE. (n_fixed+n_moved)) STOP 'n_sofar'
	  ELSE
	    IF (ib_tot .NE. (n_fixed+n_moved)) STOP 'ib'
	    DO 460 k		= 1, bn (3)
	      DO 460 j		= 1, bn (2)
		DO 460 i	= 1, bn (1)
		  indh1		= brk_ind (i,j,k,1)
		  indh2		= brk_ind (i,j,k,2)
		  IF ((indh1.EQ.0) .AND. (indh2.EQ.0)) THEN
		    n_here	= 0
		  ELSE
		    n_here	= indh2 - indh1 + 1
		  END IF
		  IF (n_here .NE. tally (i,j,k)) STOP 'tally'
460	    CONTINUE						! End i,j,k
	  END IF
500	CONTINUE						! End i_pass

C Write out some bricking info.

	WRITE (4,2500) step
	WRITE (4,2520) 'x', bg(1,1),bg(1,2), bn (1)
	WRITE (4,2520) 'y', bg(2,1),bg(2,2), bn (2)
	WRITE (4,2520) 'z', bg(3,1),bg(3,2), bn (3)

	RETURN
	END


	SUBROUTINE	which_brk (xyz, i,j,k)
C	**********	*********

C atom with coordinates xyz is stored in which brick (i,j,k) ?

	REAL		bg (3,2), xyz (3)
	INTEGER		bn (3), brk_ind (50,50,50,2),
     1			brk (770000)
	INTEGER		ijk (3)

	COMMON	/bricks/	bg, bn, brk_ind, brk, step,
     1				vdw_max_fixed, vdw_max_moved

	DO 200 n	= 1, 3
	  dist		= xyz (n) - bg (n,1)
	  ijk (n)	= 1 + INT (dist/step)
	  IF ((ijk(n).LT.1).OR.(ijk(n).GT.bn(n))) STOP 'ijk oub'
200	CONTINUE						! End n
	i		= ijk (1)
	j		= ijk (2)
	k		= ijk (3)

	RETURN
	END


	SUBROUTINE	vs_fixed (ir,xyz1,rad1,rank,nats, clashed)
C	**********	********

C Test this rotamer against fixed atoms.

	REAL		fixed (70000,4), moved (700000,4)
	INTEGER		res_rots_ind (10000,4)
	INTEGER*2	moved_ind (700000,3), fixed_ind (70000,2)
	REAL		xyz1 (15,3), rad1 (15), xyza (3), xyzb (3)
	LOGICAL		clashed
	INTEGER		brk_rad, ijk (3), brk_srch (3,2)
	REAL		bg (3,2)
	INTEGER		bn (3), brk_ind (50,50,50,2),
     1			brk (770000)
	LOGICAL		brick_atoms, collect_clash
	INTEGER*2	rank (15)

	COMMON	/ctr_clash/	brick_atoms, vdw_dec, vdw_relax,
     1				conv_frac, collect_clash
	COMMON	/ats_loads/	fixed, fixed_ind, moved, moved_ind,
     1				if, im, res_rots_ind, n_fixed,n_moved,
     2				num_residues
	COMMON	/bricks/	bg, bn, brk_ind, brk, step,
     1				vdw_max_fixed, vdw_max_moved

	clashed		= .FALSE.

	DO 500 ia	= 1, nats
	  DO 200 n	= 1, 3
	    xyza (n)	= xyz1 (ia,n)
200	  CONTINUE						! End n
	  rada	= rad1 (ia)

	  IF (brick_atoms) THEN

	    CALL	which_brk (xyza, i,j,k)
	    ijk (1)	= i
	    ijk (2)	= j
	    ijk (3)	= k
	    brk_rad	= 1 + INT ((rada+vdw_max_fixed)/step)
	    DO 220 n		= 1, 3
	      brk_srch (n,1)	= ijk (n) - brk_rad
	      IF (brk_srch (n,1).LT.1) brk_srch (n,1) = 1
	      brk_srch (n,2)	= ijk (n) + brk_rad
	      IF (brk_srch(n,2) .GT. bn(n)) brk_srch (n,2) = bn (n)
220	    CONTINUE						! End n
	    DO 400 k		= brk_srch (3,1), brk_srch (3,2)
	      DO 400 j		= brk_srch (2,1), brk_srch (2,2)
	        DO 400 i	= brk_srch (1,1), brk_srch (1,2)
		  ib1		= brk_ind (i,j,k,1)
		  ib2		= brk_ind (i,j,k,2)
		  IF (ib1.NE.0 .AND. ib2.NE.0) THEN
		    DO 300 ib	= ib1, ib2
C In brk, -ve indices point to locn in fixed, +ve indices point to moved.
		      IF (brk (ib) .LT. 0) THEN

C ?? Check for fixed dist mates (same ir) in the fixed array - the tuffery
C ?? set avoids its own mc - just leave in clashes with the mc O.

			IF (fixed_ind (-brk(ib),1) .EQ. ir) THEN
			  IF      (fixed_ind(-brk(ib),2).EQ.-1) THEN
			    GOTO 300
			  ELSE IF (fixed_ind(-brk(ib),2).EQ.-2) THEN
			    GOTO 300
			  ELSE IF (fixed_ind(-brk(ib),2).EQ.-3) THEN
			    GOTO 300
			  ELSE IF (fixed_ind(-brk(ib),2).NE. 0) THEN
			    STOP 'fixed_ind.ne.0'
			  END IF
			END IF

		        DO 260 n = 1, 3
			  xyzb (n) = fixed (-brk(ib),n)
260		        CONTINUE				! End n
		        radb	= fixed (-brk(ib),4)
		        dist	= SQRT ( (xyza(1)-xyzb(1))**2 +
     1					 (xyza(2)-xyzb(2))**2 +
     2					 (xyza(3)-xyzb(3))**2 )
		        dist_test = rada + radb - vdw_relax
		        IF (dist .LT. dist_test) THEN
			  clashed	= .TRUE.
			  RETURN
		        END IF					! EndIf clash
		      END IF					! EndIf fixed
300		    CONTINUE					! End ib
		  END IF
400	    CONTINUE						! End i,j,k

	  ELSE

	    DO 460 ifh		= 1, n_fixed

C ?? Check for fixed dist mates (same ir) in the fixed array - the tuffery
C ?? set avoids its own mc - just leave in clashes with the mc O.

	      IF (fixed_ind (ifh,1) .EQ. ir) THEN
		IF      (fixed_ind(ifh,2).EQ.-1) THEN
		  GOTO 460
		ELSE IF (fixed_ind(ifh,2).EQ.-2) THEN
		  GOTO 460
		ELSE IF (fixed_ind(ifh,2).EQ.-3) THEN
		  GOTO 460
		ELSE IF (fixed_ind(ifh,2).NE. 0) THEN
		  STOP 'fixed_ind.ne.0'
		END IF
	      END IF

	      DO 440 n		= 1, 3
		xyzb (n)	= fixed (ifh,n)
440	      CONTINUE						! End n
	      radb		= fixed (ifh,4)
	      dist		= SQRT ( (xyza(1)-xyzb(1))**2 +
     1					 (xyza(2)-xyzb(2))**2 +
     2					 (xyza(3)-xyzb(3))**2 )
	      dist_test = rada + radb - vdw_relax
	      IF (dist .LT. dist_test) THEN
		clashed	= .TRUE.
		RETURN
	      END IF						! EndIf clash
460	    CONTINUE						! End ifh

	  END IF						! EndIf brked
500	CONTINUE						! End ia

	RETURN
	END


	SUBROUTINE	vs_moved (ir,xyz1,rad1,nats, strike_out)
C	**********	********

C Test this rotamer against moved atoms (non-ir).  Keep a record of all
C whether each non-ir rotamer has a clash with this ir rotamer - then
C multiply by p_old (jr,jrot) at the finish to give p_new (ir,irot).

	REAL		fixed (70000,4), moved (700000,4)
	INTEGER		res_rots_ind (10000,4)
	INTEGER*2	moved_ind (700000,3), fixed_ind (70000,2)
	REAL		xyz1 (15,3), rad1 (15), xyza (3), xyzb (3)
	INTEGER		brk_rad, ijk (3), brk_srch (3,2)
	REAL		bg (3,2)
	INTEGER		bn (3), brk_ind (50,50,50,2),
     1			brk (770000)
	LOGICAL		brick_atoms, collect_clash,
     1			rem_oddmc, incl_exp (10000)
	LOGICAL*1	strike_out (10000,50)
	CHARACTER*4	fix_type

	COMMON	/ctr_clash/	brick_atoms, vdw_dec, vdw_relax,
     1				conv_frac, collect_clash
	COMMON	/ats_loads/	fixed, fixed_ind, moved, moved_ind,
     1				if, im, res_rots_ind, n_fixed,n_moved,
     2				num_residues
	COMMON	/bricks/	bg, bn, brk_ind, brk, step,
     1				vdw_max_fixed, vdw_max_moved
	COMMON	/experiment/	fix_type, rem_oddmc
	COMMON	/incl_exprot/	incl_exp

	DO 180 jr		= 1, num_residues
	  IF (res_rots_ind (jr,1) .NE. -999) THEN
	    IF (incl_exp (jr)) THEN
	      mrots		= 1 + res_rots_ind (jr,1)
	    ELSE
	      mrots		=     res_rots_ind (jr,1)
	    END IF
	    DO 170 jrot		= 1, mrots
	      strike_out (jr,jrot) = .FALSE.
170	    CONTINUE						! End jrot
	  END IF
180	CONTINUE						! End jr

	DO 500 ia	= 1, nats
	  DO 200 n	= 1, 3
	    xyza (n)	= xyz1 (ia,n)
200	  CONTINUE						! End n
	  rada		= rad1 (ia)

	  IF (brick_atoms) THEN

	    CALL	which_brk (xyza, i,j,k)
	    ijk (1)	= i
	    ijk (2)	= j
	    ijk (3)	= k
	    brk_rad	= 1 + INT ((rada+vdw_max_moved)/step)
	    DO 220 n		= 1, 3
	      brk_srch (n,1)	= ijk (n) - brk_rad
	      IF (brk_srch (n,1).LT.1) brk_srch (n,1) = 1
	      brk_srch (n,2)	= ijk (n) + brk_rad
	      IF (brk_srch(n,2) .GT. bn(n)) brk_srch (n,2) = bn (n)
220	    CONTINUE						! End n
	    DO 400 k		= brk_srch (3,1), brk_srch (3,2)
	      DO 400 j		= brk_srch (2,1), brk_srch (2,2)
	        DO 400 i	= brk_srch (1,1), brk_srch (1,2)
		  ib1		= brk_ind (i,j,k,1)
		  ib2		= brk_ind (i,j,k,2)
		  IF (ib1.NE.0 .AND. ib2.NE.0) THEN
		    DO 300 ib	= ib1, ib2
C In brk, -ve indices point to locn in fixed, +ve indices point to moved.
		      IF (brk (ib) .GT. 0) THEN
		        imh	= brk (ib)
		        jr	= moved_ind (imh,1)
		        IF (jr .EQ. ir) GOTO 300
		        jrot	= moved_ind (imh,2)
		        IF (strike_out(jr,jrot)) GOTO 300
		        DO 260 n = 1, 3
			  xyzb (n) = moved (imh,n)
260		        CONTINUE				! End n
		        radb	= moved (imh,4)
		        dist	= SQRT ( (xyza(1)-xyzb(1))**2 +
     1					 (xyza(2)-xyzb(2))**2 +
     2					 (xyza(3)-xyzb(3))**2 )
		        dist_test = rada + radb - vdw_relax
		IF (dist.LT.dist_test) strike_out(jr,jrot) = .TRUE.
		      END IF					! EndIf moved
300		    CONTINUE					! End ib
		  END IF
400	    CONTINUE						! End i,j,k

	  ELSE

	    DO 460 imh		= 1, n_moved
	      jr		= moved_ind (imh,1)
	      IF (jr .EQ. ir) GOTO 460
	      jrot		= moved_ind (imh,2)
	      IF (strike_out(jr,jrot)) GOTO 460
	      DO 440 n		= 1, 3
		xyzb (n)	= moved (imh,n)
440	      CONTINUE						! End n
	      radb		= moved (imh,4)
	      dist		= SQRT ( (xyza(1)-xyzb(1))**2 +
     1					 (xyza(2)-xyzb(2))**2 +
     2					 (xyza(3)-xyzb(3))**2 )
	      dist_test		= rada + radb - vdw_relax
	      IF (dist.LT.dist_test) strike_out(jr,jrot) = .TRUE.
460	    CONTINUE						! End imh

	  END IF						! EndIf brked

500	CONTINUE						! End ia

	RETURN
	END


	SUBROUTINE xyz_to_rtp (X,R,TH,PH)
C	********** **********
C
C This routine generates the spherical polar theta,phi ordinates
C corresponding to the cartesian ordinates X, (angles in degrees).
C It is assumed that the radius is already known. R is not
C checked against mag(X) *****
C
	REAL X(3)
C Theta ranges from 0 to 180 degrees, and arccos is single
C valued in this range.
Clnx	TH=ACOSD (X(3)/R)
	th_rads		= ACOS (X(3)/R)
	TH		= (th_rads*180.)/3.14159

C Phi ranges from 0 to 180 degrees, and ATAN is not single
C valued in this range. The ATAN2D routine outputs arctan(a1/a2)
C in the range -180 to +180. The result has the sign of a1.
C Adjust so that for ATAN2 -ve -> + 360 should **** give
C continuity of phi.
Clnx	PH=ATAN2D (X(2),X(1))
	PH_RADS	= ATAN2 (X(2),X(1))
	PH	= (PH_RADS*180.)/3.14159

	IF(PH.LT.0.) PH=PH + 360.
	RETURN
	END


	SUBROUTINE	rtp_to_xyz (r, t_deg, p_deg, xyz)
C	**********	**********

C Routine converts the spherical polar (rtp) set to cartesian (xyz).

	REAL		xyz (3)

Clnx	xyz (1)		= r * SIND (t_deg) * COSD (p_deg)
Clnx	xyz (2)		= r * SIND (t_deg) * SIND (p_deg)
Clnx	xyz (3)		= r * COSD (t_deg)

	t_rads		= (t_deg*3.14159)/180.
	p_rads		= (p_deg*3.14159)/180.
	xyz (1)		= r * SIN (t_rads) * COS (p_rads)
	xyz (2)		= r * SIN (t_rads) * SIN (p_rads)
	xyz (3)		= r * COS (t_rads)

	RETURN
	END


	SUBROUTINE	leader (char_array, itop)
C	**********	******

	CHARACTER*60	char_array

	itop	= 1
	DO WHILE (char_array (itop:itop) .NE. ' ')
	  itop	= itop + 1
	END DO
	itop	= itop - 1

	RETURN
	END


	SUBROUTINE	pro_adhoc (sc_geom_xyz, sc_geom_atyp,
C	**********	*********
     1				   cg_mult, cd_mult)

C Work out the multipliers of 3 vec basis set to give cg and cd.
C 3 vec basis set = ca->cb, ca->n, perp to these 2.
C Put new  cg or cd = mult*these 3, work out mult from matrix inversion
C for existing cg, cd solution from sc_geometry.
C Then just use these multipliers for new pro cg, cd positions.
C sc_geom slots 1-5 have ca,cb,cg,cd,n

	REAL		sc_geom_xyz (20,12,3)
	CHARACTER*4	sc_geom_atyp (20,12)
	REAL		cg_mult (3), cd_mult (3)
	REAL		perp(3),mat(3,3),inv(3,3),ca_cg(3),ca_cd(3),
     1			ca_cb(3),ca_n(3)
	LOGICAL		singular

	IF (sc_geom_atyp (13,1) .NE. 'CA  ') STOP 'pro - CA'
	IF (sc_geom_atyp (13,2) .NE. 'CB  ') STOP 'pro - CB'
	IF (sc_geom_atyp (13,3) .NE. 'CG  ') STOP 'pro - CG'
	IF (sc_geom_atyp (13,4) .NE. 'CD  ') STOP 'pro - CD'
	IF (sc_geom_atyp (13,5) .NE. 'N   ') STOP 'pro - N'
	DO 160 ja	= 1, 5
	  DO 160 n	= 1,3 
	    IF (sc_geom_xyz (13,ja,n) .EQ. 999.) STOP 'pro - 999'
160	CONTINUE						! End n, ja

	DO 200 n	= 1, 3
	  ca_cb (n)	= sc_geom_xyz (13,2,n) - sc_geom_xyz (13,1,n)
	  ca_n (n)	= sc_geom_xyz (13,5,n) - sc_geom_xyz (13,1,n)
	  ca_cg (n)	= sc_geom_xyz (13,3,n) - sc_geom_xyz (13,1,n)
	  ca_cd (n)	= sc_geom_xyz (13,4,n) - sc_geom_xyz (13,1,n)
200	CONTINUE						! End n
	CALL cross_product (ca_cb, ca_n, perp)
	DO 300 n	= 1, 3
	  mat (n,1)	= ca_cb (n)
	  mat (n,2)	= ca_n (n)
	  mat (n,3)	= perp (n)
300	CONTINUE						! End n
	CALL	INVERT3 (mat, inv, determinant, singular)
	IF (singular) STOP 'pro - singular'
	CALL	mat_vec_mult (inv, ca_cg, cg_mult)
	CALL	mat_vec_mult (inv, ca_cd, cd_mult)

C?? hewitt are we done here?
	RETURN
	END


	SUBROUTINE	sc_templates
C	**********	************

C Read in the sc templates, to be used in constructing missing scs.

	REAL		xyz (3)
	CHARACTER*60	data_path
	CHARACTER*4	atom, atyp, res, atm, res_here
	CHARACTER*1	chain

	INTEGER		n_sc_ats (20), chi_ats (20,5,4), sc_chi (20)
	CHARACTER*4	sc_ats (20,10), chi_start (20,3)
	INTEGER*2	sc_rank (20,10)

	REAL		res_chi (20,50,5)
	INTEGER		res_nchi (20), res_natm (20), res_nrot (20)
	CHARACTER*4	res_tag (20)

	REAL		sc_geom_xyz (20,12,3)
	CHARACTER*4	sc_geom_atyp (20,12)
	REAL		cg_mult (3), cd_mult (3), cb_mult (3)

	COMMON	/rot_dict/	res_nchi, res_natm, res_chi,
     1				res_nrot, res_tag
	COMMON	/rot_scs/	n_sc_ats, sc_ats, chi_ats, sc_chi,
     1				chi_start, sc_rank
	COMMON	/data_loc/	data_path, idp
	COMMON	/sc_templ/	sc_geom_xyz, sc_geom_atyp,
     1				cg_mult, cd_mult, cb_mult

1200	FORMAT (A4, 9X, A4, A4, 1X, I4, 4X, 3F8.3)

C Read in the dictionary sidechain geometries - to give bond lengths/angles.
C Do not need to read on Gly info, since without a CA we're stuffed anyhow.
C File will have Ca-Cb-rest of sc.
C Initialise sc_geom_xyz (coords) so that evident if nothing picked up.

	DO 140 ir		= 1, 20
	  DO 140 ja		= 1, 12
	    DO 140 n		= 1, 3
	      sc_geom_xyz (ir,ja,n) = 999.
140	CONTINUE						! End ir

	CALL getenv ('DATA', data_path)
	idp			= 1
	DO WHILE (data_path (idp:idp) .NE. ' ')
	  idp			= idp + 1
	END DO
	OPEN (UNIT=2, NAME=data_path(1:idp-1)//'sc_geometry.dat',
     1	      TYPE='OLD')
	READ (2,*)

	Do 180 ir		= 1, 20
	  IF (ir.EQ.6) GOTO 180
	  res_here		= res_tag (ir)
160	  READ (2,1200,END=170) atom, atyp, res, nres, xyz
	  IF (atom .NE. 'ATOM') STOP 'ATOM - sc_geometry'
	  IF (res .EQ. res_here) THEN

C?? For Pro we need one extra atom (mc N) to define geom for sidechain - put
C?? it in slot 5 - after the 4 we have for ca, cb, cg, cd.
	    IF (res .EQ. 'PRO ') THEN				! Pro only
	      IF (ir .NE. 13) STOP '13 - sc_geometry'
	      IF (atyp .NE. 'N   ') STOP 'N - sc_geometry'
	      sc_geom_atyp (ir,5) = 'N   '
	      DO 161 n		= 1, 3
		sc_geom_xyz (ir,5,n) = xyz (n)
161	      CONTINUE						! End n
	      READ (2,1200,END=170) atom, atyp, res, nres, xyz
	      IF (atom .NE. 'ATOM') STOP 'ATOM - sc_geometry'
	      IF (res .NE. res_here) STOP 'res - sc_geometry0'
	    END IF						! End Pro only

C?? For Ala we need extra atoms (mc N, C) to define geom for CB tepmplate -
C?? put these in slots 3,4 - after the 2 we have for ca, cb.
	    IF (res .EQ. 'ALA ') THEN				! Pro only
	      IF (ir .NE. 1) STOP '1 - sc_geometry'
	      IF (atyp .NE. 'N   ') STOP 'N - sc_geometry'
	      sc_geom_atyp (ir,3) = 'N   '
	      DO 158 n		= 1, 3
		sc_geom_xyz (ir,3,n) = xyz (n)
158	      CONTINUE						! End n
	      READ (2,1200,END=170) atom, atyp, res, nres, xyz
	      IF (atom .NE. 'ATOM') STOP 'ATOM - sc_geometry'
	      IF (res .NE. res_here) STOP 'res - sc_geometry0'
	      IF (atyp .NE. 'C   ') STOP 'C - sc_geometry'
	      sc_geom_atyp (ir,4) = 'C   '
	      DO 159 n		= 1, 3
		sc_geom_xyz (ir,4,n) = xyz (n)
159	      CONTINUE						! End n
	      READ (2,1200,END=170) atom, atyp, res, nres, xyz
	      IF (atom .NE. 'ATOM') STOP 'ATOM - sc_geometry'
	      IF (res .NE. res_here) STOP 'res - sc_geometry0'
	    END IF						! End Ala only

	    IF (atyp .NE. 'CA  ') STOP 'CA - sc_geometry'
	    sc_geom_atyp (ir,1)	= 'CA  '
	    DO 162 n		= 1, 3
	      sc_geom_xyz (ir,1,n)	= xyz (n)
162	    CONTINUE						! End n
	    READ (2,1200,END=170) atom, atyp, res, nres, xyz
	    IF (atom .NE. 'ATOM') STOP 'ATOM - sc_geometry'
	    IF (res .NE. res_here) STOP 'res - sc_geometry1'
	    IF (atyp .NE. 'CB  ') STOP 'CB - sc_geometry'
	    sc_geom_atyp (ir,2)	= 'CB  '
	    DO 163 n		= 1, 3
	      sc_geom_xyz (ir,2,n)	= xyz (n)
163	    CONTINUE						! End n

	    DO 166 ja		= 1, n_sc_ats (ir)
	      READ (2,1200,END=170) atom, atyp, res, nres, xyz
	      IF (atom .NE. 'ATOM') STOP 'ATOM - sc_geometry'
	      IF (res .NE. res_here) STOP 'res - sc_geometry2'
	      IF (atyp .NE. sc_ats (ir,ja)) STOP 'atyp - sc_geometry'
	      sc_geom_atyp (ir,2+ja) = atyp
	      DO 164 n		= 1, 3
		sc_geom_xyz (ir,2+ja,n) = xyz (n)
164	      CONTINUE						! End n
166	    CONTINUE						! End ja

	    REWIND 2
	    READ (2,*)
	    GOTO 180
	  END IF
	  GOTO 160

180	CONTINUE						! End ir
	GOTO 190

170	STOP 'eof - sc_geometry'

190	CLOSE (UNIT=2)

C?? Take the Pro model entry from sc_geom_xyz and work out ad hoc params to
C?? add on CG and CD from the fixed coords file.

	CALL	pro_adhoc (sc_geom_xyz,sc_geom_atyp,cg_mult,cd_mult)

C Similar ad-hoc procedure for CB adding (use Ala sc as template).
	CALL	cb_adhoc (sc_geom_xyz,sc_geom_atyp,cb_mult)

	RETURN
	END


	SUBROUTINE	pro_make (n_targ, ca_targ, cb_targ, cg_mult,
C	**********	********
     1				  cd_mult, cg_targ, cd_targ)

C Just use the n,ca,cb atoms and the worked out multipliers to get cg,cd.

	REAL		n_targ(3), ca_targ(3), cb_targ(3), cg_mult(3),
     1			cd_mult(3), cg_targ(3), cd_targ (3),
     2			ca_n(3), ca_cb(3), perp(3)

	DO 200 n	= 1, 3
	  ca_n (n)	= n_targ (n) - ca_targ (n)
	  ca_cb (n)	= cb_targ (n) - ca_targ (n)
200	CONTINUE						! End n
	CALL cross_product (ca_cb, ca_n, perp)
	DO 300 n	= 1, 3
	  cg_targ (n)	= cg_mult (1) * ca_cb (n) +
     1			  cg_mult (2) * ca_n (n) +
     2			  cg_mult (3) * perp (n) +
     3			  ca_targ (n)
	  cd_targ (n)	= cd_mult (1) * ca_cb (n) +
     1			  cd_mult (2) * ca_n (n) +
     2			  cd_mult (3) * perp (n) +
     3			  ca_targ (n)
300	CONTINUE						! End n
C??hewitt
	RETURN
	END


	SUBROUTINE	cb_adhoc (sc_geom_xyz, sc_geom_atyp,
C	**********	********
     1				  cb_mult)

C Work out the multipliers of 3 vec basis set to give cb.
C 3 vec basis set = ca->c, ca->n, perp to these 2.
C Put new cb = mult*these 3, work out mult from matrix inversion
C for existing cb solution from sc_geometry.
C Then just use these multipliers for any new cb position.
C sc_geom slots 1-5 have ca,cb,cg,cd,n

	REAL		sc_geom_xyz (20,12,3)
	CHARACTER*4	sc_geom_atyp (20,12)
	REAL		cb_mult (3)
	REAL		perp(3),mat(3,3),inv(3,3),ca_cb(3),
     1			ca_c(3),ca_n(3)
	LOGICAL		singular

	IF (sc_geom_atyp (1,1) .NE. 'CA  ') STOP 'pro - CA'
	IF (sc_geom_atyp (1,2) .NE. 'CB  ') STOP 'pro - CB'
	IF (sc_geom_atyp (1,3) .NE. 'N   ') STOP 'ala - N'
	IF (sc_geom_atyp (1,4) .NE. 'C   ') STOP 'ala - C'
	DO 160 ja	= 1, 4
	  DO 160 n	= 1, 3
	    IF (sc_geom_xyz (1,ja,n) .EQ. 999.) STOP 'ala - 999'
160	CONTINUE						! End n, ja

	DO 200 n	= 1, 3
	  ca_cb (n)	= sc_geom_xyz (1,2,n) - sc_geom_xyz (1,1,n)
	  ca_n (n)	= sc_geom_xyz (1,3,n) - sc_geom_xyz (1,1,n)
	  ca_c (n)	= sc_geom_xyz (1,4,n) - sc_geom_xyz (1,1,n)
200	CONTINUE						! End n
	CALL cross_product (ca_c, ca_n, perp)
	DO 300 n	= 1, 3
	  mat (n,1)	= ca_c (n)
	  mat (n,2)	= ca_n (n)
	  mat (n,3)	= perp (n)
300	CONTINUE						! End n
	CALL	INVERT3 (mat, inv, determinant, singular)
	IF (singular) STOP 'ala - singular'
	CALL	mat_vec_mult (inv, ca_cb, cb_mult)
	RETURN
	END


	SUBROUTINE	cb_make (n_targ,ca_targ,c_targ,cb_mult,cb_targ)
C	**********	*******

C Just use the n,ca,c atoms and the worked out multiplier to get cb.

	REAL		n_targ(3), ca_targ(3), cb_targ(3), cb_mult(3),
     1			ca_n(3), ca_cb(3), perp(3),c_targ (3),ca_c (3)

	DO 200 n	= 1, 3
	  ca_n (n)	= n_targ (n) - ca_targ (n)
	  ca_c (n)	= c_targ (n) - ca_targ (n)
200	CONTINUE						! End n
	CALL cross_product (ca_c, ca_n, perp)
	DO 300 n	= 1, 3
	  cb_targ (n)	= cb_mult (1) * ca_c (n) +
     1			  cb_mult (2) * ca_n (n) +
     2			  cb_mult (3) * perp (n) +
     3			  ca_targ (n)
300	CONTINUE						! End n
	RETURN
	END

